@@ -167,13 +167,15 @@ static bool SendCacheDomainRequestToAllContentProcesses(
  */
 static bool MustBeGenericAccessible(nsIContent* aContent,
                                     DocAccessible* aDocument) {
-  if (aContent->IsInNativeAnonymousSubtree() || aContent->IsSVGElement()) {
+  if (aContent->IsInNativeAnonymousSubtree() || aContent->IsSVGElement() ||
+      aContent == aDocument->DocumentNode()->GetRootElement()) {
     // We should not force create accs for anonymous content.
     // This is an issue for inputs, which have an intermediate
     // container with relevant overflow styling between the input
     // and its internal input content.
     // We should also avoid this for SVG elements (ie. `<foreignobject>`s
     // which have default overflow:hidden styling).
+    // We should avoid this for the document root.
     return false;
   }
   nsIFrame* frame = aContent->GetPrimaryFrame();
@@ -562,27 +564,15 @@ void nsAccessibilityService::NotifyOfAnchorJumpTo(nsIContent* aTargetNode) {
   if (!document) {
     return;
   }
-  // If the document has focus when we get this notification, ensure that
-  // we fire a start scrolling event.
-  const Accessible* focusedAcc = FocusedAccessible();
-  if (focusedAcc &&
-      (focusedAcc == document || focusedAcc->IsNonInteractive())) {
-    LocalAccessible* targetAcc =
-        document->GetAccessibleOrContainer(aTargetNode);
-    // If targetAcc is the document, this isn't useful. It's possible we just
-    // haven't built the initial tree yet. Regardless, we don't want to fire an
-    // event for the document here.
-    if (targetAcc && !targetAcc->IsDoc()) {
-      nsEventShell::FireEvent(nsIAccessibleEvent::EVENT_SCROLLING_START,
-                              targetAcc);
-      document->SetAnchorJump(nullptr);
-    } else {
-      // We can't find the target accessible in the document yet. Set the
-      // anchor jump so that we can fire the scrolling start event later.
-      document->SetAnchorJump(aTargetNode);
-    }
-  } else {
-    document->SetAnchorJump(aTargetNode);
+  document->SetAnchorJump(aTargetNode);
+  // If there is a pending update, the target node might not have been added to
+  // the accessibility tree yet, so do not process the anchor jump here. It will
+  // be processed in NotificationController::WillRefresh after the tree is up to
+  // date. On the other hand, if there is no pending update, process the anchor
+  // jump here because the tree is already up to date and there might not be an
+  // update in the near future.
+  if (!document->Controller()->IsUpdatePending()) {
+    document->ProcessAnchorJump();
   }
 }
 
@@ -593,21 +583,34 @@ void nsAccessibilityService::FireAccessibleEvent(uint32_t aEvent,
 
 void nsAccessibilityService::NotifyOfPossibleBoundsChange(
     mozilla::PresShell* aPresShell, nsIContent* aContent) {
+  if (!aContent || (!IPCAccessibilityActive() && !aContent->IsText())) {
+    return;
+  }
+  DocAccessible* document = aPresShell->GetDocAccessible();
+  if (!document) {
+    return;
+  }
+  LocalAccessible* accessible = document->GetAccessible(aContent);
+  if (!accessible && aContent == document->GetContent()) {
+    // DocAccessible::GetAccessible() won't return the document if a root
+    // element like body is passed. In that case we need the doc accessible
+    // itself.
+    accessible = document;
+  }
+  if (!accessible) {
+    return;
+  }
   if (IPCAccessibilityActive()) {
-    DocAccessible* document = aPresShell->GetDocAccessible();
-    if (document) {
-      LocalAccessible* accessible = document->GetAccessible(aContent);
-      if (!accessible && aContent == document->GetContent()) {
-        // DocAccessible::GetAccessible() won't return the document if a root
-        // element like body is passed. In that case we need the doc accessible
-        // itself.
-        accessible = document;
-      }
-
-      if (accessible) {
-        document->QueueCacheUpdate(accessible, CacheDomain::Bounds);
-      }
-    }
+    document->QueueCacheUpdate(accessible, CacheDomain::Bounds);
+  }
+  if (accessible->IsTextLeaf() &&
+      accessible->AsTextLeaf()->Text().EqualsLiteral(" ")) {
+    // This space might be becoming invisible, even though it still has a frame.
+    // In this case, the frame will have 0 width. Unfortunately, we can't check
+    // the frame width here because layout isn't ready yet, so we need to defer
+    // this until the refresh driver tick.
+    MOZ_ASSERT(aContent->IsText());
+    document->UpdateText(aContent);
   }
 }
 
@@ -1171,7 +1174,7 @@ LocalAccessible* nsAccessibilityService::CreateAccessible(
   if (!aNode->IsContent()) return nullptr;
 
   nsIContent* content = aNode->AsContent();
-  if (aria::HasDefinedARIAHidden(content)) {
+  if (aria::IsValidARIAHidden(content)) {
     if (aIsSubtreeHidden) {
       *aIsSubtreeHidden = true;
     }
@@ -1293,6 +1296,7 @@ LocalAccessible* nsAccessibilityService::CreateAccessible(
     // Ignore not rendered text nodes and whitespace text nodes between table
     // cells.
     if (text.mString.IsEmpty() ||
+        nsCoreUtils::IsTrimmedWhitespaceBeforeHardLineBreak(frame) ||
         (aContext->IsTableRow() &&
          nsCoreUtils::IsWhitespaceString(text.mString))) {
       if (aIsSubtreeHidden) *aIsSubtreeHidden = true;
@@ -1450,11 +1454,11 @@ LocalAccessible* nsAccessibilityService::CreateAccessible(
       }
 
       // Fall back to text when encountering Content MathML.
-      if (!newAcc && !content->IsAnyOfMathMLElements(
-                         nsGkAtoms::annotation_, nsGkAtoms::annotation_xml_,
-                         nsGkAtoms::mpadded_, nsGkAtoms::mphantom_,
-                         nsGkAtoms::maligngroup_, nsGkAtoms::malignmark_,
-                         nsGkAtoms::mspace_, nsGkAtoms::semantics_)) {
+      if (!newAcc &&
+          !content->IsAnyOfMathMLElements(
+              nsGkAtoms::annotation, nsGkAtoms::annotation_xml,
+              nsGkAtoms::mpadded, nsGkAtoms::mphantom, nsGkAtoms::maligngroup,
+              nsGkAtoms::malignmark, nsGkAtoms::mspace, nsGkAtoms::semantics)) {
         newAcc = new HyperTextAccessible(content, document);
       }
     } else if (content->IsGeneratedContentContainerForMarker()) {
@@ -1520,6 +1524,10 @@ mozilla::Monitor& nsAccessibilityService::GetAndroidMonitor() {
 bool nsAccessibilityService::Init(uint64_t aCacheDomains) {
   AUTO_PROFILER_MARKER_UNTYPED("nsAccessibilityService::Init", A11Y, {});
   // DO NOT ADD CODE ABOVE HERE: THIS CODE IS MEASURING TIMINGS.
+  PerfStats::AutoMetricRecording<
+      PerfStats::Metric::A11Y_AccessibilityServiceInit>
+      autoRecording;
+  // DO NOT ADD CODE ABOVE THIS BLOCK: THIS CODE IS MEASURING TIMINGS.
 
   // Initialize accessible document manager.
   if (!DocManager::Init()) return false;
@@ -2078,5 +2086,34 @@ void PrefChanged(const char* aPref, void* aClosure) {
   }
 }
 
+uint32_t CacheDomainActivationBlocker::sEntryCount = 0;
+
+CacheDomainActivationBlocker::CacheDomainActivationBlocker() {
+  AssertIsOnMainThread();
+  if (sEntryCount++ != 0) {
+    // We're re-entering. This can happen if an earlier event (even in a
+    // different document) ends up calling an XUL method, since that can run
+    // script which can cause other events to fire. Only the outermost usage
+    // should change the flag.
+    return;
+  }
+  if (nsAccessibilityService* service = GetAccService()) {
+    MOZ_ASSERT(service->mShouldAllowNewCacheDomains);
+    service->mShouldAllowNewCacheDomains = false;
+  }
+}
+
+CacheDomainActivationBlocker::~CacheDomainActivationBlocker() {
+  AssertIsOnMainThread();
+  if (--sEntryCount != 0) {
+    // Only the outermost usage should change the flag.
+    return;
+  }
+  if (nsAccessibilityService* service = GetAccService()) {
+    MOZ_ASSERT(!service->mShouldAllowNewCacheDomains);
+    service->mShouldAllowNewCacheDomains = true;
+  }
+}
+
 }  // namespace a11y
 }  // namespace mozilla
