@@ -15,6 +15,7 @@
 #include "mozilla/a11y/Role.h"
 #include "States.h"
 #include "TextAttrs.h"
+#include "TextLeafRange.h"
 #include "TextRange.h"
 #include "TreeWalker.h"
 
@@ -171,7 +172,18 @@ uint32_t HyperTextAccessible::DOMPointToOffset(nsINode* aNode,
     }
   }
 
-  if (descendant && descendant->IsTextLeaf()) {
+  if (!descendant) {
+    // This DOM point can't be mapped to an offset in this HyperTextAccessible.
+    // Return the length as a fallback.
+    return CharacterCount();
+  }
+
+  if (aNode->IsText() && descendant->GetContent() != aNode) {
+    // `offset` is relative to aNode, but aNode doesn't have an Accessible, so
+    // we used the next Accessible. This means that `offset` is no longer valid.
+    NS_WARNING("No Accessible for DOM text node");
+    offset = 0;
+  } else if (descendant->IsTextLeaf()) {
     uint32_t length = nsAccUtils::TextLength(descendant);
     if (offset > length) {
       // This can happen if text in the accessibility tree is out of date with
@@ -316,7 +328,7 @@ void HyperTextAccessible::SetMathMLXMLRoles(AccAttributes* aAttributes) {
             }
             if (NS_MATHML_EMBELLISH_IS_SEPARATOR(embellishData.flags)) {
               aAttributes->SetAttribute(nsGkAtoms::xmlroles,
-                                        nsGkAtoms::separator_);
+                                        nsGkAtoms::separator);
             }
           }
         }
@@ -390,7 +402,7 @@ void HyperTextAccessible::SetMathMLXMLRoles(AccAttributes* aAttributes) {
             for (child = child->GetNextSibling(); child;
                  child = child->GetNextSibling()) {
               if (!child->IsMathMLElement()) continue;
-              if (child->IsMathMLElement(nsGkAtoms::mprescripts_)) {
+              if (child->IsMathMLElement(nsGkAtoms::mprescripts)) {
                 postscript = false;
                 subscript = true;
                 continue;
@@ -544,56 +556,6 @@ already_AddRefed<EditorBase> HyperTextAccessible::GetEditor() const {
  * =================== Caret & Selection ======================
  */
 
-nsresult HyperTextAccessible::SetSelectionRange(int32_t aStartPos,
-                                                int32_t aEndPos) {
-  // Before setting the selection range, we need to ensure that the editor
-  // is initialized. (See bug 804927.)
-  // Otherwise, it's possible that lazy editor initialization will override
-  // the selection we set here and leave the caret at the end of the text.
-  // By calling GetEditor here, we ensure that editor initialization is
-  // completed before we set the selection.
-  RefPtr<EditorBase> editorBase = GetEditor();
-
-  bool isFocusable = InteractiveState() & states::FOCUSABLE;
-
-  // If accessible is focusable then focus it before setting the selection to
-  // neglect control's selection changes on focus if any (for example, inputs
-  // that do select all on focus).
-  // some input controls
-  if (isFocusable) TakeFocus();
-
-  RefPtr<dom::Selection> domSel = DOMSelection();
-  NS_ENSURE_STATE(domSel);
-
-  // Set up the selection.
-  domSel->RemoveAllRanges(IgnoreErrors());
-  SetSelectionBoundsAt(0, aStartPos, aEndPos);
-
-  // Make sure it is visible
-  domSel->ScrollIntoView(nsISelectionController::SELECTION_FOCUS_REGION,
-                         ScrollAxis(), ScrollAxis(),
-                         ScrollFlags::ScrollOverflowHidden);
-
-  // When selection is done, move the focus to the selection if accessible is
-  // not focusable. That happens when selection is set within hypertext
-  // accessible.
-  if (isFocusable) return NS_OK;
-
-  nsFocusManager* DOMFocusManager = nsFocusManager::GetFocusManager();
-  if (DOMFocusManager) {
-    NS_ENSURE_TRUE(mDoc, NS_ERROR_FAILURE);
-    dom::Document* docNode = mDoc->DocumentNode();
-    NS_ENSURE_TRUE(docNode, NS_ERROR_FAILURE);
-    nsCOMPtr<nsPIDOMWindowOuter> window = docNode->GetWindow();
-    RefPtr<dom::Element> result;
-    DOMFocusManager->MoveFocus(
-        window, nullptr, nsIFocusManager::MOVEFOCUS_CARET,
-        nsIFocusManager::FLAG_BYMOVEFOCUS, getter_AddRefs(result));
-  }
-
-  return NS_OK;
-}
-
 int32_t HyperTextAccessible::CaretOffset() const {
   // Not focused focusable accessible except document accessible doesn't have
   // a caret.
@@ -647,18 +609,20 @@ int32_t HyperTextAccessible::CaretOffset() const {
   return DOMPointToOffset(focusNode, focusOffset);
 }
 
-LayoutDeviceIntRect HyperTextAccessible::GetCaretRect(nsIWidget** aWidget) {
-  *aWidget = nullptr;
-
+std::pair<LayoutDeviceIntRect, nsIWidget*> HyperTextAccessible::GetCaretRect() {
   RefPtr<nsCaret> caret = mDoc->PresShellPtr()->GetCaret();
-  NS_ENSURE_TRUE(caret, LayoutDeviceIntRect());
+  NS_ENSURE_TRUE(caret, {});
 
   bool isVisible = caret->IsVisible();
-  if (!isVisible) return LayoutDeviceIntRect();
+  if (!isVisible) {
+    return {};
+  }
 
   nsRect rect;
   nsIFrame* frame = caret->GetGeometry(&rect);
-  if (!frame || rect.IsEmpty()) return LayoutDeviceIntRect();
+  if (!frame || rect.IsEmpty()) {
+    return {};
+  }
 
   PresShell* presShell = mDoc->PresShellPtr();
   // Transform rect to be relative to the root frame.
@@ -690,121 +654,33 @@ LayoutDeviceIntRect HyperTextAccessible::GetCaretRect(nsIWidget** aWidget) {
     // the DOM node contaning the caret might be focused, but the Accessible
     // might not be; e.g. due to an autocomplete popup suggestion having a11y
     // focus.
-    return LayoutDeviceIntRect();
+    return {};
   }
-  LayoutDeviceIntRect charRect = CharBounds(
-      caretOffset, nsIAccessibleCoordinateType::COORDTYPE_SCREEN_RELATIVE);
+
+  // Vertically align the caret to the top of the line.
+  TextLeafPoint caretPoint = TextLeafPoint::GetCaret(this);
+  if (caretPoint.mIsEndOfLineInsertionPoint) {
+    caretPoint =
+        caretPoint.FindBoundary(nsIAccessibleText::BOUNDARY_CHAR, eDirPrevious);
+  }
+
+  LayoutDeviceIntRect charRect = caretPoint.CharBounds();
   if (!charRect.IsEmpty()) {
     caretRect.SetTopEdge(charRect.Y());
   }
 
-  *aWidget = frame->GetNearestWidget();
-  return caretRect;
-}
-
-void HyperTextAccessible::GetSelectionDOMRanges(SelectionType aSelectionType,
-                                                nsTArray<nsRange*>* aRanges) {
-  if (IsDoc() && !AsDoc()->HasLoadState(DocAccessible::eTreeConstructed)) {
-    // Rarely, a client query can be handled after a DocAccessible is created
-    // but before the initial tree is constructed, since DoInitialUpdate happens
-    // during a refresh tick. In that case, there might be a DOM selection, but
-    // we can't use it. We will crash if we try due to mContent being null, etc.
-    // This should only happen in the parent process because we should never
-    // try to push the cache in a content process before the initial tree is
-    // constructed.
-    MOZ_ASSERT(XRE_IsParentProcess(), "Query before DoInitialUpdate");
-    return;
-  }
-  // Ignore selection if it is not visible.
-  RefPtr<nsFrameSelection> frameSelection = FrameSelection();
-  if (!frameSelection || frameSelection->GetDisplaySelection() <=
-                             nsISelectionController::SELECTION_HIDDEN) {
-    return;
-  }
-
-  dom::Selection* domSel = frameSelection->GetSelection(aSelectionType);
-  if (!domSel) return;
-
-  nsINode* startNode = GetNode();
-
-  RefPtr<EditorBase> editorBase = GetEditor();
-  if (editorBase) {
-    startNode = editorBase->GetRoot();
-  }
-
-  if (!startNode) return;
-
-  uint32_t childCount = startNode->GetChildCount();
-  nsresult rv = domSel->GetDynamicRangesForIntervalArray(
-      startNode, 0, startNode, childCount, true, aRanges);
-  NS_ENSURE_SUCCESS_VOID(rv);
-
-  // Remove collapsed ranges
-  aRanges->RemoveElementsBy(
-      [](const auto& range) { return range->Collapsed(); });
-}
-
-int32_t HyperTextAccessible::SelectionCount() {
-  nsTArray<nsRange*> ranges;
-  GetSelectionDOMRanges(SelectionType::eNormal, &ranges);
-  return ranges.Length();
-}
-
-bool HyperTextAccessible::SelectionBoundsAt(int32_t aSelectionNum,
-                                            int32_t* aStartOffset,
-                                            int32_t* aEndOffset) {
-  *aStartOffset = *aEndOffset = 0;
-
-  nsTArray<nsRange*> ranges;
-  GetSelectionDOMRanges(SelectionType::eNormal, &ranges);
-
-  uint32_t rangeCount = ranges.Length();
-  if (aSelectionNum < 0 || aSelectionNum >= static_cast<int32_t>(rangeCount)) {
-    return false;
-  }
-
-  nsRange* range = ranges[aSelectionNum];
-
-  // Make sure start is before end, by swapping DOM points.  This occurs when
-  // the user selects backwards in the text.
-  const Maybe<int32_t> order =
-      nsContentUtils::ComparePoints(range->EndRef(), range->StartRef());
-
-  if (!order) {
-    MOZ_ASSERT_UNREACHABLE();
-    return false;
-  }
-
-  const RangeBoundary& precedingBoundary =
-      *order < 0 ? range->EndRef() : range->StartRef();
-  const RangeBoundary& followingBoundary =
-      *order < 0 ? range->StartRef() : range->EndRef();
-
-  if (!precedingBoundary.GetContainer()->IsInclusiveDescendantOf(mContent)) {
-    *aStartOffset = 0;
-  } else {
-    *aStartOffset = DOMPointToOffset(
-        precedingBoundary.GetContainer(),
-        AssertedCast<int32_t>(*precedingBoundary.Offset(
-            RangeBoundary::OffsetFilter::kValidOrInvalidOffsets)));
-  }
-
-  if (!followingBoundary.GetContainer()->IsInclusiveDescendantOf(mContent)) {
-    *aEndOffset = CharacterCount();
-  } else {
-    *aEndOffset = DOMPointToOffset(
-        followingBoundary.GetContainer(),
-        AssertedCast<int32_t>(*followingBoundary.Offset(
-            RangeBoundary::OffsetFilter::kValidOrInvalidOffsets)),
-        true);
-  }
-  return true;
+  return {caretRect, frame->GetNearestWidget()};
 }
 
 bool HyperTextAccessible::RemoveFromSelection(int32_t aSelectionNum) {
   RefPtr<dom::Selection> domSel = DOMSelection();
   if (!domSel) return false;
 
+  if (aSelectionNum == TextLeafRange::kRemoveAllExistingSelectedRanges) {
+    domSel->RemoveAllRanges(IgnoreErrors());
+    return true;
+  }
+
   if (aSelectionNum < 0 ||
       aSelectionNum >= static_cast<int32_t>(domSel->RangeCount())) {
     return false;
@@ -878,11 +754,24 @@ void HyperTextAccessible::ScrollSubstringToPoint(int32_t aStartOffset,
 
 void HyperTextAccessible::SelectionRanges(
     nsTArray<a11y::TextRange>* aRanges) const {
-  dom::Selection* sel = DOMSelection();
-  if (!sel) {
+  if (IsDoc() && !AsDoc()->HasLoadState(DocAccessible::eTreeConstructed)) {
+    // Rarely, a client query can be handled after a DocAccessible is created
+    // but before the initial tree is constructed, since DoInitialUpdate happens
+    // during a refresh tick. In that case, there might be a DOM selection, but
+    // we can't use it. We will crash if we try due to mContent being null, etc.
+    // This should only happen in the parent process because we should never
+    // try to push the cache in a content process before the initial tree is
+    // constructed.
+    MOZ_ASSERT(XRE_IsParentProcess(), "Query before DoInitialUpdate");
     return;
   }
-
+  // Ignore selection if it is not visible.
+  RefPtr<nsFrameSelection> frameSelection = FrameSelection();
+  if (!frameSelection || frameSelection->GetDisplaySelection() <=
+                             nsISelectionController::SELECTION_HIDDEN) {
+    return;
+  }
+  dom::Selection* sel = &frameSelection->NormalSelection();
   TextRange::TextRangesFromSelection(sel, aRanges);
 }
 
@@ -892,7 +781,8 @@ void HyperTextAccessible::ReplaceText(const nsAString& aText) {
     return;
   }
 
-  SetSelectionRange(0, CharacterCount());
+  SetSelectionBoundsAt(TextLeafRange::kRemoveAllExistingSelectedRanges, 0,
+                       CharacterCount());
 
   RefPtr<EditorBase> editorBase = GetEditor();
   if (!editorBase) {
@@ -907,7 +797,8 @@ void HyperTextAccessible::InsertText(const nsAString& aText,
                                      int32_t aPosition) {
   RefPtr<EditorBase> editorBase = GetEditor();
   if (editorBase) {
-    SetSelectionRange(aPosition, aPosition);
+    SetSelectionBoundsAt(TextLeafRange::kRemoveAllExistingSelectedRanges,
+                         aPosition, aPosition);
     DebugOnly<nsresult> rv = editorBase->InsertTextAsAction(aText);
     NS_WARNING_ASSERTION(NS_SUCCEEDED(rv), "Failed to insert the text");
   }
@@ -916,7 +807,8 @@ void HyperTextAccessible::InsertText(const nsAString& aText,
 void HyperTextAccessible::CopyText(int32_t aStartPos, int32_t aEndPos) {
   RefPtr<EditorBase> editorBase = GetEditor();
   if (editorBase) {
-    SetSelectionRange(aStartPos, aEndPos);
+    SetSelectionBoundsAt(TextLeafRange::kRemoveAllExistingSelectedRanges,
+                         aStartPos, aEndPos);
     editorBase->Copy();
   }
 }
@@ -924,7 +816,8 @@ void HyperTextAccessible::CopyText(int32_t aStartPos, int32_t aEndPos) {
 void HyperTextAccessible::CutText(int32_t aStartPos, int32_t aEndPos) {
   RefPtr<EditorBase> editorBase = GetEditor();
   if (editorBase) {
-    SetSelectionRange(aStartPos, aEndPos);
+    SetSelectionBoundsAt(TextLeafRange::kRemoveAllExistingSelectedRanges,
+                         aStartPos, aEndPos);
     editorBase->Cut();
   }
 }
@@ -934,7 +827,8 @@ void HyperTextAccessible::DeleteText(int32_t aStartPos, int32_t aEndPos) {
   if (!editorBase) {
     return;
   }
-  SetSelectionRange(aStartPos, aEndPos);
+  SetSelectionBoundsAt(TextLeafRange::kRemoveAllExistingSelectedRanges,
+                       aStartPos, aEndPos);
   DebugOnly<nsresult> rv =
       editorBase->DeleteSelectionAsAction(nsIEditor::eNone, nsIEditor::eStrip);
   NS_WARNING_ASSERTION(NS_SUCCEEDED(rv), "Failed to delete text");
@@ -948,7 +842,8 @@ void HyperTextAccessible::PasteText(int32_t aPosition) {
     // to replace it, just as would happen when pasting using the keyboard or
     // GUI.
     if (aPosition != nsIAccessibleText::TEXT_OFFSET_CARET) {
-      SetSelectionRange(aPosition, aPosition);
+      SetSelectionBoundsAt(TextLeafRange::kRemoveAllExistingSelectedRanges,
+                           aPosition, aPosition);
     }
     editorBase->PasteAsAction(nsIClipboard::kGlobalClipboard,
                               EditorBase::DispatchPasteEvent::Yes);
@@ -1013,7 +908,7 @@ Relation HyperTextAccessible::RelationByType(RelationType aType) const {
         if (parent) {
           nsIContent* parentContent = parent->GetContent();
           if (parentContent &&
-              parentContent->IsMathMLElement(nsGkAtoms::mroot_)) {
+              parentContent->IsMathMLElement(nsGkAtoms::mroot)) {
             // Add a relation pointing to the parent <mroot>.
             rel.AppendTarget(parent);
           }
@@ -1021,7 +916,7 @@ Relation HyperTextAccessible::RelationByType(RelationType aType) const {
       }
       break;
     case RelationType::NODE_PARENT_OF:
-      if (HasOwnContent() && mContent->IsMathMLElement(nsGkAtoms::mroot_)) {
+      if (HasOwnContent() && mContent->IsMathMLElement(nsGkAtoms::mroot)) {
         LocalAccessible* base = LocalChildAt(0);
         LocalAccessible* index = LocalChildAt(1);
         if (base && index) {
