@@ -54,7 +54,7 @@ void HTMLFormAccessible::DOMAttributeChanged(int32_t aNameSpaceID,
     for (uint32_t i = 0; i < length; i++) {
       if (LocalAccessible* acc = mDoc->GetAccessible(controls->Item(i))) {
         if (acc->IsTextField() && !acc->IsPassword()) {
-          if (!acc->Elm()->HasAttr(nsGkAtoms::list_) &&
+          if (!acc->Elm()->HasAttr(nsGkAtoms::list) &&
               !acc->Elm()->AttrValueIs(kNameSpaceID_None,
                                        nsGkAtoms::autocomplete, nsGkAtoms::OFF,
                                        eIgnoreCase)) {
@@ -206,9 +206,8 @@ uint64_t HTMLButtonAccessible::NativeState() const {
     if (!popoverAcc || !popoverAcc->IsAncestorOf(this)) {
       if (popover->IsPopoverOpen()) {
         state |= states::EXPANDED;
-      } else {
-        state |= states::COLLAPSED;
       }
+      state |= states::EXPANDABLE;
     }
   }
 
@@ -289,9 +288,16 @@ role HTMLTextFieldAccessible::NativeRole() const {
   if (mType == eHTMLTextPasswordFieldType) {
     return roles::PASSWORD_TEXT;
   }
-  if (mContent->AsElement()->HasAttr(nsGkAtoms::list_)) {
+  dom::Element* el = mContent->AsElement();
+  if (el->HasAttr(nsGkAtoms::list)) {
     return roles::EDITCOMBOBOX;
   }
+  if (const nsAttrValue* attr = el->GetParsedAttr(nsGkAtoms::type)) {
+    RefPtr<nsAtom> inputType = attr->GetAsAtom();
+    if (inputType == nsGkAtoms::search) {
+      return roles::SEARCHBOX;
+    }
+  }
   return roles::ENTRY;
 }
 
@@ -360,7 +366,7 @@ void HTMLTextFieldAccessible::Value(nsString& aValue) const {
 }
 
 bool HTMLTextFieldAccessible::AttributeChangesState(nsAtom* aAttribute) {
-  if (aAttribute == nsGkAtoms::readonly || aAttribute == nsGkAtoms::list_ ||
+  if (aAttribute == nsGkAtoms::readonly || aAttribute == nsGkAtoms::list ||
       aAttribute == nsGkAtoms::autocomplete) {
     return true;
   }
@@ -381,7 +387,8 @@ uint64_t HTMLTextFieldAccessible::NativeState() const {
   state |= states::EDITABLE;
 
   // can be focusable, focused, protected. readonly, unavailable, selected
-  if (mContent->AsElement()->AttrValueIs(kNameSpaceID_None, nsGkAtoms::type,
+  if (mContent->IsHTMLElement(nsGkAtoms::input) &&
+      mContent->AsElement()->AttrValueIs(kNameSpaceID_None, nsGkAtoms::type,
                                          nsGkAtoms::password, eIgnoreCase)) {
     state |= states::PROTECTED;
   }
@@ -401,7 +408,7 @@ uint64_t HTMLTextFieldAccessible::NativeState() const {
   }
 
   // Expose autocomplete state if it has associated autocomplete list.
-  if (mContent->AsElement()->HasAttr(nsGkAtoms::list_)) {
+  if (mContent->AsElement()->HasAttr(nsGkAtoms::list)) {
     return state | states::SUPPORTS_AUTOCOMPLETION | states::HASPOPUP;
   }
 
@@ -680,7 +687,7 @@ ENameValueFlag HTMLGroupboxAccessible::NativeName(nsString& aName) const {
   }
 
   aName.CompressWhitespace();
-  return eNameOK;
+  return aName.IsEmpty() ? eNameOK : eNameFromRelations;
 }
 
 Relation HTMLGroupboxAccessible::RelationByType(RelationType aType) const {
@@ -729,7 +736,7 @@ ENameValueFlag HTMLFigureAccessible::NativeName(nsString& aName) const {
   }
 
   aName.CompressWhitespace();
-  return eNameOK;
+  return aName.IsEmpty() ? eNameOK : eNameFromRelations;
 }
 
 Relation HTMLFigureAccessible::RelationByType(RelationType aType) const {
