@@ -1209,20 +1209,36 @@ Relation RemoteAccessible::RelationByType(RelationType aType) const {
   // the cached relations need to take precedence. For example, a <figure> with
   // both aria-labelledby and a <figcaption> must return two LABELLED_BY
   // targets: the aria-labelledby and then the <figcaption>.
-  if (aType == RelationType::LABELLED_BY && TagName() == nsGkAtoms::figure) {
+  auto AddChildWithTag = [this, &rel](nsAtom* aTarget) {
     uint32_t count = ChildCount();
     for (uint32_t c = 0; c < count; ++c) {
       RemoteAccessible* child = RemoteChildAt(c);
       MOZ_ASSERT(child);
-      if (child->TagName() == nsGkAtoms::figcaption) {
+      if (child->TagName() == aTarget) {
         rel.AppendTarget(child);
       }
     }
-  } else if (aType == RelationType::LABEL_FOR &&
-             TagName() == nsGkAtoms::figcaption) {
-    if (RemoteAccessible* parent = RemoteParent()) {
-      if (parent->TagName() == nsGkAtoms::figure) {
-        rel.AppendTarget(parent);
+  };
+  if (aType == RelationType::LABELLED_BY) {
+    auto tag = TagName();
+    if (tag == nsGkAtoms::figure) {
+      AddChildWithTag(nsGkAtoms::figcaption);
+    } else if (tag == nsGkAtoms::fieldset) {
+      AddChildWithTag(nsGkAtoms::legend);
+    }
+  } else if (aType == RelationType::LABEL_FOR) {
+    auto tag = TagName();
+    if (tag == nsGkAtoms::figcaption) {
+      if (RemoteAccessible* parent = RemoteParent()) {
+        if (parent->TagName() == nsGkAtoms::figure) {
+          rel.AppendTarget(parent);
+        }
+      }
+    } else if (tag == nsGkAtoms::legend) {
+      if (RemoteAccessible* parent = RemoteParent()) {
+        if (parent->TagName() == nsGkAtoms::fieldset) {
+          rel.AppendTarget(parent);
+        }
       }
     }
   }
@@ -1516,6 +1532,16 @@ RefPtr<const AccAttributes> RemoteAccessible::GetCachedTextAttributes() {
   return nullptr;
 }
 
+std::pair<LayoutDeviceIntRect, nsIWidget*> RemoteAccessible::GetCaretRect() {
+  nsIWidget* widget = nullptr;
+  LocalAccessible* outerDoc = OuterDocOfRemoteBrowser();
+  if (outerDoc) {
+    widget = nsContentUtils::WidgetForContent(outerDoc->GetContent());
+  }
+
+  return {mDoc->GetCachedCaretRect(), widget};
+}
+
 already_AddRefed<AccAttributes> RemoteAccessible::DefaultTextAttributes() {
   if (RequestDomainsIfInactive(CacheDomain::Text)) {
     return nullptr;
@@ -1528,10 +1554,10 @@ already_AddRefed<AccAttributes> RemoteAccessible::DefaultTextAttributes() {
   return result.forget();
 }
 
-RefPtr<const AccAttributes> RemoteAccessible::GetCachedARIAAttributes() const {
+const AccAttributes* RemoteAccessible::GetCachedARIAAttributes() const {
   ASSERT_DOMAINS_ACTIVE(CacheDomain::ARIA);
   if (mCachedFields) {
-    auto attrs = mCachedFields->GetAttributeRefPtr<AccAttributes>(
+    auto attrs = mCachedFields->GetAttributeWeakPtr<AccAttributes>(
         CacheKey::ARIAAttributes);
     VERIFY_CACHE(CacheDomain::ARIA);
     return attrs;
@@ -1564,13 +1590,6 @@ uint64_t RemoteAccessible::State() {
             mCachedFields->GetAttribute<uint64_t>(CacheKey::State)) {
       VERIFY_CACHE(CacheDomain::State);
       state = *rawState;
-      // Handle states that are derived from other states.
-      if (!(state & states::UNAVAILABLE)) {
-        state |= states::ENABLED | states::SENSITIVE;
-      }
-      if (state & states::EXPANDABLE && !(state & states::EXPANDED)) {
-        state |= states::COLLAPSED;
-      }
     }
 
     ApplyImplicitState(state);
@@ -1829,7 +1848,7 @@ void RemoteAccessible::LiveRegionAttributes(nsAString* aLive,
   if (!mCachedFields) {
     return;
   }
-  RefPtr<const AccAttributes> attrs = GetCachedARIAAttributes();
+  auto attrs = GetCachedARIAAttributes();
   if (!attrs) {
     return;
   }
@@ -2309,15 +2328,11 @@ nsTArray<int32_t>& RemoteAccessible::GetCachedHyperTextOffsets() {
       CacheKey::HyperTextOffsets);
 }
 
-void RemoteAccessible::SetCaretOffset(int32_t aOffset) {
-  Unused << mDoc->SendSetCaretOffset(mID, aOffset);
-}
-
 Maybe<int32_t> RemoteAccessible::GetIntARIAAttr(nsAtom* aAttrName) const {
   if (RequestDomainsIfInactive(CacheDomain::ARIA)) {
     return Nothing();
   }
-  if (RefPtr<const AccAttributes> attrs = GetCachedARIAAttributes()) {
+  if (auto attrs = GetCachedARIAAttributes()) {
     if (auto val = attrs->GetAttribute<int32_t>(aAttrName)) {
       return val;
     }
@@ -2325,6 +2340,18 @@ Maybe<int32_t> RemoteAccessible::GetIntARIAAttr(nsAtom* aAttrName) const {
   return Nothing();
 }
 
+bool RemoteAccessible::GetStringARIAAttr(nsAtom* aAttrName,
+                                         nsAString& aAttrValue) const {
+  if (RequestDomainsIfInactive(CacheDomain::ARIA)) {
+    return false;
+  }
+  if (auto attrs = GetCachedARIAAttributes()) {
+    return attrs->GetAttribute(aAttrName, aAttrValue);
+  }
+
+  return false;
+}
+
 void RemoteAccessible::Language(nsAString& aLocale) {
   if (RequestDomainsIfInactive(CacheDomain::Text)) {
     return;
