@@ -26,6 +26,7 @@
 #include "mozilla/dom/BrowserParent.h"
 #include "OuterDocAccessible.h"
 #include "nsChildView.h"
+#include "TextLeafRange.h"
 #include "xpcAccessibleMacInterface.h"
 
 #include "nsRect.h"
@@ -41,8 +42,6 @@ using namespace mozilla::a11y;
 #pragma mark -
 
 @interface mozAccessible ()
-- (BOOL)providesLabelNotTitle;
-
 - (void)maybePostA11yUtilNotification;
 @end
 
@@ -128,16 +127,6 @@ using namespace mozilla::a11y;
   }
 }
 
-- (BOOL)providesLabelNotTitle {
-  // These accessible types are the exception to the rule of label vs. title:
-  // They may be named explicitly, but they still provide a label not a title.
-  return mRole == roles::GROUPING || mRole == roles::RADIO_GROUP ||
-         mRole == roles::FIGURE || mRole == roles::GRAPHIC ||
-         mRole == roles::DOCUMENT || mRole == roles::OUTLINE ||
-         mRole == roles::ARTICLE || mRole == roles::ENTRY ||
-         mRole == roles::SPINBUTTON;
-}
-
 - (mozilla::a11y::Accessible*)geckoAccessible {
   return mGeckoAccessible;
 }
@@ -387,7 +376,7 @@ using namespace mozilla::a11y;
   // and are instructed by the ARIA map to use the native host role.
   roleAtom = [self ARIARole];
 
-  if (roleAtom == nsGkAtoms::log_) {
+  if (roleAtom == nsGkAtoms::log) {
     return @"AXApplicationLog";
   }
 
@@ -411,7 +400,7 @@ struct RoleDescrMap {
   const nsString description;
 };
 
-static const RoleDescrMap sRoleDescrMap[] = {
+MOZ_RUNINIT static const RoleDescrMap sRoleDescrMap[] = {
     {@"AXApplicationAlert", u"alert"_ns},
     {@"AXApplicationAlertDialog", u"alertDialog"_ns},
     {@"AXApplicationDialog", u"dialog"_ns},
@@ -468,7 +457,7 @@ struct RoleDescrComparator {
 
   if (subrole) {
     size_t idx = 0;
-    if (BinarySearchIf(sRoleDescrMap, 0, ArrayLength(sRoleDescrMap),
+    if (BinarySearchIf(sRoleDescrMap, 0, std::size(sRoleDescrMap),
                        RoleDescrComparator(subrole), &idx)) {
       return utils::LocalizedString(sRoleDescrMap[idx].description);
     }
@@ -477,6 +466,33 @@ struct RoleDescrComparator {
   return NSAccessibilityRoleDescription([self moxRole], subrole);
 }
 
+static bool ProvidesTitle(const Accessible* aAccessible, nsString& aName) {
+  ENameValueFlag flag = aAccessible->Name(aName);
+
+  switch (aAccessible->Role()) {
+    case roles::PAGETAB:
+    case roles::COMBOBOX_OPTION:
+    case roles::PARENT_MENUITEM:
+    case roles::MENUITEM:
+      // These roles always supply a title.
+      return true;
+    case roles::GROUPING:
+    case roles::RADIO_GROUP:
+    case roles::DOCUMENT:
+    case roles::OUTLINE:
+    case roles::ARTICLE:
+    case roles::FIGURE:
+      // These roles never supply a title.
+      return false;
+    default:
+      break;
+  }
+
+  // If the name was calculated from visible text (eg. label or subtree), we
+  // supply a title.
+  return flag != eNameOK;
+}
+
 - (NSString*)moxLabel {
   if ([self isExpired]) {
     return nil;
@@ -484,42 +500,27 @@ struct RoleDescrComparator {
 
   nsAutoString name;
 
-  /* If our accessible is:
-   * 1. Named by invisible text, or
-   * 2. Has more than one labeling relation, or
-   * 3. Is a special role defined in providesLabelNotTitle
-   *   ... return its name as a label (AXDescription).
-   */
-  ENameValueFlag flag = mGeckoAccessible->Name(name);
-  if (flag == eNameFromSubtree) {
+  if (ProvidesTitle(mGeckoAccessible, name)) {
+    // If it provides title, it is not a description.
     return nil;
   }
 
-  if (![self providesLabelNotTitle]) {
-    NSArray* relations = [self getRelationsByType:RelationType::LABELLED_BY];
-    if ([relations count] == 1) {
-      return nil;
-    }
-  }
-
   return nsCocoaUtils::ToNSString(name);
 }
 
 - (NSString*)moxTitle {
   NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
 
-  // In some special cases we provide the name in the label (AXDescription).
-  if ([self providesLabelNotTitle]) {
-    return nil;
-  }
-
-  nsAutoString title;
-  mGeckoAccessible->Name(title);
-  if (nsCoreUtils::IsWhitespaceString(title)) {
+  nsAutoString name;
+  if (!ProvidesTitle(mGeckoAccessible, name)) {
     return @"";
   }
 
-  return nsCocoaUtils::ToNSString(title);
+  if (nsCoreUtils::IsWhitespaceString(name)) {
+    return @"";
+  }
+
+  return nsCocoaUtils::ToNSString(name);
 
   NS_OBJC_END_TRY_BLOCK_RETURN(nil);
 }
@@ -586,6 +587,26 @@ struct RoleDescrComparator {
   return @YES;
 }
 
+- (NSString*)moxInvalid {
+  // For controls that support text input, we will expose
+  // the string value of `aria-invalid` when it exists.
+  // See mozTextAccessible::moxInvalid for that work.
+  // Unfortunately, NSBools do not autoconvert to usable
+  // NSStrings, so we expose "true" and "false" manually.
+  return ([self stateWithMask:states::INVALID] != 0) ? @"true" : @"false";
+}
+
+- (NSArray*)moxErrorMessageElements {
+  if (![[self moxInvalid] isEqualToString:@"false"]) {
+    NSArray* relations = [self getRelationsByType:RelationType::ERRORMSG];
+    if ([relations count] > 0) {
+      return relations;
+    }
+  }
+
+  return nil;
+}
+
 - (NSNumber*)moxFocused {
   return @([self stateWithMask:states::FOCUSED] != 0);
 }
@@ -664,12 +685,39 @@ struct RoleDescrComparator {
 - (id)moxTitleUIElement {
   MOZ_ASSERT(mGeckoAccessible);
 
-  NSArray* relations = [self getRelationsByType:RelationType::LABELLED_BY];
-  if ([relations count] == 1) {
-    return [relations firstObject];
+  nsAutoString unused;
+  if (mGeckoAccessible->Name(unused) != eNameFromRelations) {
+    return nil;
   }
 
-  return nil;
+  Relation rel = mGeckoAccessible->RelationByType(RelationType::LABELLED_BY);
+  Accessible* label = rel.Next();
+  if (!label || rel.Next()) {
+    // Zero or more than one relation.
+    return nil;
+  }
+
+  if (label->IsAncestorOf(mGeckoAccessible)) {
+    // Don't support labelling for a relation that references an ancestor.
+    // VO walks the label's subtree and tries to construct the name for the
+    // control. It does not strip whitespace from the text leaf children, and
+    // since the calculated name of the control is stripped, it sees it as two
+    // different names and inclues both.
+    return nil;
+  }
+
+  if (RefPtr<nsAtom>(label->DisplayStyle()) == nsGkAtoms::block) {
+    TextLeafPoint endPoint =
+        TextLeafPoint(label, nsIAccessibleText::TEXT_OFFSET_END_OF_TEXT)
+            .FindBoundary(nsIAccessibleText::BOUNDARY_CHAR, eDirPrevious);
+    if (endPoint.IsSpace()) {
+      // A label that is a block element with trailing space causes VO be
+      // unhappy.
+      return nil;
+    }
+  }
+
+  return GetNativeFromGeckoAccessible(label);
 }
 
 - (NSString*)moxDOMIdentifier {
@@ -769,6 +817,19 @@ struct RoleDescrComparator {
   return nsCocoaUtils::ToNSString(lang);
 }
 
+- (NSString*)moxKeyShortcutsValue {
+  MOZ_ASSERT(mGeckoAccessible);
+
+  nsAutoString shortcut;
+
+  if (!mGeckoAccessible->GetStringARIAAttr(nsGkAtoms::aria_keyshortcuts,
+                                           shortcut)) {
+    return nil;
+  }
+
+  return nsCocoaUtils::ToNSString(shortcut);
+}
+
 #ifndef RELEASE_OR_BETA
 - (NSString*)moxMozDebugDescription {
   NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
@@ -887,22 +948,19 @@ struct RoleDescrComparator {
                                  : nullptr;
   if (maybeRoot && maybeRoot->IsRoot() &&
       [[self moxDOMIdentifier] isEqualToString:@"a11y-announcement"]) {
-    // Our actual announcement should be stored as a child of the alert,
-    // so we verify a child exists, and then query that child below.
-    NSArray* children = [self moxChildren];
-    MOZ_ASSERT([children count] == 1 && children[0],
-               "A11yUtil event received, but no announcement found?");
-
-    mozAccessible* announcement = children[0];
-    NSString* key;
-    if ([announcement providesLabelNotTitle]) {
-      key = [announcement moxLabel];
+    nsAutoString name;
+    // Our actual announcement should be stored as a child of the alert.
+    if (Accessible* announcement = mGeckoAccessible->FirstChild()) {
+      announcement->Name(name);
     } else {
-      key = [announcement moxTitle];
+      MOZ_ASSERT_UNREACHABLE(
+          "A11yUtil event received, but no announcement found?");
     }
 
     NSDictionary* info = @{
-      NSAccessibilityAnnouncementKey : key ? key : @(""),
+      NSAccessibilityAnnouncementKey : name.IsEmpty()
+          ? @("")
+          : nsCocoaUtils::ToNSString(name),
       // High priority means VO will stop what it is currently speaking
       // to speak our announcement.
       NSAccessibilityPriorityKey : @(NSAccessibilityPriorityHigh)
@@ -940,6 +998,7 @@ struct RoleDescrComparator {
                                inserted:(BOOL)isInserted
                             inContainer:(Accessible*)container
                                      at:(int32_t)start {
+  [self maybePostValidationErrorChanged];
 }
 
 - (void)handleAccessibleEvent:(uint32_t)eventType {
@@ -992,15 +1051,43 @@ struct RoleDescrComparator {
     case nsIAccessibleEvent::EVENT_LIVE_REGION_REMOVED:
       mIsLiveRegion = false;
       break;
-    case nsIAccessibleEvent::EVENT_NAME_CHANGE:
-      if (![self providesLabelNotTitle]) {
+    case nsIAccessibleEvent::EVENT_NAME_CHANGE: {
+      nsAutoString nameNotUsed;
+      if (ProvidesTitle(mGeckoAccessible, nameNotUsed)) {
         [self moxPostNotification:NSAccessibilityTitleChangedNotification];
       }
       break;
+    }
     case nsIAccessibleEvent::EVENT_LIVE_REGION_CHANGED:
       MOZ_ASSERT(mIsLiveRegion);
       [self moxPostNotification:@"AXLiveRegionChanged"];
       break;
+    case nsIAccessibleEvent::EVENT_ERRORMESSAGE_CHANGED: {
+      // aria-errormessage was changed. If aria-invalid != "true", it means that
+      // VoiceOver should (a) expose a new message or (b) remove an
+      // old message
+      if (![[self moxInvalid] isEqualToString:@"false"]) {
+        [self moxPostNotification:@"AXValidationErrorChanged"];
+      }
+
+      break;
+    }
+  }
+}
+
+- (void)maybePostValidationErrorChanged {
+  NSArray* relations =
+      [self getRelationsByType:(mozilla::a11y::RelationType::ERRORMSG_FOR)];
+  if ([relations count] > 0) {
+    // only fire AXValidationErrorChanged if related node is not
+    // `aria-invalid="false"`
+    for (mozAccessible* related : relations) {
+      NSString* invalidStr = [related moxInvalid];
+      if (![invalidStr isEqualToString:@"false"]) {
+        [self moxPostNotification:@"AXValidationErrorChanged"];
+        break;
+      }
+    }
   }
 }
 
