@@ -264,7 +264,6 @@ var BrowserCommands = {
           inBackground: loadInBackground,
           triggeringPrincipal:
             Services.scriptSecurityManager.getSystemPrincipal(),
-          csp: null,
         });
         if (!loadInBackground) {
           if (isBlankPageURL(homePage)) {
@@ -396,7 +395,9 @@ var BrowserCommands = {
 
     // In a multi-select context, close all selected tabs
     if (gBrowser.multiSelectedTabsCount) {
-      gBrowser.removeMultiSelectedTabs();
+      gBrowser.removeMultiSelectedTabs(
+        gBrowser.TabMetrics.userTriggeredContext()
+      );
       return;
     }
 
@@ -414,7 +415,10 @@ var BrowserCommands = {
     }
 
     // If the current tab is the last one, this will close the window.
-    gBrowser.removeCurrentTab({ animate: true });
+    gBrowser.removeCurrentTab({
+      animate: true,
+      ...gBrowser.TabMetrics.userTriggeredContext(),
+    });
   },
 
   tryToCloseWindow(event) {
@@ -596,4 +600,8 @@ var BrowserCommands = {
       Ci.nsIWebNavigation.LOAD_FLAGS_CHARSET_CHANGE
     );
   },
+
+  processCloseRequest() {
+    gBrowser.selectedBrowser.processCloseRequest();
+  },
 };
