@@ -57,9 +57,6 @@ document.addEventListener(
         case "context-savelink":
           gContextMenu.saveLink();
           break;
-        case "context-savelinktopocket":
-          Pocket.savePage(gContextMenu.browser, gContextMenu.linkURL);
-          break;
         case "context-copyemail":
           gContextMenu.copyEmail();
           break;
@@ -69,6 +66,9 @@ document.addEventListener(
         case "context-copylink":
           gContextMenu.copyLink();
           break;
+        case "context-previewlink":
+          gContextMenu.previewLink();
+          break;
         case "context-stripOnShareLink":
           gContextMenu.copyStrippedLink();
           break;
@@ -163,13 +163,6 @@ document.addEventListener(
         case "context-savepage":
           gContextMenu.savePageAs();
           break;
-        case "context-pocket":
-          Pocket.savePage(
-            gContextMenu.browser,
-            gContextMenu.browser.currentURI.spec,
-            gContextMenu.browser.contentTitle
-          );
-          break;
         case "fill-login-generated-password":
           gContextMenu.useGeneratedPassword();
           break;
@@ -213,28 +206,38 @@ document.addEventListener(
           gContextMenu.takeScreenshot();
           break;
         case "context-keywordfield":
-          AddKeywordForSearchField();
+          if (!gContextMenu) {
+            throw new Error("Context menu doesn't seem to be open.");
+          }
+          gContextMenu.addKeywordForSearchField();
+          break;
+        case "context-add-engine":
+          if (!gContextMenu) {
+            throw new Error("Context menu doesn't seem to be open.");
+          }
+          gContextMenu.addSearchFieldAsEngine().catch(console.error);
           break;
         case "context-searchselect": {
-          let { searchTerms, usePrivate, principal, csp } = event.target;
+          let { searchTerms, usePrivate, principal, policyContainer } =
+            event.target;
           SearchUIUtils.loadSearchFromContext(
             window,
             searchTerms,
             usePrivate,
             principal,
-            csp,
+            policyContainer,
             event
           );
           break;
         }
         case "context-searchselect-private": {
-          let { searchTerms, principal, csp } = event.target;
+          let { searchTerms, principal, policyContainer } = event.target;
           SearchUIUtils.loadSearchFromContext(
             window,
             searchTerms,
             true,
             principal,
-            csp,
+            policyContainer,
             event
           );
           break;
@@ -263,9 +266,6 @@ document.addEventListener(
         case "context-printframe":
           gContextMenu.printFrame();
           break;
-        case "context-take-frame-screenshot":
-          gContextMenu.takeScreenshot();
-          break;
         case "context-viewframesource":
           gContextMenu.viewFrameSource();
           break;
@@ -303,7 +303,7 @@ document.addEventListener(
         case "context-copy-clean-link-to-highlight":
           gContextMenu.copyLinkToHighlight(/* stripSiteTracking */ true);
           break;
-        case "context-remove-all-highlights":
+        case "context-remove-highlight":
           gContextMenu.removeAllTextFragments();
           break;
       }
