@@ -18,7 +18,7 @@ let gCUITestUtils = new CustomizableUITestUtils(window);
 
 const PERMISSIONS_PAGE =
   "https://example.com/browser/browser/base/content/test/permissions/permissions.html";
-const afterUrlBarButton = "save-to-pocket-button";
+const afterUrlBarButton = "fxa-toolbar-menu-button";
 const sidebarRevampEnabled = Services.prefs.getBoolPref(
   "sidebar.revamp",
   false
@@ -120,10 +120,13 @@ const BOOKMARKS_COUNT = 100;
 add_setup(async function () {
   await SpecialPowers.pushPrefEnv({
     set: [
+      ["test.wait300msAfterTabSwitch", true],
       // TODO: Reenable in https://bugzilla.mozilla.org/show_bug.cgi?id=1923388
       ["browser.urlbar.scotchBonnet.enableOverride", false],
       ["browser.toolbars.keyboard_navigation", true],
       ["accessibility.tabfocus", 7],
+      // Bug 1968055 - Temporarily enabled pocket pref while we remove the pref entirely
+      ["extensions.pocket.enabled", true],
     ],
   });
   resetToolbarWithoutDevEditionButtons();
@@ -344,14 +347,13 @@ add_task(async function testArrowsToolbarbuttons() {
   AddOldMenuSideButtons();
   await BrowserTestUtils.withNewTab("about:blank", async function () {
     startFromUrlBar();
-    await expectFocusAfterKey("Tab", afterUrlBarButton);
+    await expectFocusAfterKey("Tab", "library-button");
     EventUtils.synthesizeKey("KEY_ArrowLeft");
     is(
       document.activeElement.id,
-      afterUrlBarButton,
+      "library-button",
       "ArrowLeft at end of button group does nothing"
     );
-    await expectFocusAfterKey("ArrowRight", "library-button");
     if (!sidebarRevampEnabled) {
       await expectFocusAfterKey("ArrowRight", "sidebar-button");
     }
@@ -450,8 +452,7 @@ add_task(async function testArrowsOverflowButton() {
       CustomizableUI.AREA_FIXED_OVERFLOW_PANEL
     );
     startFromUrlBar();
-    await expectFocusAfterKey("Tab", afterUrlBarButton);
-    await expectFocusAfterKey("ArrowRight", "library-button");
+    await expectFocusAfterKey("Tab", "library-button");
     if (!sidebarRevampEnabled) {
       await expectFocusAfterKey("ArrowRight", "sidebar-button");
     }
@@ -498,24 +499,15 @@ add_task(async function testArrowsInPanelMultiView() {
 // Test that right/left arrows move in the expected direction for RTL locales.
 add_task(async function testArrowsRtl() {
   AddOldMenuSideButtons();
-  await SpecialPowers.pushPrefEnv({ set: [["intl.l10n.pseudo", "bidi"]] });
-  // window.RTL_UI doesn't update in existing windows when this pref is changed,
-  // so we need to test in a new window.
-  let win = await BrowserTestUtils.openNewBrowserWindow();
-  startFromUrlBar(win);
-  await expectFocusAfterKey("Tab", afterUrlBarButton, false, win);
-  EventUtils.synthesizeKey("KEY_ArrowRight", {}, win);
-  is(
-    win.document.activeElement.id,
-    afterUrlBarButton,
-    "ArrowRight at end of button group does nothing"
-  );
-  await expectFocusAfterKey("ArrowLeft", "library-button", false, win);
+  await BrowserTestUtils.enableRtlLocale();
+  startFromUrlBar(window);
+  await expectFocusAfterKey("Tab", "library-button");
+  EventUtils.synthesizeKey("KEY_ArrowRight", {});
   if (!sidebarRevampEnabled) {
-    await expectFocusAfterKey("ArrowLeft", "sidebar-button", false, win);
+    await expectFocusAfterKey("ArrowLeft", "sidebar-button");
   }
-  await BrowserTestUtils.closeWindow(win);
-  await SpecialPowers.popPrefEnv();
+  await expectFocusAfterKey("ArrowLeft", "unified-extensions-button");
+  await BrowserTestUtils.disableRtlLocale();
   RemoveOldMenuSideButtons();
 });
 
@@ -559,8 +551,7 @@ add_task(async function testPanelCloseRestoresFocus() {
     // We can't use forceFocus because that removes focusability immediately.
     // Instead, we must let ToolbarKeyboardNavigator handle this properly.
     startFromUrlBar();
-    await expectFocusAfterKey("Tab", afterUrlBarButton);
-    await expectFocusAfterKey("ArrowRight", "library-button");
+    await expectFocusAfterKey("Tab", "library-button");
     let view = document.getElementById("appMenu-libraryView");
     let shown = BrowserTestUtils.waitForEvent(view, "ViewShown");
     EventUtils.synthesizeKey(" ");
@@ -622,16 +613,7 @@ add_task(async function testCharacterNavigation() {
     // Escape should reset the search.
     EventUtils.synthesizeKey("KEY_Escape");
     if (!sidebarRevampEnabled) {
-      // Now that the search is reset, pressing s should focus Save to Pocket.
-      await expectFocusAfterKey("s", "save-to-pocket-button");
-      // Pressing i makes the search "si", so it should focus Sidebars.
-      await expectFocusAfterKey("i", "sidebar-button");
-    }
-    // Reset the search.
-    EventUtils.synthesizeKey("KEY_Escape");
-    if (!sidebarRevampEnabled) {
-      await expectFocusAfterKey("s", "save-to-pocket-button");
-      // Pressing s again should find the next button starting with s: Sidebars.
+      // Pressing s should find the button starting with s: Sidebars.
       await expectFocusAfterKey("s", "sidebar-button");
     }
   });
@@ -668,9 +650,7 @@ add_task(async function testTabStopsAfterSearchBarAdded() {
   await withNewBlankTab(async function () {
     startFromUrlBar();
     await expectFocusAfterKey("Tab", "searchbar", true);
-    await expectFocusAfterKey("Tab", afterUrlBarButton);
-    await expectFocusAfterKey("ArrowRight", "library-button");
-    await expectFocusAfterKey("ArrowLeft", afterUrlBarButton);
+    await expectFocusAfterKey("Tab", "library-button");
     await expectFocusAfterKey("Shift+Tab", "searchbar", true);
     await expectFocusAfterKey("Shift+Tab", gURLBar.inputField);
   });
