@@ -17,26 +17,29 @@ class BackupTest(MarionetteTestCase):
 
     def setUp(self):
         MarionetteTestCase.setUp(self)
-        # We need to quit the browser and restart with the browser.backup.log
-        # pref already set to true in order for it to be displayed.
+
+        # We need to force the "browser.backup.log" pref already set to true
+        # before Firefox starts in order for it to be displayed.
+        self.marionette.enforce_gecko_prefs({"browser.backup.log": True})
+
+        self.marionette.set_context("chrome")
+
+    def tearDown(self):
+        # Restart Firefox with a new profile to get rid from all modifications.
         self.marionette.quit()
-        self.marionette.instance.prefs = {
-            "browser.backup.log": True,
-        }
-        # Now restart the browser.
         self.marionette.instance.switch_profile()
         self.marionette.start_session()
 
-    def test_backup(self):
-        self.marionette.set_context("chrome")
+        MarionetteTestCase.tearDown(self)
 
+    def test_backup(self):
         self.add_test_cookie()
         self.add_test_login()
         self.add_test_certificate()
         self.add_test_saved_address()
         self.add_test_identity_credential()
         self.add_test_form_history()
-        self.add_test_activity_stream_snippets_data()
+        self.add_test_asrouter_snippets_data()
         self.add_test_protections_data()
         self.add_test_bookmarks()
         self.add_test_history()
@@ -81,9 +84,8 @@ class BackupTest(MarionetteTestCase):
 
         # Restart the browser to force all of the test data we just added
         # to be flushed to disk and to be made ready for backup
-        self.marionette.quit()
-        self.marionette.start_session()
-        self.marionette.set_context("chrome")
+        self.marionette.restart()
+
         # Put the OSKeyStore label back, since it would have been cleared
         # from memory during the restart.
         self.marionette.execute_script(
@@ -147,7 +149,7 @@ class BackupTest(MarionetteTestCase):
         # Start a brand new profile, one without any of the data we created or
         # backed up. This is the one that we'll be starting recovery from.
         self.marionette.quit()
-        self.marionette.instance.profile = None
+        self.marionette.instance.switch_profile()
         self.marionette.start_session()
         self.marionette.set_context("chrome")
 
@@ -237,7 +239,7 @@ class BackupTest(MarionetteTestCase):
         self.verify_recovered_saved_address()
         self.verify_recovered_identity_credential()
         self.verify_recovered_form_history()
-        self.verify_recovered_activity_stream_snippets_data()
+        self.verify_recovered_asrouter_snippets_data()
         self.verify_recovered_protections_data()
         self.verify_recovered_bookmarks()
         self.verify_recovered_history()
@@ -273,12 +275,12 @@ class BackupTest(MarionetteTestCase):
         )
         self.assertEqual(recoveredClientID, expectedClientID)
 
-        # Try not to pollute the profile list by getting rid of the one we just
-        # created.
         self.marionette.quit()
         self.marionette.instance.profile = originalProfile
         self.marionette.start_session()
         self.marionette.set_context("chrome")
+
+        # Don't pollute the profile list by getting rid of the one we just created.
         self.marionette.execute_async_script(
             """
           let [newProfileName, outerResolve] = arguments;
@@ -292,8 +294,7 @@ class BackupTest(MarionetteTestCase):
             script_args=[newProfileName],
         )
 
-        # Cleanup the archive we moved, and the recovery folder we decompressed
-        # to.
+        # Cleanup the archive we moved, and the recovery folder we decompressed to.
         mozfile.remove(archivePath)
         mozfile.remove(recoveryPath)
 
@@ -313,9 +314,9 @@ class BackupTest(MarionetteTestCase):
               false,
               false,
               false,
-              Date.now() / 1000 + 1,
+              Date.now() + 1000,
               {},
-              Ci.nsICookie.SAMESITE_NONE,
+              Ci.nsICookie.SAMESITE_UNSET,
               Ci.nsICookie.SCHEME_HTTP
             );
           })().then(outerResolve);
@@ -582,17 +583,17 @@ class BackupTest(MarionetteTestCase):
         )
         self.assertEqual(formHistoryResultsLength, 1)
 
-    def add_test_activity_stream_snippets_data(self):
+    def add_test_asrouter_snippets_data(self):
         self.marionette.execute_async_script(
             """
-          const { ActivityStreamStorage } = ChromeUtils.importESModule(
-            "resource://activity-stream/lib/ActivityStreamStorage.sys.mjs",
+          const { ASRouterStorage } = ChromeUtils.importESModule(
+            "resource:///modules/asrouter/ASRouterStorage.sys.mjs",
           );
           const SNIPPETS_TABLE_NAME = "snippets";
 
           let [outerResolve] = arguments;
           (async () => {
-            let storage = new ActivityStreamStorage({
+            let storage = new ASRouterStorage({
               storeNames: [SNIPPETS_TABLE_NAME],
             });
             let snippetsTable = await storage.getDbTable(SNIPPETS_TABLE_NAME);
@@ -601,17 +602,17 @@ class BackupTest(MarionetteTestCase):
         """
         )
 
-    def verify_recovered_activity_stream_snippets_data(self):
+    def verify_recovered_asrouter_snippets_data(self):
         snippetsResult = self.marionette.execute_async_script(
             """
-          const { ActivityStreamStorage } = ChromeUtils.importESModule(
-            "resource://activity-stream/lib/ActivityStreamStorage.sys.mjs",
+          const { ASRouterStorage } = ChromeUtils.importESModule(
+            "resource:///modules/asrouter/ASRouterStorage.sys.mjs",
           );
           const SNIPPETS_TABLE_NAME = "snippets";
 
           let [outerResolve] = arguments;
           (async () => {
-            let storage = new ActivityStreamStorage({
+            let storage = new ASRouterStorage({
               storeNames: [SNIPPETS_TABLE_NAME],
             });
             let snippetsTable = await storage.getDbTable(SNIPPETS_TABLE_NAME);
