@@ -7,6 +7,11 @@
 // This is loaded into chrome windows with the subscript loader. Wrap in
 // a block to prevent accidentally leaking globals onto `window`.
 {
+  const lazy = {};
+  ChromeUtils.defineESModuleGetters(lazy, {
+    TabMetrics: "moz-src:///browser/components/tabbrowser/TabMetrics.sys.mjs",
+  });
+
   class MozTabbrowserTab extends MozElements.MozTab {
     static markup = `
       <stack class="tab-stack" flex="1">
@@ -89,7 +94,7 @@
         ".tab-icon-pending":
           "fadein,pinned,busy,progress,selected=visuallyselected,pendingicon",
         ".tab-icon-image":
-          "src=image,triggeringprincipal=iconloadingprincipal,requestcontextid,fadein,pinned,selected=visuallyselected,busy,crashed,sharing,pictureinpicture,pending",
+          "src=image,triggeringprincipal=iconloadingprincipal,requestcontextid,fadein,pinned,selected=visuallyselected,busy,crashed,sharing,pictureinpicture,pending,discarded",
         ".tab-sharing-icon-overlay": "sharing,selected=visuallyselected,pinned",
         ".tab-icon-overlay":
           "sharing,pictureinpicture,crashed,busy,soundplaying,soundplaying-scheduledremoval,pinned,muted,blocked,selected=visuallyselected,activemedia-blocked",
@@ -105,10 +110,18 @@
       };
     }
 
+    #lastGroup;
     connectedCallback() {
+      this.#updateOnTabGrouped();
+      this.#lastGroup = this.group;
+
       this.initialize();
     }
 
+    disconnectedCallback() {
+      this.#updateOnTabUngrouped();
+    }
+
     initialize() {
       if (this._initialized) {
         return;
@@ -127,10 +140,18 @@
       let labelContainer = this.querySelector(".tab-label-container");
       labelContainer.addEventListener("overflow", this);
       labelContainer.addEventListener("underflow", this);
+
+      // Tabs in the tab strip default to being at the top level (level 1)
+      // Tabs in tab groups are one level down (level 2); this tab will
+      // update its value when it moves in and out of tab groups.
+      this.setAttribute("aria-level", 1);
     }
 
     #elementIndex;
     get elementIndex() {
+      if (!this.visible) {
+        throw new Error("Tab is not visible, so does not have an elementIndex");
+      }
       // Make sure the index is up to date.
       this.container.ariaFocusableItems;
       return this.#elementIndex;
@@ -203,7 +224,9 @@
     }
 
     get visible() {
-      return this.isOpen && !this.hidden && !this.group?.collapsed;
+      return (
+        this.isOpen && !this.hidden && (!this.group?.collapsed || this.selected)
+      );
     }
 
     get hidden() {
@@ -376,9 +399,9 @@
       }
 
       const diff_in_msec = Date.now() - this._lastUnloaded;
-      Services.telemetry
-        .getHistogramById("TAB_UNLOAD_TO_RELOAD")
-        .add(diff_in_msec / 1000);
+      Glean.browserEngagement.tabUnloadToReload.accumulateSingleSample(
+        diff_in_msec / 1000
+      );
       Glean.browserEngagement.tabReloadCount.add(1);
       delete this._lastUnloaded;
     }
@@ -542,11 +565,18 @@
 
       if (event.target.classList.contains("tab-close-button")) {
         if (this.multiselected) {
-          gBrowser.removeMultiSelectedTabs();
+          gBrowser.removeMultiSelectedTabs(
+            lazy.TabMetrics.userTriggeredContext(
+              lazy.TabMetrics.METRIC_SOURCE.TAB_STRIP
+            )
+          );
         } else {
           gBrowser.removeTab(this, {
             animate: true,
             triggeringEvent: event,
+            ...lazy.TabMetrics.userTriggeredContext(
+              lazy.TabMetrics.METRIC_SOURCE.TAB_STRIP
+            ),
           });
         }
         // This enables double-click protection for the tab container
@@ -698,6 +728,45 @@
     on_underflow(event) {
       event.currentTarget.removeAttribute("textoverflow");
     }
+
+    #updateOnTabGrouped() {
+      if (this.group && this.#lastGroup != this.group) {
+        // Trigger TabGrouped on the tab group, not the tab itself. This is a
+        // bit unorthodox, but fixes bug1964152 where tab group events are not
+        // fired correctly when tabs change windows (because the tab is
+        // detached from the DOM at time of the event).
+        this.group.dispatchEvent(
+          new CustomEvent("TabGrouped", {
+            bubbles: true,
+            detail: this,
+          })
+        );
+        this.setAttribute("aria-level", 2);
+      }
+    }
+
+    #updateOnTabUngrouped() {
+      if (this.#lastGroup && this.#lastGroup != this.group) {
+        // Trigger TabUngrouped on the tab group, not the tab itself. This is a
+        // bit unorthodox, but fixes bug1964152 where tab group events are not
+        // fired correctly when tabs change windows (because the tab is
+        // detached from the DOM at time of the event).
+        this.#lastGroup.dispatchEvent(
+          new CustomEvent("TabUngrouped", {
+            bubbles: true,
+            detail: this,
+          })
+        );
+        // Tab could have moved to be ungrouped (level 1)
+        // or to a different group (level 2).
+        this.setAttribute("aria-level", this.group ? 2 : 1);
+        // `posinset` and `setsize` only need to be set explicitly
+        // on grouped tabs so that a11y tools can tell users that a
+        // given tab is "2 of 7" in the group, for example.
+        this.removeAttribute("aria-posinset");
+        this.removeAttribute("aria-setsize");
+      }
+    }
   }
 
   customElements.define("tabbrowser-tab", MozTabbrowserTab, {
