@@ -145,7 +145,7 @@ function promiseControllerNotification(
         get: (target, name) => {
           if (name == notification) {
             return (...args) => {
-              controller.removeQueryListener(proxifiedObserver);
+              controller.removeListener(proxifiedObserver);
               if (expected) {
                 resolve(args);
               } else {
@@ -157,7 +157,7 @@ function promiseControllerNotification(
         },
       }
     );
-    controller.addQueryListener(proxifiedObserver);
+    controller.addListener(proxifiedObserver);
   });
 }
 
@@ -330,21 +330,33 @@ function defaultRichSuggestionsFn(searchStr) {
   ];
 }
 
-async function addOpenPages(uri, count = 1, userContextId = 0) {
+async function addOpenPages(
+  uri,
+  count = 1,
+  userContextId = 0,
+  tabGroupId = null
+) {
   for (let i = 0; i < count; i++) {
     await UrlbarProviderOpenTabs.registerOpenTab(
       uri.spec,
       userContextId,
+      tabGroupId,
       false
     );
   }
 }
 
-async function removeOpenPages(aUri, aCount = 1, aUserContextId = 0) {
+async function removeOpenPages(
+  aUri,
+  aCount = 1,
+  aUserContextId = 0,
+  tabGroupId = null
+) {
   for (let i = 0; i < aCount; i++) {
     await UrlbarProviderOpenTabs.unregisterOpenTab(
       aUri.spec,
       aUserContextId,
+      tabGroupId,
       false
     );
   }
@@ -538,12 +550,14 @@ function makeOmniboxResult(
  * @param {string} [options.iconUri]
  *   A URI for the page icon.
  * @param {number} [options.userContextId]
- *   A id of the userContext in which the tab is located.
+ *   An id of the userContext in which the tab is located.
+ * @param {string} [options.tabGroup]
+ *   An id of the tab group in which the tab is located.
  * @returns {UrlbarResult}
  */
 function makeTabSwitchResult(
   queryContext,
-  { uri, title, iconUri, userContextId }
+  { uri, title, iconUri, userContextId, tabGroup }
 ) {
   return new UrlbarResult(
     UrlbarUtils.RESULT_TYPE.TAB_SWITCH,
@@ -554,6 +568,7 @@ function makeTabSwitchResult(
       // Check against undefined so consumers can pass in the empty string.
       icon: typeof iconUri != "undefined" ? iconUri : `page-icon:${uri}`,
       userContextId: [userContextId || 0],
+      tabGroup: [tabGroup || null],
     })
   );
 }
@@ -1033,7 +1048,22 @@ async function check_results({
     richSuggestionIconVariation: { optional: true },
   };
 
-  let optionalPayloadProperties = new Set(["lastVisit"]);
+  // Payload properties to conditionally check. Properties not specified here
+  // will always be checked.
+  //
+  // ignore:
+  //   Always ignore the property.
+  // optional:
+  //   Ignore the property if it's not in the expected result.
+  let conditionalPayloadProperties = {
+    frecency: { optional: true },
+    lastVisit: { optional: true },
+    // `suggestionObject` is only used for dismissing Suggest Rust results, and
+    // important properties in this object are reflected in the top-level
+    // payload object, so ignore it. There are Suggest tests specifically for
+    // dismissals that indirectly test the important aspects of this property.
+    suggestionObject: { ignore: true },
+  };
 
   for (let i = 0; i < matches.length; i++) {
     let actual = context.results[i];
@@ -1047,7 +1077,7 @@ async function check_results({
     );
 
     for (let [key, { optional, map }] of Object.entries(propertiesToCheck)) {
-      if (!optional || expected.hasOwnProperty(key)) {
+      if (!optional || expected[key] !== undefined) {
         map ??= v => v;
         Assert.equal(
           map(actual[key]),
@@ -1080,12 +1110,18 @@ async function check_results({
     }
 
     if (expected.payload) {
-      let expectedEntries = new Set(Object.keys(expected.payload));
-      let actualEntries = new Set(Object.keys(actual.payload)).difference(
-        optionalPayloadProperties
-      );
+      let expectedKeys = new Set(Object.keys(expected.payload));
+      let actualKeys = new Set(Object.keys(actual.payload));
+
+      for (let key of actualKeys.union(expectedKeys)) {
+        let condition = conditionalPayloadProperties[key];
+        if (
+          condition?.ignore ||
+          (condition?.optional && !expected.hasOwnProperty(key))
+        ) {
+          continue;
+        }
 
-      for (let key of actualEntries.union(expectedEntries)) {
         Assert.deepEqual(
           actual.payload[key],
           expected.payload[key],
