@@ -14,27 +14,19 @@ import { Search } from "content-src/components/Search/Search";
 import { Sections } from "content-src/components/Sections/Sections";
 import { Logo } from "content-src/components/Logo/Logo";
 import { Weather } from "content-src/components/Weather/Weather";
+import { DownloadModalToggle } from "content-src/components/DownloadModalToggle/DownloadModalToggle";
 import { Notifications } from "content-src/components/Notifications/Notifications";
 import { TopicSelection } from "content-src/components/DiscoveryStreamComponents/TopicSelection/TopicSelection";
+import { DownloadMobilePromoHighlight } from "../DiscoveryStreamComponents/FeatureHighlight/DownloadMobilePromoHighlight";
 import { WallpaperFeatureHighlight } from "../DiscoveryStreamComponents/FeatureHighlight/WallpaperFeatureHighlight";
+import { MessageWrapper } from "content-src/components/MessageWrapper/MessageWrapper";
 
 const VISIBLE = "visible";
 const VISIBILITY_CHANGE_EVENT = "visibilitychange";
-const WALLPAPER_HIGHLIGHT_DISMISSED_PREF =
-  "newtabWallpapers.highlightDismissed";
-const PREF_THUMBS_UP_DOWN_ENABLED = "discoverystream.thumbsUpDown.enabled";
-const PREF_THUMBS_UP_DOWN_LAYOUT_ENABLED =
-  "discoverystream.thumbsUpDown.searchTopsitesCompact";
-
-export const PrefsButton = ({ onClick, icon }) => (
-  <div className="prefs-button">
-    <button
-      className={`icon ${icon || "icon-settings"}`}
-      onClick={onClick}
-      data-l10n-id="newtab-settings-button"
-    />
-  </div>
-);
+const PREF_INFERRED_PERSONALIZATION_SYSTEM =
+  "discoverystream.sections.personalization.inferred.enabled";
+const PREF_INFERRED_PERSONALIZATION_USER =
+  "discoverystream.sections.personalization.inferred.user.enabled";
 
 // Returns a function will not be continuously triggered when called. The
 // function will be triggered if called again after `wait` milliseconds.
@@ -120,18 +112,22 @@ export class BaseContent extends React.PureComponent {
     this.handleOnKeyDown = this.handleOnKeyDown.bind(this);
     this.onWindowScroll = debounce(this.onWindowScroll.bind(this), 5);
     this.setPref = this.setPref.bind(this);
-    this.shouldShowWallpapersHighlight =
-      this.shouldShowWallpapersHighlight.bind(this);
+    this.shouldShowOMCHighlight = this.shouldShowOMCHighlight.bind(this);
     this.updateWallpaper = this.updateWallpaper.bind(this);
     this.prefersDarkQuery = null;
     this.handleColorModeChange = this.handleColorModeChange.bind(this);
     this.shouldDisplayTopicSelectionModal =
       this.shouldDisplayTopicSelectionModal.bind(this);
+    this.toggleDownloadHighlight = this.toggleDownloadHighlight.bind(this);
+    this.handleDismissDownloadHighlight =
+      this.handleDismissDownloadHighlight.bind(this);
     this.state = {
       fixedSearch: false,
       firstVisibleTimestamp: null,
       colorMode: "",
       fixedNavStyle: {},
+      wallpaperTheme: "",
+      showDownloadHighlightOverride: null,
     };
   }
 
@@ -146,6 +142,8 @@ export class BaseContent extends React.PureComponent {
   componentDidMount() {
     global.addEventListener("scroll", this.onWindowScroll);
     global.addEventListener("keydown", this.handleOnKeyDown);
+    const prefs = this.props.Prefs.values;
+    const wallpapersEnabled = prefs["newtabWallpapers.enabled"];
     if (this.props.document.visibilityState === VISIBLE) {
       this.setFirstVisibleTimestamp();
       this.shouldDisplayTopicSelectionModal();
@@ -176,6 +174,43 @@ export class BaseContent extends React.PureComponent {
       this.handleColorModeChange
     );
     this.handleColorModeChange();
+    if (wallpapersEnabled) {
+      this.updateWallpaper();
+    }
+  }
+
+  componentDidUpdate(prevProps) {
+    const prefs = this.props.Prefs.values;
+    const wallpapersEnabled = prefs["newtabWallpapers.enabled"];
+    if (wallpapersEnabled) {
+      // destructure current and previous props with fallbacks
+      // (preventing undefined errors)
+      const {
+        Wallpapers: { uploadedWallpaper = null, wallpaperList = null } = {},
+      } = this.props;
+
+      const {
+        Wallpapers: {
+          uploadedWallpaper: prevUploadedWallpaper = null,
+          wallpaperList: prevWallpaperList = null,
+        } = {},
+        Prefs: { values: prevPrefs = {} } = {},
+      } = prevProps;
+
+      const selectedWallpaper = prefs["newtabWallpapers.wallpaper"];
+      const prevSelectedWallpaper = prevPrefs["newtabWallpapers.wallpaper"];
+
+      // don't update wallpaper unless the wallpaper is being changed.
+      if (
+        selectedWallpaper !== prevSelectedWallpaper || // selecting a new wallpaper
+        uploadedWallpaper !== prevUploadedWallpaper || // uploading a new wallpaper
+        wallpaperList !== prevWallpaperList || // remote settings wallpaper list updates
+        this.props.App.isForStartupCache.Wallpaper !==
+          prevProps.App.isForStartupCache.Wallpaper // Startup cached page wallpaper is updating
+      ) {
+        this.updateWallpaper();
+      }
+    }
   }
 
   handleColorModeChange() {
@@ -215,69 +250,25 @@ export class BaseContent extends React.PureComponent {
     }
 
     const logoAlwaysVisible = prefs["logowordmark.alwaysVisible"];
-    const layoutsVariantAEnabled = prefs["newtabLayouts.variant-a"];
-    const layoutsVariantBEnabled = prefs["newtabLayouts.variant-b"];
-    const layoutsVariantAorB = layoutsVariantAEnabled || layoutsVariantBEnabled;
-    const thumbsUpDownEnabled = prefs[PREF_THUMBS_UP_DOWN_ENABLED];
-    // For the compact layout to be active,
-    // thumbs also has to be enabled until Bug 1932242 is fixed
-    const thumbsUpDownLayoutEnabled =
-      prefs[PREF_THUMBS_UP_DOWN_LAYOUT_ENABLED] && thumbsUpDownEnabled;
 
     /* Bug 1917937: The logic presented below is fragile but accurate to the pixel. As new tab experiments with layouts, we have a tech debt of competing styles and classes the slightly modify where the search bar sits on the page. The larger solution for this is to replace everything with an intersection observer, but would require a larger refactor of this file. In the interim, we can programmatically calculate when to fire the fixed-scroll event and account for the moved elements so that topsites/etc stays in the same place. The CSS this references has been flagged to reference this logic so (hopefully) keep them in sync. */
 
     let SCROLL_THRESHOLD = 0; // When the fixed-scroll event fires
     let MAIN_OFFSET_PADDING = 0; // The padding to compensate for the moved elements
 
+    const CSS_VAR_SPACE_XXLARGE = 32.04; // Custom Acorn themed variable (8 * 0.267rem);
+
     let layout = {
-      outerWrapperPaddingTop: 30,
-      searchWrapperPaddingTop: 34,
-      searchWrapperPaddingBottom: 38,
+      outerWrapperPaddingTop: 24,
+      searchWrapperPaddingTop: 16,
+      searchWrapperPaddingBottom: CSS_VAR_SPACE_XXLARGE,
       searchWrapperFixedScrollPaddingTop: 27,
       searchWrapperFixedScrollPaddingBottom: 27,
       searchInnerWrapperMinHeight: 52,
-      logoAndWordmarkWrapperHeight: 64,
-      logoAndWordmarkWrapperMarginBottom: 48,
+      logoAndWordmarkWrapperHeight: 0,
+      logoAndWordmarkWrapperMarginBottom: 0,
     };
 
-    const CSS_VAR_SPACE_XXLARGE = 34.2; // Custom Acorn themed variable (8 * 0.267rem);
-
-    // Experimental layouts
-    // (Note these if statements are ordered to match the CSS cascade)
-    if (thumbsUpDownLayoutEnabled || layoutsVariantAorB) {
-      // Thumbs Compact View Layout
-      if (thumbsUpDownLayoutEnabled) {
-        layout.logoAndWordmarkWrapperMarginBottom = CSS_VAR_SPACE_XXLARGE;
-        if (!logoAlwaysVisible) {
-          layout.searchWrapperPaddingTop = CSS_VAR_SPACE_XXLARGE;
-          layout.searchWrapperPaddingBottom = CSS_VAR_SPACE_XXLARGE;
-        }
-      }
-
-      // Variant B Layout
-      if (layoutsVariantAEnabled) {
-        layout.outerWrapperPaddingTop = 24;
-        if (!thumbsUpDownLayoutEnabled) {
-          layout.searchWrapperPaddingTop = 0;
-          layout.searchWrapperPaddingBottom = 32;
-          layout.logoAndWordmarkWrapperMarginBottom = 32;
-        }
-      }
-
-      // Variant B Layout
-      if (layoutsVariantBEnabled) {
-        layout.outerWrapperPaddingTop = 24;
-        // Logo is positioned absolute, so remove it
-        layout.logoAndWordmarkWrapperHeight = 0;
-        layout.logoAndWordmarkWrapperMarginBottom = 0;
-        layout.searchWrapperPaddingTop = 16;
-        layout.searchWrapperPaddingBottom = CSS_VAR_SPACE_XXLARGE;
-        if (!thumbsUpDownLayoutEnabled) {
-          layout.searchWrapperPaddingBottom = 32;
-        }
-      }
-    }
-
     // Logo visibility applies to all layouts
     if (!logoAlwaysVisible) {
       layout.logoAndWordmarkWrapperHeight = 0;
@@ -378,16 +369,38 @@ export class BaseContent extends React.PureComponent {
 
   async updateWallpaper() {
     const prefs = this.props.Prefs.values;
-
     const selectedWallpaper = prefs["newtabWallpapers.wallpaper"];
-    const { wallpaperList } = this.props.Wallpapers;
+    const { wallpaperList, uploadedWallpaper: uploadedWallpaperUrl } =
+      this.props.Wallpapers;
+    let lightWallpaper = {};
+    let darkWallpaper = {};
+
+    if (selectedWallpaper === "custom" && uploadedWallpaperUrl) {
+      try {
+        global.document?.body.style.setProperty(
+          "--newtab-wallpaper",
+          `url(${uploadedWallpaperUrl})`
+        );
+
+        global.document?.body.style.setProperty(
+          "--newtab-wallpaper-color",
+          "transparent"
+        );
+
+        // Based on the current colorMode, add the corresponding dark/light CSS classes
+        if (this.state.colorMode) {
+          this.setState(prevState => ({
+            wallpaperTheme: prevState.colorMode,
+          }));
+        }
+      } catch (e) {}
+
+      return;
+    }
 
     if (wallpaperList) {
       let wallpaper = wallpaperList.find(wp => wp.title === selectedWallpaper);
-
-      let lightWallpaper = {};
-      let darkWallpaper = {};
-      if (selectedWallpaper) {
+      if (selectedWallpaper && wallpaper) {
         // if selectedWallpaper exists - we override what light and dark prefs are to match that
         lightWallpaper = wallpaper;
         darkWallpaper = wallpaper;
@@ -435,60 +448,44 @@ export class BaseContent extends React.PureComponent {
         }
       }
 
-      // Add helper class to body if user has a wallpaper selected
-      if (wallpaperTheme === "light") {
-        global.document?.body.classList.add("lightWallpaper");
-        global.document?.body.classList.remove("darkWallpaper");
-      }
-
-      if (wallpaperTheme === "dark") {
-        global.document?.body.classList.add("darkWallpaper");
-        global.document?.body.classList.remove("lightWallpaper");
-      }
+      this.setState({ wallpaperTheme });
     }
   }
 
-  // Contains all the logic to show the wallpapers Feature Highlight
-  shouldShowWallpapersHighlight() {
-    const prefs = this.props.Prefs.values;
-
-    // If wallpapers (or v2 wallpapers) are not enabled, don't show the highlight.
-    const wallpapersV2Enabled = prefs["newtabWallpapers.v2.enabled"];
-    if (!wallpapersV2Enabled) {
+  shouldShowOMCHighlight(componentId) {
+    const messageData = this.props.Messages?.messageData;
+    if (!messageData || Object.keys(messageData).length === 0) {
       return false;
     }
+    return messageData?.content?.messageType === componentId;
+  }
 
-    // If user has interacted/dismissed the highlight, don't show
-    const wallpapersHighlightDismissed =
-      prefs[WALLPAPER_HIGHLIGHT_DISMISSED_PREF];
-    if (wallpapersHighlightDismissed) {
-      return false;
-    }
+  toggleDownloadHighlight() {
+    this.setState(prevState => {
+      const override = !(
+        prevState.showDownloadHighlightOverride ??
+        this.shouldShowOMCHighlight("DownloadMobilePromoHighlight")
+      );
 
-    // If the user has selected a wallpaper, don't show the pop-up
-    const activeWallpaper = prefs[`newtabWallpapers.wallpaper`];
+      if (override) {
+        // Emit an open event manually since OMC isn't handling it
+        this.props.dispatch(
+          ac.DiscoveryStreamUserEvent({
+            event: "FEATURE_HIGHLIGHT_OPEN",
+            source: "FEATURE_HIGHLIGHT",
+            value: { feature: "FEATURE_DOWNLOAD_MOBILE_PROMO" },
+          })
+        );
+      }
 
-    if (activeWallpaper) {
-      this.props.dispatch(ac.SetPref(WALLPAPER_HIGHLIGHT_DISMISSED_PREF, true));
-      return false;
-    }
+      return {
+        showDownloadHighlightOverride: override,
+      };
+    });
+  }
 
-    // If the user has seen* the highlight more than three times
-    // *Seen means they loaded HNT page and the highlight was observed for more than 3 seconds
-    const { highlightSeenCounter } = this.props.Wallpapers;
-    if (highlightSeenCounter.value > 3) {
-      return false;
-    }
-
-    // Show the highlight if available
-    const wallpapersHighlightEnabled =
-      prefs["newtabWallpapers.highlightEnabled"];
-    if (wallpapersHighlightEnabled) {
-      return true;
-    }
-
-    // Default return value
-    return false;
+  handleDismissDownloadHighlight() {
+    this.setState({ showDownloadHighlightOverride: false });
   }
 
   getRGBColors(input) {
@@ -548,24 +545,21 @@ export class BaseContent extends React.PureComponent {
     }
   }
 
+  // eslint-disable-next-line max-statements, complexity
   render() {
     const { props } = this;
     const { App, DiscoveryStream } = props;
     const { initialized, customizeMenuVisible } = App;
     const prefs = props.Prefs.values;
 
-    const layoutsVariantAEnabled = prefs["newtabLayouts.variant-a"];
-    const layoutsVariantBEnabled = prefs["newtabLayouts.variant-b"];
     const shortcutsRefresh = prefs["newtabShortcuts.refresh"];
-    const layoutsVariantAorB = layoutsVariantAEnabled || layoutsVariantBEnabled;
 
     const activeWallpaper = prefs[`newtabWallpapers.wallpaper`];
-    const wallpapersV2Enabled = prefs["newtabWallpapers.v2.enabled"];
+    const wallpapersEnabled = prefs["newtabWallpapers.enabled"];
     const weatherEnabled = prefs.showWeather;
     const { showTopicSelection } = DiscoveryStream;
     const mayShowTopicSelection =
       showTopicSelection && prefs["discoverystream.topicSelection.enabled"];
-
     const { pocketConfig } = prefs;
 
     const isDiscoveryStream =
@@ -592,20 +586,46 @@ export class BaseContent extends React.PureComponent {
     const enabledSections = {
       topSitesEnabled: prefs["feeds.topsites"],
       pocketEnabled: prefs["feeds.section.topstories"],
-      highlightsEnabled: prefs["feeds.section.highlights"],
-      showSponsoredTopSitesEnabled: prefs.showSponsoredTopSites,
-      showSponsoredPocketEnabled: prefs.showSponsored,
+      showInferredPersonalizationEnabled:
+        prefs[PREF_INFERRED_PERSONALIZATION_USER],
       showRecentSavesEnabled: prefs.showRecentSaves,
       topSitesRowsCount: prefs.topSitesRows,
       weatherEnabled: prefs.showWeather,
+      trendingSearchEnabled: prefs["trendingSearch.enabled"],
     };
 
     const pocketRegion = prefs["feeds.system.topstories"];
     const mayHaveSponsoredStories = prefs["system.showSponsored"];
+    const mayHaveInferredPersonalization =
+      prefs[PREF_INFERRED_PERSONALIZATION_SYSTEM];
     const mayHaveWeather = prefs["system.showWeather"];
     const { mayHaveSponsoredTopSites } = prefs;
     const supportUrl = prefs["support.url"];
 
+    // Trending Searches experiment pref check
+    const mayHaveTrendingSearch =
+      prefs["system.trendingSearch.enabled"] &&
+      prefs["trendingSearch.defaultSearchEngine"].toLowerCase() === "google";
+
+    // Mobile Download Promo Pref Checks
+    const mobileDownloadPromoEnabled = prefs["mobileDownloadModal.enabled"];
+    const mobileDownloadPromoVariantAEnabled =
+      prefs["mobileDownloadModal.variant-a"];
+    const mobileDownloadPromoVariantBEnabled =
+      prefs["mobileDownloadModal.variant-b"];
+    const mobileDownloadPromoVariantCEnabled =
+      prefs["mobileDownloadModal.variant-c"];
+    const mobileDownloadPromoVariantABorC =
+      mobileDownloadPromoVariantAEnabled ||
+      mobileDownloadPromoVariantBEnabled ||
+      mobileDownloadPromoVariantCEnabled;
+    const mobileDownloadPromoWrapperHeightModifier =
+      prefs["weather.display"] === "detailed" &&
+      weatherEnabled &&
+      mayHaveWeather
+        ? "is-tall"
+        : "";
+
     const hasThumbsUpDownLayout =
       prefs["discoverystream.thumbsUpDown.searchTopsitesCompact"];
     const hasThumbsUpDown = prefs["discoverystream.thumbsUpDown.enabled"];
@@ -613,18 +633,25 @@ export class BaseContent extends React.PureComponent {
     const topicLabelsEnabled = prefs["discoverystream.topicLabels.enabled"];
     const sectionsCustomizeMenuPanelEnabled =
       prefs["discoverystream.sections.customizeMenuPanel.enabled"];
+    const sectionsPersonalizationEnabled =
+      prefs["discoverystream.sections.personalization.enabled"];
+
     // Logic to show follow/block topic mgmt panel in Customize panel
-    const mayHaveTopicSections =
+    const mayHavePersonalizedTopicSections =
+      sectionsPersonalizationEnabled &&
       topicLabelsEnabled &&
       sectionsEnabled &&
       sectionsCustomizeMenuPanelEnabled &&
       DiscoveryStream.feeds.loaded;
 
     const featureClassName = [
-      weatherEnabled && mayHaveWeather && "has-weather", // Show is weather is enabled/visible
+      mobileDownloadPromoEnabled &&
+        mobileDownloadPromoVariantABorC &&
+        "has-mobile-download-promo", // Mobile download promo modal is enabled/visible
+      weatherEnabled && mayHaveWeather && "has-weather", // Weather widget is enabled/visible
       prefs.showSearch ? "has-search" : "no-search",
-      layoutsVariantAEnabled ? "layout-variant-a" : "", // Layout experiment variant A
-      layoutsVariantBEnabled ? "layout-variant-b" : "", // Layout experiment variant B
+      // layoutsVariantAEnabled ? "layout-variant-a" : "", // Layout experiment variant A
+      // layoutsVariantBEnabled ? "layout-variant-b" : "", // Layout experiment variant B
       shortcutsRefresh ? "shortcuts-refresh" : "", // Shortcuts refresh experiment
       pocketEnabled ? "has-recommended-stories" : "no-recommended-stories",
       sectionsEnabled ? "has-sections-grid" : "",
@@ -651,10 +678,25 @@ export class BaseContent extends React.PureComponent {
     ]
       .filter(v => v)
       .join(" ");
-    if (wallpapersV2Enabled) {
-      this.updateWallpaper();
+    if (wallpapersEnabled) {
+      // Add helper class to body if user has a wallpaper selected
+      if (this.state.wallpaperTheme === "light") {
+        global.document?.body.classList.add("lightWallpaper");
+        global.document?.body.classList.remove("darkWallpaper");
+      }
+
+      if (this.state.wallpaperTheme === "dark") {
+        global.document?.body.classList.add("darkWallpaper");
+        global.document?.body.classList.remove("lightWallpaper");
+      }
     }
 
+    // If state.showDownloadHighlightOverride has value, let it override the logic
+    // Otherwise, defer to OMC message display logic
+    const shouldShowDownloadHighlight =
+      this.state.showDownloadHighlightOverride ??
+      this.shouldShowOMCHighlight("DownloadMobilePromoHighlight");
+
     return (
       <div className={featureClassName}>
         {/* Floating menu for customize menu toggle */}
@@ -665,21 +707,25 @@ export class BaseContent extends React.PureComponent {
             openPreferences={this.openPreferences}
             setPref={this.setPref}
             enabledSections={enabledSections}
-            wallpapersV2Enabled={wallpapersV2Enabled}
+            wallpapersEnabled={wallpapersEnabled}
             activeWallpaper={activeWallpaper}
             pocketRegion={pocketRegion}
-            mayHaveTopicSections={mayHaveTopicSections}
+            mayHaveTopicSections={mayHavePersonalizedTopicSections}
             mayHaveSponsoredTopSites={mayHaveSponsoredTopSites}
             mayHaveSponsoredStories={mayHaveSponsoredStories}
+            mayHaveInferredPersonalization={mayHaveInferredPersonalization}
             mayHaveWeather={mayHaveWeather}
+            mayHaveTrendingSearch={mayHaveTrendingSearch}
             spocMessageVariant={spocMessageVariant}
             showing={customizeMenuVisible}
           />
-          {this.shouldShowWallpapersHighlight() && (
-            <WallpaperFeatureHighlight
-              position="inset-block-end inset-inline-start"
-              dispatch={this.props.dispatch}
-            />
+          {this.shouldShowOMCHighlight("CustomWallpaperHighlight") && (
+            <MessageWrapper dispatch={this.props.dispatch}>
+              <WallpaperFeatureHighlight
+                position="inset-block-start inset-inline-start"
+                dispatch={this.props.dispatch}
+              />
+            </MessageWrapper>
           )}
         </menu>
         <div className="weatherWrapper">
@@ -689,6 +735,31 @@ export class BaseContent extends React.PureComponent {
             </ErrorBoundary>
           )}
         </div>
+        <div
+          className={`mobileDownloadPromoWrapper ${mobileDownloadPromoWrapperHeightModifier}`}
+        >
+          {mobileDownloadPromoEnabled && mobileDownloadPromoVariantABorC && (
+            <ErrorBoundary>
+              <DownloadModalToggle
+                isActive={shouldShowDownloadHighlight}
+                onClick={this.toggleDownloadHighlight}
+              />
+              {shouldShowDownloadHighlight && (
+                <MessageWrapper
+                  hiddenOverride={shouldShowDownloadHighlight}
+                  onDismiss={this.handleDismissDownloadHighlight}
+                  dispatch={this.props.dispatch}
+                >
+                  <DownloadMobilePromoHighlight
+                    position={`inset-inline-start inset-block-end`}
+                    dispatch={this.props.dispatch}
+                  />
+                </MessageWrapper>
+              )}
+            </ErrorBoundary>
+          )}
+        </div>
+
         {/* eslint-disable-next-line jsx-a11y/click-events-have-key-events, jsx-a11y/no-static-element-interactions*/}
         <div className={outerClassName} onClick={this.closeCustomizationMenu}>
           <main className="newtab-main" style={this.state.fixedNavStyle}>
@@ -706,9 +777,7 @@ export class BaseContent extends React.PureComponent {
               </div>
             )}
             {/* Bug 1914055: Show logo regardless if search is enabled */}
-            {!prefs.showSearch && layoutsVariantAorB && !noSectionsEnabled && (
-              <Logo />
-            )}
+            {!prefs.showSearch && !noSectionsEnabled && <Logo />}
             <div className={`body-wrapper${initialized ? " on" : ""}`}>
               {isDiscoveryStream ? (
                 <ErrorBoundary className="borderless-error">
@@ -723,7 +792,7 @@ export class BaseContent extends React.PureComponent {
               )}
             </div>
             <ConfirmDialog />
-            {wallpapersV2Enabled && this.renderWallpaperAttribution()}
+            {wallpapersEnabled && this.renderWallpaperAttribution()}
           </main>
           <aside>
             {this.props.Notifications?.showNotifications && (
@@ -751,6 +820,7 @@ export const Base = connect(state => ({
   Prefs: state.Prefs,
   Sections: state.Sections,
   DiscoveryStream: state.DiscoveryStream,
+  Messages: state.Messages,
   Notifications: state.Notifications,
   Search: state.Search,
   Wallpapers: state.Wallpapers,
