@@ -8,6 +8,7 @@ import { CollapsibleSection } from "content-src/components/CollapsibleSection/Co
 import { connect } from "react-redux";
 import { DSMessage } from "content-src/components/DiscoveryStreamComponents/DSMessage/DSMessage";
 import { DSPrivacyModal } from "content-src/components/DiscoveryStreamComponents/DSPrivacyModal/DSPrivacyModal";
+import { ReportContent } from "../DiscoveryStreamComponents/ReportContent/ReportContent";
 import { DSSignup } from "content-src/components/DiscoveryStreamComponents/DSSignup/DSSignup";
 import { DSTextPromo } from "content-src/components/DiscoveryStreamComponents/DSTextPromo/DSTextPromo";
 import { Highlights } from "content-src/components/DiscoveryStreamComponents/Highlights/Highlights";
@@ -19,6 +20,7 @@ import { SectionTitle } from "content-src/components/DiscoveryStreamComponents/S
 import { selectLayoutRender } from "content-src/lib/selectLayoutRender";
 import { TopSites } from "content-src/components/TopSites/TopSites";
 import { CardSections } from "../DiscoveryStreamComponents/CardSections/CardSections";
+import { Widgets } from "content-src/components/Widgets/Widgets";
 
 const ALLOWED_CSS_URL_PREFIXES = [
   "chrome://",
@@ -219,6 +221,7 @@ export class _DiscoveryStreamBase extends React.PureComponent {
             recentSavesEnabled={this.props.DiscoveryStream.recentSavesEnabled}
             hideDescriptions={this.props.DiscoveryStream.hideDescriptions}
             firstVisibleTimestamp={this.props.firstVisibleTimestamp}
+            spocPositions={component.spocs?.positions}
           />
         );
       }
@@ -226,6 +229,8 @@ export class _DiscoveryStreamBase extends React.PureComponent {
         return <HorizontalRule />;
       case "PrivacyLink":
         return <PrivacyLink properties={component.properties} />;
+      case "Widgets":
+        return <Widgets />;
       default:
         return <div>{component.type}</div>;
     }
@@ -251,6 +256,9 @@ export class _DiscoveryStreamBase extends React.PureComponent {
     const { config } = this.props.DiscoveryStream;
     const topicSelectionEnabled =
       this.props.Prefs.values["discoverystream.topicSelection.enabled"];
+    const reportAdsEnabled =
+      this.props.Prefs.values["discoverystream.reportAds.enabled"];
+    const spocsEnabled = this.props.Prefs.values["unifiedAds.spocs.enabled"];
 
     // Allow rendering without extracting special components
     if (!config.collapsible) {
@@ -284,6 +292,7 @@ export class _DiscoveryStreamBase extends React.PureComponent {
 
     // Extract TopSites to render before the rest and Message to use for header
     const topSites = extractComponent("TopSites");
+    const widgets = extractComponent("Widgets");
     const sponsoredCollection = extractComponent("CollectionCardGrid");
     const message = extractComponent("Message") || {
       header: {
@@ -315,12 +324,19 @@ export class _DiscoveryStreamBase extends React.PureComponent {
       }
     }
 
-    // Render a DS-style TopSites then the rest if any in a collapsible section
+    const { DiscoveryStream } = this.props;
+
     return (
       <React.Fragment>
         {this.props.DiscoveryStream.isPrivacyInfoModalVisible && (
           <DSPrivacyModal dispatch={this.props.dispatch} />
         )}
+
+        {/* Reporting stories/articles will only be available in sections, not the default card grid  */}
+        {((reportAdsEnabled && spocsEnabled) || sectionsEnabled) && (
+          <ReportContent spocs={DiscoveryStream.spocs} />
+        )}
+
         {topSites &&
           this.renderLayout([
             {
@@ -329,6 +345,14 @@ export class _DiscoveryStreamBase extends React.PureComponent {
               sectionType: "topsites",
             },
           ])}
+        {widgets &&
+          this.renderLayout([
+            {
+              width: 12,
+              components: [widgets],
+              sectionType: "widgets",
+            },
+          ])}
         {sponsoredCollection &&
           this.renderLayout([
             {
@@ -378,10 +402,12 @@ export class _DiscoveryStreamBase extends React.PureComponent {
     const styles = [];
     let [data] = layoutRender;
     // Add helper class for topsites
-    const topsitesClass = data.sectionType ? "ds-layout-topsites" : "";
+    const sectionClass = data.sectionType
+      ? `ds-layout-${data.sectionType}`
+      : "";
 
     return (
-      <div className={`discovery-stream ds-layout ${topsitesClass}`}>
+      <div className={`discovery-stream ds-layout ${sectionClass}`}>
         {layoutRender.map((row, rowIndex) => (
           <div
             key={`row-${rowIndex}`}
