@@ -13,6 +13,7 @@ import { FluentOrText } from "../../FluentOrText/FluentOrText.jsx";
 import { actionCreators as ac, actionTypes as at } from "common/Actions.mjs";
 import React, { useEffect, useState, useRef, useCallback } from "react";
 import { connect, useSelector } from "react-redux";
+import { TrendingSearches } from "../TrendingSearches/TrendingSearches.jsx";
 const PREF_ONBOARDING_EXPERIENCE_DISMISSED =
   "discoverystream.onboardingExperience.dismissed";
 const PREF_SECTIONS_CARDS_ENABLED = "discoverystream.sections.cards.enabled";
@@ -31,6 +32,10 @@ const PREF_BILLBOARD_ENABLED = "newtabAdSize.billboard";
 const PREF_LEADERBOARD_ENABLED = "newtabAdSize.leaderboard";
 const PREF_LEADERBOARD_POSITION = "newtabAdSize.leaderboard.position";
 const PREF_BILLBOARD_POSITION = "newtabAdSize.billboard.position";
+const PREF_TRENDING_SEARCH = "trendingSearch.enabled";
+const PREF_TRENDING_SEARCH_SYSTEM = "system.trendingSearch.enabled";
+const PREF_SEARCH_ENGINE = "trendingSearch.defaultSearchEngine";
+const PREF_TRENDING_SEARCH_VARIANT = "trendingSearch.variant";
 const INTERSECTION_RATIO = 0.5;
 const VISIBLE = "visible";
 const VISIBILITY_CHANGE_EVENT = "visibilitychange";
@@ -358,6 +363,12 @@ export class _CardGrid extends React.PureComponent {
     const listFeedSelectedFeed = prefs[PREF_LIST_FEED_SELECTED_FEED];
     const billboardEnabled = prefs[PREF_BILLBOARD_ENABLED];
     const leaderboardEnabled = prefs[PREF_LEADERBOARD_ENABLED];
+    const trendingEnabled =
+      prefs[PREF_TRENDING_SEARCH] &&
+      prefs[PREF_TRENDING_SEARCH_SYSTEM] &&
+      prefs[PREF_SEARCH_ENGINE]?.toLowerCase() === "google";
+    const trendingVariant = prefs[PREF_TRENDING_SEARCH_VARIANT];
+
     // filter out recs that should be in ListFeed
     const recs = this.props.data.recommendations
       .filter(item => !item.feedName)
@@ -375,7 +386,7 @@ export class _CardGrid extends React.PureComponent {
           rec.placeholder ||
           (rec.flight_id &&
             !spocsStartupCacheEnabled &&
-            this.props.App.isForStartupCache) ? (
+            this.props.App.isForStartupCache.DiscoveryStream) ? (
           <PlaceholderDSCard key={`dscard-${index}`} />
         ) : (
           <DSCard
@@ -389,10 +400,11 @@ export class _CardGrid extends React.PureComponent {
             time_to_read={rec.time_to_read}
             title={rec.title}
             topic={rec.topic}
+            features={rec.features}
             showTopics={showTopics}
             selectedTopics={selectedTopics}
-            availableTopics={availableTopics}
             excerpt={rec.excerpt}
+            availableTopics={availableTopics}
             url={rec.url}
             id={rec.id}
             shim={rec.shim}
@@ -422,6 +434,7 @@ export class _CardGrid extends React.PureComponent {
             received_rank={rec.received_rank}
             format={rec.format}
             alt_text={rec.alt_text}
+            isTimeSensitive={rec.isTimeSensitive}
           />
         )
       );
@@ -476,10 +489,22 @@ export class _CardGrid extends React.PureComponent {
         );
       }
     }
+    if (trendingEnabled && trendingVariant === "b") {
+      const firstSpocPosition = this.props.spocPositions[0]?.index;
+      // double check that a spoc/mrec is actually in the index it should be in
+      const format = cards[firstSpocPosition]?.props?.format;
+      const isSpoc = format === "spoc" || format === "rectangle";
+      // if the spoc is not in its position, place TrendingSearches in the 3rd position
+      cards.splice(isSpoc ? firstSpocPosition + 1 : 2, 1, <TrendingSearches />);
+    }
 
     // if a banner ad is enabled and we have any available, place them in the grid
     const { spocs } = this.props.DiscoveryStream;
-    if ((billboardEnabled || leaderboardEnabled) && spocs.data.newtab_spocs) {
+
+    if (
+      (billboardEnabled || leaderboardEnabled) &&
+      spocs?.data?.newtab_spocs?.items
+    ) {
       // Only render one AdBanner in the grid -
       // Prioritize rendering a leaderboard if it exists,
       // otherwise render a billboard
@@ -523,6 +548,7 @@ export class _CardGrid extends React.PureComponent {
               type={this.props.type}
               firstVisibleTimestamp={this.props.firstVisibleTimestamp}
               row={row}
+              prefs={prefs}
             />
           );
         };
@@ -617,7 +643,6 @@ export class _CardGrid extends React.PureComponent {
   }
 
   renderGridClassName() {
-    const prefs = this.props.Prefs.values;
     const {
       hybridLayout,
       hideCardBackground,
@@ -626,19 +651,6 @@ export class _CardGrid extends React.PureComponent {
       hideDescriptions,
     } = this.props;
 
-    const adSizingVariantAEnabled = prefs["newtabAdSize.variant-a"];
-    const adSizingVariantBEnabled = prefs["newtabAdSize.variant-b"];
-    const adSizingVariantEnabled =
-      adSizingVariantAEnabled || adSizingVariantBEnabled;
-
-    let adSizingVariantClassName = "";
-    if (adSizingVariantEnabled) {
-      // Ad sizing experiment variant, we want to ensure only 1 of these is ever enabled.
-      adSizingVariantClassName = adSizingVariantAEnabled
-        ? `ad-sizing-variant-a`
-        : `ad-sizing-variant-b`;
-    }
-
     const hideCardBackgroundClass = hideCardBackground
       ? `ds-card-grid-hide-background`
       : ``;
@@ -653,7 +665,7 @@ export class _CardGrid extends React.PureComponent {
       ? `ds-card-grid-hybrid-layout`
       : ``;
 
-    const gridClassName = `ds-card-grid ${hybridLayoutClassName} ${hideCardBackgroundClass} ${fourCardLayoutClass} ${hideDescriptionsClassName} ${compactGridClassName} ${adSizingVariantClassName}`;
+    const gridClassName = `ds-card-grid ${hybridLayoutClassName} ${hideCardBackgroundClass} ${fourCardLayoutClass} ${hideDescriptionsClassName} ${compactGridClassName}`;
     return gridClassName;
   }
 
