@@ -15,10 +15,12 @@ import { actionCreators as ac } from "common/Actions.mjs";
 import { DSLinkMenu } from "content-src/components/DiscoveryStreamComponents/DSLinkMenu/DSLinkMenu";
 import { DSImage } from "content-src/components/DiscoveryStreamComponents/DSImage/DSImage";
 import React from "react";
-import { INITIAL_STATE } from "common/Reducers.sys.mjs";
+import { INITIAL_STATE, reducers } from "common/Reducers.sys.mjs";
 import { SafeAnchor } from "content-src/components/DiscoveryStreamComponents/SafeAnchor/SafeAnchor";
 import { shallow, mount } from "enzyme";
 import { FluentOrText } from "content-src/components/FluentOrText/FluentOrText";
+import { Provider } from "react-redux";
+import { combineReducers, createStore } from "redux";
 
 const DEFAULT_PROPS = {
   url: "about:robots",
@@ -83,16 +85,34 @@ describe("<DSCard>", () => {
   });
 
   it("should render badges for pocket, bookmark when not a spoc element ", () => {
-    wrapper = mount(<DSCard context_type="bookmark" {...DEFAULT_PROPS} />);
-    wrapper.setState({ isSeen: true });
-    const contextFooter = wrapper.find(DSContextFooter);
+    const store = createStore(combineReducers(reducers), INITIAL_STATE);
 
+    wrapper = mount(
+      <Provider store={store}>
+        <DSCard context_type="bookmark" {...DEFAULT_PROPS} />
+      </Provider>
+    );
+
+    const dsCardInstance = wrapper.find(DSCard).instance();
+    dsCardInstance.setState({ isSeen: true });
+    wrapper.update();
+
+    const contextFooter = wrapper.find(DSContextFooter);
     assert.lengthOf(contextFooter.find(StatusMessage), 1);
   });
 
   it("should render thumbs up/down UI when not a spoc element ", () => {
-    wrapper = mount(<DSCard mayHaveThumbsUpDown={true} {...DEFAULT_PROPS} />);
-    wrapper.setState({ isSeen: true });
+    const store = createStore(combineReducers(reducers), INITIAL_STATE);
+    wrapper = mount(
+      <Provider store={store}>
+        <DSCard mayHaveThumbsUpDown={true} {...DEFAULT_PROPS} />
+      </Provider>
+    );
+
+    const dsCardInstance = wrapper.find(DSCard).instance();
+    dsCardInstance.setState({ isSeen: true });
+    wrapper.update();
+
     const thumbs_up_down_buttons_component = wrapper.find(
       DSThumbsUpDownButtons
     );
@@ -100,8 +120,17 @@ describe("<DSCard>", () => {
   });
 
   it("thumbs up button should have active class when isThumbsUpActive is true", () => {
-    wrapper = mount(<DSCard mayHaveThumbsUpDown={true} {...DEFAULT_PROPS} />);
-    wrapper.setState({ isSeen: true, isThumbsUpActive: true });
+    const store = createStore(combineReducers(reducers), INITIAL_STATE);
+    wrapper = mount(
+      <Provider store={store}>
+        <DSCard mayHaveThumbsUpDown={true} {...DEFAULT_PROPS} />
+      </Provider>
+    );
+
+    const dsCardInstance = wrapper.find(DSCard).instance();
+    dsCardInstance.setState({ isSeen: true, isThumbsUpActive: true });
+    wrapper.update();
+
     const thumbs_up_down_buttons_component = wrapper.find(
       DSThumbsUpDownButtons
     );
@@ -112,10 +141,20 @@ describe("<DSCard>", () => {
   });
 
   it("should NOT render thumbs up/down UI when a spoc element ", () => {
+    const store = createStore(combineReducers(reducers), INITIAL_STATE);
+
     wrapper = mount(
-      <DSCard mayHaveThumbsUpDown={true} sponsor="Mozilla" {...DEFAULT_PROPS} />
+      <Provider store={store}>
+        <DSCard
+          mayHaveThumbsUpDown={true}
+          sponsor="Mozilla"
+          {...DEFAULT_PROPS}
+        />
+      </Provider>
     );
-    wrapper.setState({ isSeen: true });
+    const dsCardInstance = wrapper.find(DSCard).instance();
+    dsCardInstance.setState({ isSeen: true });
+    wrapper.update();
     // Note: The wrapper is still rendered for DSCard height but the contents is not
     const thumbs_up_down_buttons_component = wrapper.find(
       DSThumbsUpDownButtons
@@ -129,10 +168,17 @@ describe("<DSCard>", () => {
   it("should render Sponsored Context for a spoc element", () => {
     // eslint-disable-next-line no-shadow
     const context = "Sponsored by Foo";
+    const store = createStore(combineReducers(reducers), INITIAL_STATE);
     wrapper = mount(
-      <DSCard context_type="bookmark" context={context} {...DEFAULT_PROPS} />
+      <Provider store={store}>
+        <DSCard context_type="bookmark" context={context} {...DEFAULT_PROPS} />
+      </Provider>
     );
-    wrapper.setState({ isSeen: true });
+
+    const dsCardInstance = wrapper.find(DSCard).instance();
+    dsCardInstance.setState({ isSeen: true });
+    wrapper.update();
+
     const contextFooter = wrapper.find(DSContextFooter);
 
     assert.lengthOf(contextFooter.find(StatusMessage), 0);
@@ -140,19 +186,25 @@ describe("<DSCard>", () => {
   });
 
   it("should render time to read", () => {
+    const store = createStore(combineReducers(reducers), INITIAL_STATE);
     const discoveryStream = {
       ...INITIAL_STATE.DiscoveryStream,
       readTime: true,
     };
     wrapper = mount(
-      <DSCard
-        time_to_read={4}
-        {...DEFAULT_PROPS}
-        DiscoveryStream={discoveryStream}
-        Prefs={INITIAL_STATE.Prefs}
-      />
+      <Provider store={store}>
+        <DSCard
+          time_to_read={4}
+          {...DEFAULT_PROPS}
+          DiscoveryStream={discoveryStream}
+          Prefs={INITIAL_STATE.Prefs}
+        />
+      </Provider>
     );
-    wrapper.setState({ isSeen: true });
+    const dsCardInstance = wrapper.find(DSCard).instance();
+    dsCardInstance.setState({ isSeen: true });
+    wrapper.update();
+
     const defaultMeta = wrapper.find(DefaultMeta);
     assert.lengthOf(defaultMeta, 1);
     assert.equal(defaultMeta.props().timeToRead, 4);
@@ -230,7 +282,7 @@ describe("<DSCard>", () => {
         innerWidth: 1000,
         innerHeight: 900,
       };
-      wrapper = mount(
+      wrapper = shallow(
         <DSCard {...DEFAULT_PROPS} dispatch={dispatch} windowObj={fakeWindow} />
       );
     });
@@ -252,6 +304,7 @@ describe("<DSCard>", () => {
           source: "FOO",
           action_position: 1,
           value: {
+            event_source: "card",
             card_type: "organic",
             recommendation_id: undefined,
             tile_id: "fooidx",
@@ -262,9 +315,11 @@ describe("<DSCard>", () => {
             recommended_at: undefined,
             received_rank: undefined,
             topic: undefined,
+            features: undefined,
             matches_selected_topic: undefined,
             selected_topics: undefined,
             is_list_card: undefined,
+            format: "medium-card",
           },
         })
       );
@@ -282,6 +337,7 @@ describe("<DSCard>", () => {
               topic: undefined,
               selected_topics: undefined,
               is_list_card: undefined,
+              format: "medium-card",
             },
           ],
           window_inner_width: 1000,
@@ -291,7 +347,13 @@ describe("<DSCard>", () => {
     });
 
     it("should set the right card_type on spocs", () => {
-      wrapper.setProps({ id: "fooidx", pos: 1, type: "foo", flightId: 12345 });
+      wrapper.setProps({
+        id: "fooidx",
+        pos: 1,
+        type: "foo",
+        flightId: 12345,
+        format: "spoc",
+      });
       sandbox
         .stub(wrapper.instance(), "doesLinkTopicMatchSelectedTopic")
         .returns(undefined);
@@ -305,6 +367,7 @@ describe("<DSCard>", () => {
           source: "FOO",
           action_position: 1,
           value: {
+            event_source: "card",
             card_type: "spoc",
             recommendation_id: undefined,
             tile_id: "fooidx",
@@ -315,9 +378,11 @@ describe("<DSCard>", () => {
             recommended_at: undefined,
             received_rank: undefined,
             topic: undefined,
+            features: undefined,
             matches_selected_topic: undefined,
             selected_topics: undefined,
             is_list_card: undefined,
+            format: "spoc",
           },
         })
       );
@@ -335,6 +400,7 @@ describe("<DSCard>", () => {
               topic: undefined,
               selected_topics: undefined,
               is_list_card: undefined,
+              format: "spoc",
             },
           ],
           window_inner_width: 1000,
@@ -366,6 +432,7 @@ describe("<DSCard>", () => {
           source: "FOO",
           action_position: 1,
           value: {
+            event_source: "card",
             card_type: "organic",
             recommendation_id: undefined,
             tile_id: "fooidx",
@@ -377,9 +444,11 @@ describe("<DSCard>", () => {
             recommended_at: undefined,
             received_rank: undefined,
             topic: undefined,
+            features: undefined,
             matches_selected_topic: undefined,
             selected_topics: undefined,
             is_list_card: undefined,
+            format: "medium-card",
           },
         })
       );
@@ -398,6 +467,7 @@ describe("<DSCard>", () => {
               topic: undefined,
               selected_topics: undefined,
               is_list_card: undefined,
+              format: "medium-card",
             },
           ],
           window_inner_width: 1000,
@@ -436,8 +506,15 @@ describe("<DSCard>", () => {
 
   describe("DSCard with CTA", () => {
     beforeEach(() => {
-      wrapper = mount(<DSCard {...DEFAULT_PROPS} />);
-      wrapper.setState({ isSeen: true });
+      const store = createStore(combineReducers(reducers), INITIAL_STATE);
+      wrapper = mount(
+        <Provider store={store}>
+          <DSCard {...DEFAULT_PROPS} />
+        </Provider>
+      );
+      const dsCardInstance = wrapper.find(DSCard).instance();
+      dsCardInstance.setState({ isSeen: true });
+      wrapper.update();
     });
 
     it("should render Default Meta", () => {
@@ -474,14 +551,21 @@ describe("<DSCard>", () => {
       assert.calledWith(wrapper.instance().observer.unobserve, "element");
     });
 
-    it("should setup proper placholder ref for isSeen", () => {
+    it("should setup proper placeholder ref for isSeen", () => {
       wrapper.instance().setPlaceholderRef("element");
       assert.equal(wrapper.instance().placeholderElement, "element");
     });
 
     it("should setup observer on componentDidMount", () => {
-      wrapper = mount(<DSCard {...DEFAULT_PROPS} />);
-      assert.isTrue(!!wrapper.instance().observer);
+      const store = createStore(combineReducers(reducers), INITIAL_STATE);
+
+      wrapper = mount(
+        <Provider store={store}>
+          <DSCard {...DEFAULT_PROPS} />
+        </Provider>
+      );
+
+      assert.isTrue(!!wrapper.find(DSCard).instance().observer);
     });
   });
 
@@ -508,16 +592,24 @@ describe("<DSCard>", () => {
     beforeEach(() => {
       const props = {
         App: {
-          isForStartupCache: true,
+          isForStartupCache: {
+            App: true,
+          },
         },
         DiscoveryStream: INITIAL_STATE.DiscoveryStream,
         Prefs: INITIAL_STATE.Prefs,
       };
-      wrapper = mount(<DSCard {...props} />);
+      const store = createStore(combineReducers(reducers), INITIAL_STATE);
+      wrapper = mount(
+        <Provider store={store}>
+          <DSCard {...props} />
+        </Provider>
+      );
     });
 
     it("should be set as isSeen automatically", () => {
-      assert.isTrue(wrapper.instance().state.isSeen);
+      const dsCardInstance = wrapper.find(DSCard).instance();
+      assert.isTrue(dsCardInstance.state.isSeen);
     });
   });
 
@@ -627,8 +719,16 @@ describe("<DSCard>", () => {
         requestIdleCallback: sinon.stub().returns(1),
         cancelIdleCallback: sinon.stub(),
       };
-      wrapper = mount(<DSCard {...DEFAULT_PROPS} windowObj={fakeWindow} />);
-      wrapper.setState({ isSeen: true });
+      const store = createStore(combineReducers(reducers), INITIAL_STATE);
+
+      wrapper = mount(
+        <Provider store={store}>
+          <DSCard {...DEFAULT_PROPS} windowObj={fakeWindow} />
+        </Provider>
+      );
+      const dsCardInstance = wrapper.find(DSCard).instance();
+      dsCardInstance.setState({ isSeen: true });
+      wrapper.update();
       cardNode = wrapper.getDOMNode();
     });
 
@@ -636,56 +736,49 @@ describe("<DSCard>", () => {
       // Add active class name to DSCard wrapper
       // to simulate menu open state
       cardNode.classList.add("active");
-      assert.equal(
-        cardNode.className,
-        "ds-card ds-card-title-lines-3 ds-card-desc-lines-3 active"
-      );
+      assert.include(cardNode.className, "active");
 
-      wrapper.instance().onMenuUpdate(false);
+      const dsCardInstance = wrapper.find(DSCard).instance();
+      dsCardInstance.onMenuUpdate(false);
       wrapper.update();
 
-      assert.equal(
-        cardNode.className,
-        "ds-card ds-card-title-lines-3 ds-card-desc-lines-3"
-      );
+      assert.notInclude(cardNode.className, "active");
     });
 
     it("Should add active on Menu Show", async () => {
-      await wrapper.instance().onMenuShow();
+      const dsCardInstance = wrapper.find(DSCard).instance();
+      await dsCardInstance.onMenuShow();
       wrapper.update();
-      assert.equal(
-        cardNode.className,
-        "ds-card ds-card-title-lines-3 ds-card-desc-lines-3 active"
-      );
+      assert.include(cardNode.className, "active");
     });
 
     it("Should add last-item to support resized window", async () => {
       fakeWindow.scrollMaxX = 20;
-      await wrapper.instance().onMenuShow();
+      const dsCardInstance = wrapper.find(DSCard).instance();
+      await dsCardInstance.onMenuShow();
       wrapper.update();
-      assert.equal(
-        cardNode.className,
-        "ds-card ds-card-title-lines-3 ds-card-desc-lines-3 last-item active"
-      );
+      assert.include(cardNode.className, "last-item");
+      assert.include(cardNode.className, "active");
     });
 
     it("should remove .active and .last-item classes", () => {
-      const instance = wrapper.instance();
+      const dsCardInstance = wrapper.find(DSCard).instance();
+
       const remove = sinon.stub();
-      instance.contextMenuButtonHostElement = {
+      dsCardInstance.contextMenuButtonHostElement = {
         classList: { remove },
       };
-      instance.onMenuUpdate();
+      dsCardInstance.onMenuUpdate();
       assert.calledOnce(remove);
     });
 
     it("should add .active and .last-item classes", async () => {
-      const instance = wrapper.instance();
+      const dsCardInstance = wrapper.find(DSCard).instance();
       const add = sinon.stub();
-      instance.contextMenuButtonHostElement = {
+      dsCardInstance.contextMenuButtonHostElement = {
         classList: { add },
       };
-      await instance.onMenuShow();
+      await dsCardInstance.onMenuShow();
       assert.calledOnce(add);
     });
   });
@@ -746,14 +839,6 @@ describe("Listfeed <DSCard />", () => {
     sandbox.restore();
   });
 
-  it("should not show save to pocket UI", () => {
-    wrapper.setState({ saveToPocketCard: true });
-
-    let stpButton = wrapper.find(".card-stp-button");
-
-    assert.ok(!stpButton.exists());
-  });
-
   it("should not render thumbs up/down UI", () => {
     wrapper.setState({ mayHaveThumbsUpDown: true });
     const thumbs_up_down_buttons_component = wrapper.find(
