@@ -11,7 +11,7 @@
 #
 # - No cyclic dependencies.
 #
-# - No normal header should #include a inlines.h/-inl.h file.
+# - No normal header should #include a -inl.h file.
 #
 # - #ifndef wrappers should have the right form. (XXX: not yet implemented)
 #   - Every header file should have one.
@@ -58,11 +58,19 @@ included_inclnames_to_ignore = set(
     [
         "ffi.h",  # generated in ctypes/libffi/
         "devtools/Instruments.h",  # we ignore devtools/ in general
+        "diplomat_runtime.hpp",  # ICU4X
         "double-conversion/double-conversion.h",  # strange MFBT case
         "javascript-trace.h",  # generated in $OBJDIR if HAVE_DTRACE is defined
         "frontend/ReservedWordsGenerated.h",  # generated in $OBJDIR
         "gc/StatsPhasesGenerated.h",  # generated in $OBJDIR
         "gc/StatsPhasesGenerated.inc",  # generated in $OBJDIR
+        "icu4x/Calendar.hpp",  # ICU4X
+        "icu4x/Date.hpp",  # ICU4X
+        "icu4x/GraphemeClusterSegmenter.hpp",  # ICU4X
+        "icu4x/IsoDate.hpp",  # ICU4X
+        "icu4x/Locale.hpp",  # ICU4X
+        "icu4x/SentenceSegmenter.hpp",  # ICU4X
+        "icu4x/WordSegmenter.hpp",  # ICU4X
         "jit/ABIFunctionTypeGenerated.h",  # generated in $OBJDIR"
         "jit/AtomicOperationsGenerated.h",  # generated in $OBJDIR
         "jit/CacheIROpsGenerated.h",  # generated in $OBJDIR
@@ -77,21 +85,8 @@ included_inclnames_to_ignore = set(
         "fdlibm.h",  # fdlibm
         "FuzzerDefs.h",  # included without a path
         "FuzzingInterface.h",  # included without a path
-        "diplomat_runtime.h",  # ICU4X
-        "ICU4XAnyCalendarKind.h",  # ICU4X
-        "ICU4XCalendar.h",  # ICU4X
-        "ICU4XDate.h",  # ICU4X
-        "ICU4XError.h",  # ICU4X
-        "ICU4XGraphemeClusterSegmenter.h",  # ICU4X
-        "ICU4XIsoDate.h",  # ICU4X
-        "ICU4XIsoWeekday.h",  # ICU4X
-        "ICU4XSentenceSegmenter.h",  # ICU4X
-        "ICU4XWeekCalculator.h",  # ICU4X
-        "ICU4XWeekOf.h",  # ICU4X
-        "ICU4XWeekRelativeUnit.h",  # ICU4X
-        "ICU4XWordSegmenter.h",  # ICU4X
         "mozmemory.h",  # included without a path
-        "mozmemory_utils.h",  # included without a path
+        "mozmemory_stall.h",  # included without a path
         "pratom.h",  # NSPR
         "prcvar.h",  # NSPR
         "prerror.h",  # NSPR
@@ -114,6 +109,7 @@ included_inclnames_to_ignore = set(
         "wasm/WasmBuiltinModuleGenerated.h",  # generated in $OBJDIR"
         "zydis/ZydisAPI.h",  # Zydis
         "xsum/xsum.h",  # xsum
+        "fmt/format.h",  # {fmt} main header
     ]
 )
 
@@ -227,8 +223,6 @@ js/src/tests/style/BadIncludesOrder-inl.h:28:29: error:
       -> tests/style/HeaderCycleB3-inl.h
          -> tests/style/HeaderCycleB4-inl.h
             -> tests/style/HeaderCycleB1-inl.h
-            -> tests/style/jsheadercycleB5inlines.h
-               -> tests/style/HeaderCycleB1-inl.h
       -> tests/style/HeaderCycleB4-inl.h
 
 """.splitlines(
@@ -253,7 +247,7 @@ def error(filename, linenum, *lines):
     out("")
 
 
-class FileKind(object):
+class FileKind:
     C = 1
     CPP = 2
     INL_H = 3
@@ -269,10 +263,10 @@ class FileKind(object):
         if filename.endswith(".cpp"):
             return FileKind.CPP
 
-        if filename.endswith(("inlines.h", "-inl.h")):
+        if filename.endswith("-inl.h"):
             return FileKind.INL_H
 
-        if filename.endswith(".h"):
+        if filename.endswith((".h", ".hpp")):
             return FileKind.H
 
         if filename.endswith(".tbl"):
@@ -398,13 +392,10 @@ def check_style(enable_fixup):
 
 
 def module_name(name):
-    """Strip the trailing .cpp, .h, inlines.h or -inl.h from a filename."""
+    """Strip the trailing .cpp, .h, or -inl.h from a filename."""
 
     return (
-        name.replace("inlines.h", "")
-        .replace("-inl.h", "")
-        .replace(".h", "")
-        .replace(".cpp", "")
+        name.replace("-inl.h", "").replace(".h", "").replace(".cpp", "")
     )  # NOQA: E501
 
 
@@ -435,7 +426,7 @@ def is_module_header(enclosing_inclname, header_inclname):
     return False
 
 
-class Include(object):
+class Include:
     """Important information for a single #include statement."""
 
     def __init__(self, include_prefix, inclname, line_suffix, linenum, is_system):
@@ -454,21 +445,20 @@ class Include(object):
         """Identify which section inclname belongs to.
 
         The section numbers are as follows.
-          0. Module header (e.g. jsfoo.h or jsfooinlines.h within jsfoo.cpp)
+          0. Module header (e.g. jsfoo.h or jsfoo-inl.h within jsfoo.cpp)
           1. mozilla/Foo.h
           2. <foo.h> or <foo>
           3. jsfoo.h, prmjtime.h, etc
           4. foo/Bar.h
-          5. jsfooinlines.h
-          6. foo/Bar-inl.h
-          7. non-.h, e.g. *.tbl, *.msg (these can be scattered throughout files)
+          5. foo/Bar-inl.h
+          6. non-.h, e.g. *.tbl, *.msg (these can be scattered throughout files)
         """
 
         if self.is_system:
             return 2
 
-        if not self.inclname.endswith(".h"):
-            return 7
+        if not self.inclname.endswith((".h", ".hpp")):
+            return 6
 
         # A couple of modules have the .h file in js/ and the .cpp file elsewhere and so need
         # special handling.
@@ -480,13 +470,10 @@ class Include(object):
                 return 1
 
             if self.inclname.endswith("-inl.h"):
-                return 6
+                return 5
 
             return 4
 
-        if self.inclname.endswith("inlines.h"):
-            return 5
-
         return 3
 
     def quote(self):
@@ -502,7 +489,7 @@ class Include(object):
         return self.include_prefix + self.quote() + self.line_suffix + "\n"
 
 
-class CppBlock(object):
+class CppBlock:
     """C preprocessor block: a whole file or a single #if/#elif/#else block.
 
     A #if/#endif block is the contents of a #if/#endif (or similar) section.
@@ -619,7 +606,7 @@ class CppBlock(object):
         return self.start + "".join(kid.to_source() for kid in self.kids) + self.end
 
 
-class OrdinaryCode(object):
+class OrdinaryCode:
     """A list of lines of code that aren't #include/#if/#else/#endif lines."""
 
     def __init__(self, lines=None):
