@@ -49,11 +49,6 @@ loader.lazyRequireGetter(
   "HighlightersOverlay",
   "resource://devtools/client/inspector/shared/highlighters-overlay.js"
 );
-loader.lazyRequireGetter(
-  this,
-  "ExtensionSidebar",
-  "resource://devtools/client/inspector/extensions/extension-sidebar.js"
-);
 loader.lazyRequireGetter(
   this,
   "PICKER_TYPES",
@@ -163,6 +158,7 @@ function Inspector(toolbox, commands) {
   this._onTargetSelected = this._onTargetSelected.bind(this);
   this._onWillNavigate = this._onWillNavigate.bind(this);
   this._updateSearchResultsLabel = this._updateSearchResultsLabel.bind(this);
+  this._onSearchLabelClick = this._onSearchLabelClick.bind(this);
 
   this.onDetached = this.onDetached.bind(this);
   this.onHostChanged = this.onHostChanged.bind(this);
@@ -183,7 +179,6 @@ function Inspector(toolbox, commands) {
   this.onSidebarSelect = this.onSidebarSelect.bind(this);
   this.onSidebarShown = this.onSidebarShown.bind(this);
   this.onSidebarToggle = this.onSidebarToggle.bind(this);
-  this.onReflowInSelection = this.onReflowInSelection.bind(this);
   this.listenForSearchEvents = this.listenForSearchEvents.bind(this);
 
   this.prefObserver = new PrefObserver("devtools.");
@@ -199,8 +194,20 @@ Inspector.prototype = {
    * InspectorPanel.open() is effectively an asynchronous constructor.
    * Set any attributes or listeners that rely on the document being loaded or fronts
    * from the InspectorFront and Target here.
+   *
+   * @param {Object} options
+   * @param {NodeFront|undefined} options.defaultStartupNode: Optional node front that
+   *        will be selected when the first root node is available.
+   * @param {ElementIdentifier|undefined} options.defaultStartupNodeDomReference: Optional
+   *        element identifier whose matching node front will be selected when the first
+   *        root node is available.
+   *        Will be ignored if defaultStartupNode is passed.
+   * @param {String|undefined} options.defaultStartupNodeSelectionReason: Optional string
+   *        that will be used as a reason for the node selection when either
+   *        defaultStartupNode or defaultStartupNodeDomReference is passed
+   * @returns {Inspector}
    */
-  async init() {
+  async init(options = {}) {
     // Localize all the nodes containing a data-localization attribute.
     localizeMarkup(this.panelDoc);
 
@@ -219,6 +226,19 @@ Inspector.prototype = {
     // iframe if it had already been initialized.
     this.setupSplitter();
 
+    // Optional NodeFront/ElementIdentifier set on inspector startup, to be selected once the first root
+    // node is available.
+    this._defaultStartupNode = options.defaultStartupNode;
+    this._defaultStartupNodeDomReference =
+      options.defaultStartupNodeDomReference;
+    this._defaultStartupNodeSelectionReason =
+      options.defaultStartupNodeSelectionReason;
+
+    // NodeFront for the DOM Element selected when opening the inspector, or after each
+    // navigation (i.e. each time a new Root Node is available)
+    // This is used as a fallback if the currently selected node is removed.
+    this._defaultNode = null;
+
     await this.commands.targetCommand.watchTargets({
       types: [this.commands.targetCommand.TYPES.FRAME],
       onAvailable: this._onTargetAvailable,
@@ -231,6 +251,7 @@ Inspector.prototype = {
       // To observe CSS change before opening changes view.
       TYPES.CSS_CHANGE,
       TYPES.DOCUMENT_EVENT,
+      TYPES.REFLOW,
     ];
     // The root node is retrieved from onTargetSelected which is now called
     // on startup as well as on any navigation (= new top level target).
@@ -357,6 +378,16 @@ Inspector.prototype = {
       ) {
         this._onWillNavigate();
       }
+
+      if (resource.resourceType === this.toolbox.resourceCommand.TYPES.REFLOW) {
+        this.emit("reflow");
+        if (resource.targetFront === this.selection?.nodeFront?.targetFront) {
+          // This event will be fired whenever a reflow is detected in the target front of the
+          // selected node front (so when a reflow is detected inside any of the windows that
+          // belong to the BrowsingContext where the currently selected node lives).
+          this.emit("reflow-in-selected-target");
+        }
+      }
     }
 
     return Promise.all(rootNodeAvailablePromises);
@@ -369,7 +400,6 @@ Inspector.prototype = {
     // Record new-root timing for telemetry
     this._newRootStart = this.panelWin.performance.now();
 
-    this._defaultNode = null;
     this.selection.setNodeFront(null);
     this._destroyMarkup();
 
@@ -380,8 +410,11 @@ Inspector.prototype = {
       }
 
       this.selection.setNodeFront(defaultNode, {
-        reason: "inspector-default-selection",
+        reason:
+          this._defaultStartupNodeSelectionReason ??
+          "inspector-default-selection",
       });
+      this._defaultStartupNodeSelectionReason = null;
 
       await this._initMarkupView();
 
@@ -449,8 +482,9 @@ Inspector.prototype = {
       // Only log the timing when inspector is not destroyed and is in foreground.
       if (this.toolbox && this.toolbox.currentToolId == "inspector") {
         const delay = this.panelWin.performance.now() - this._newRootStart;
-        const telemetryKey = "DEVTOOLS_INSPECTOR_NEW_ROOT_TO_RELOAD_DELAY_MS";
-        this.telemetry.getHistogramById(telemetryKey).add(delay);
+        Glean.devtoolsInspector.newRootToReloadDelay.accumulateSingleSample(
+          delay
+        );
       }
       delete this._newRootStart;
     }
@@ -518,7 +552,9 @@ Inspector.prototype = {
       this._search = new InspectorSearch(
         this,
         this.searchBox,
-        this.searchClearButton
+        this.searchClearButton,
+        this.searchPrevButton,
+        this.searchNextButton
       );
     }
 
@@ -588,19 +624,48 @@ Inspector.prototype = {
    *        The current root node front for the top walker.
    */
   async _getDefaultNodeForSelection(rootNodeFront) {
-    if (this._defaultNode) {
-      return this._defaultNode;
+    let node;
+    if (this._defaultStartupNode) {
+      node = this._defaultStartupNode;
+      this._defaultStartupNode = null;
+      this._defaultStartupNodeDomReference = null;
+      return node;
     }
 
     // Save the _pendingSelectionUnique on the current inspector instance.
     const pendingSelectionUnique = Symbol("pending-selection");
     this._pendingSelectionUnique = pendingSelectionUnique;
 
+    if (this._defaultStartupNodeDomReference) {
+      const domReference = this._defaultStartupNodeDomReference;
+      // nullify before calling the async getNodeActorFromContentDomReference so calls
+      // made to getDefaultNodeForSelection while the promise is pending will be properly
+      // ignored with the check on pendingSelectionUnique
+      this._defaultStartupNode = null;
+      this._defaultStartupNodeDomReference = null;
+
+      try {
+        node =
+          await this.inspectorFront.getNodeActorFromContentDomReference(
+            domReference
+          );
+      } catch (e) {
+        console.warn(
+          "Couldn't retrieve node front from dom reference",
+          domReference
+        );
+      }
+    }
+
     if (this._pendingSelectionUnique !== pendingSelectionUnique) {
       // If this method was called again while waiting, bail out.
       return null;
     }
 
+    if (node) {
+      return node;
+    }
+
     const walker = rootNodeFront.walkerFront;
     const cssSelectors = this.selectionCssSelectors;
     // Try to find a default node using three strategies:
@@ -620,7 +685,7 @@ Inspector.prototype = {
 
     // Try all default node selectors until a valid node is found.
     for (const selector of defaultNodeSelectors) {
-      const node = await selector();
+      node = await selector();
       if (this._pendingSelectionUnique !== pendingSelectionUnique) {
         // If this method was called again while waiting, bail out.
         return null;
@@ -653,10 +718,21 @@ Inspector.prototype = {
     this.searchResultsContainer = this.panelDoc.getElementById(
       "inspector-searchlabel-container"
     );
+    this.searchNavigationContainer = this.panelDoc.getElementById(
+      "inspector-searchnavigation-container"
+    );
+    this.searchPrevButton = this.panelDoc.getElementById(
+      "inspector-searchnavigation-button-prev"
+    );
+    this.searchNextButton = this.panelDoc.getElementById(
+      "inspector-searchnavigation-button-next"
+    );
     this.searchResultsLabel = this.panelDoc.getElementById(
       "inspector-searchlabel"
     );
 
+    this.searchResultsLabel.addEventListener("click", this._onSearchLabelClick);
+
     this.searchBox.addEventListener("focus", this.listenForSearchEvents, {
       once: true,
     });
@@ -664,6 +740,12 @@ Inspector.prototype = {
     this.createSearchBoxShortcuts();
   },
 
+  _onSearchLabelClick() {
+    // Focus on the search box as the search label
+    // appears to be "inside" input
+    this.searchBox.focus();
+  },
+
   listenForSearchEvents() {
     this.search.on("search-cleared", this._clearSearchResultsLabel);
     this.search.on("search-result", this._updateSearchResultsLabel);
@@ -703,6 +785,10 @@ Inspector.prototype = {
   },
 
   _clearSearchResultsLabel(result) {
+    // Pipe the search-cleared event as this.search is a getter that will create
+    // the InspectorSearch instance, which we don't really need/want when a callsite
+    // only want to react to the search being cleared.
+    this.emit("search-cleared");
     return this._updateSearchResultsLabel(result, true);
   },
 
@@ -715,8 +801,10 @@ Inspector.prototype = {
           result.resultsIndex + 1,
           result.resultsLength
         );
+        this.searchNavigationContainer.hidden = false;
       } else {
         str = INSPECTOR_L10N.getStr("inspector.searchResultsNone");
+        this.searchNavigationContainer.hidden = true;
       }
 
       this.searchResultsContainer.hidden = false;
@@ -1297,6 +1385,11 @@ Inspector.prototype = {
       );
     }
 
+    // Load the ExtensionSidebar component via the Browser Loader as it ultimately loads Reps and Object Inspector,
+    // which are expected to be loaded in a document scope.
+    const ExtensionSidebar = this.browserRequire(
+      "resource://devtools/client/inspector/extensions/extension-sidebar.js"
+    );
     const extensionSidebar = new ExtensionSidebar(this, { id, title });
 
     // TODO(rpl): pass some extension metadata (e.g. extension name and icon) to customize
@@ -1326,6 +1419,9 @@ Inspector.prototype = {
 
     const panel = this._panels.get(id);
 
+    const ExtensionSidebar = this.browserRequire(
+      "resource://devtools/client/inspector/extensions/extension-sidebar.js"
+    );
     if (!(panel instanceof ExtensionSidebar)) {
       throw new Error(
         `The sidebar panel with id "${id}" is not an ExtensionSidebar`
@@ -1547,7 +1643,6 @@ Inspector.prototype = {
 
     this.updateAddElementButton();
     this.updateSelectionCssSelectors();
-    this.trackReflowsInSelection();
 
     const selfUpdate = this.updating("inspector-panel");
     executeSoon(() => {
@@ -1560,56 +1655,6 @@ Inspector.prototype = {
     });
   },
 
-  /**
-   * Starts listening for reflows in the targetFront of the currently selected nodeFront.
-   */
-  async trackReflowsInSelection() {
-    this.untrackReflowsInSelection();
-    if (!this.selection.nodeFront) {
-      return;
-    }
-
-    if (this._destroyed) {
-      return;
-    }
-
-    try {
-      await this.commands.resourceCommand.watchResources(
-        [this.commands.resourceCommand.TYPES.REFLOW],
-        {
-          onAvailable: this.onReflowInSelection,
-        }
-      );
-    } catch (e) {
-      // it can happen that watchResources fails as the client closes while we're processing
-      // some asynchronous call.
-      // In order to still get valid exceptions, we re-throw the exception if the inspector
-      // isn't destroyed.
-      if (!this._destroyed) {
-        throw e;
-      }
-    }
-  },
-
-  /**
-   * Stops listening for reflows.
-   */
-  untrackReflowsInSelection() {
-    this.commands.resourceCommand.unwatchResources(
-      [this.commands.resourceCommand.TYPES.REFLOW],
-      {
-        onAvailable: this.onReflowInSelection,
-      }
-    );
-  },
-
-  onReflowInSelection() {
-    // This event will be fired whenever a reflow is detected in the target front of the
-    // selected node front (so when a reflow is detected inside any of the windows that
-    // belong to the BrowsingContext when the currently selected node lives).
-    this.emit("reflow-in-selected-target");
-  },
-
   /**
    * Delay the "inspector-updated" notification while a tool
    * is updating itself.  Returns a function that must be
@@ -1747,7 +1792,6 @@ Inspector.prototype = {
     resourceCommand.unwatchResources(this._watchedResources, {
       onAvailable: this.onResourceAvailable,
     });
-    this.untrackReflowsInSelection();
 
     this._InspectorTabPanel = null;
     this._TabBar = null;
@@ -1780,6 +1824,11 @@ Inspector.prototype = {
     this.sidebar = null;
     this.store = null;
     this.telemetry = null;
+    this.searchResultsLabel.removeEventListener(
+      "click",
+      this._onSearchLabelClick
+    );
+    this.searchResultsLabel = null;
   },
 
   _destroyMarkup() {
@@ -1807,7 +1856,9 @@ Inspector.prototype = {
   },
 
   stopEyeDropperListeners() {
-    this.toolbox.tellRDMAboutPickerState(false, PICKER_TYPES.EYEDROPPER);
+    this.toolbox
+      .tellRDMAboutPickerState(false, PICKER_TYPES.EYEDROPPER)
+      .catch(console.error);
     this.inspectorFront.off("color-pick-canceled", this.onEyeDropperDone);
     this.inspectorFront.off("color-picked", this.onEyeDropperDone);
     this.off("new-root", this.onEyeDropperDone);
