@@ -62,6 +62,17 @@ loader.lazyRequireGetter(
   "clipboardHelper",
   "resource://devtools/shared/platform/clipboard.js"
 );
+loader.lazyRequireGetter(
+  this,
+  "openContentLink",
+  "resource://devtools/client/shared/link.js",
+  true
+);
+
+const lazy = {};
+ChromeUtils.defineESModuleGetters(lazy, {
+  AppConstants: "resource://gre/modules/AppConstants.sys.mjs",
+});
 
 const HTML_NS = "http://www.w3.org/1999/xhtml";
 const PREF_UA_STYLES = "devtools.inspector.showUserAgentStyles";
@@ -269,6 +280,7 @@ function CssRuleView(inspector, document, store) {
   this.tooltips = new TooltipsOverlay(this);
 
   this.cssRegisteredPropertiesByTarget = new Map();
+  this._elementsWithPendingClicks = new this.styleWindow.WeakSet();
 }
 
 CssRuleView.prototype = {
@@ -341,31 +353,44 @@ CssRuleView.prototype = {
   },
 
   /**
-   * Highlight/unhighlight all the nodes that match a given selector
+   * Highlight/unhighlight all the nodes that match a given rule's selector
    * inside the document of the current selected node.
    * Only one selector can be highlighted at a time, so calling the method a
-   * second time with a different selector will first unhighlight the previously
+   * second time with a different rule will first unhighlight the previously
    * highlighted nodes.
-   * Calling the method a second time with the same selector will just
+   * Calling the method a second time with the same rule will just
    * unhighlight the highlighted nodes.
    *
+   * @param {Rule} rule
    * @param {String} selector
    *        Elements matching this selector will be highlighted on the page.
+   * @param {Boolean} highlightFromRulesSelector
    */
-  async toggleSelectorHighlighter(selector) {
+  async toggleSelectorHighlighter(
+    rule,
+    selector,
+    highlightFromRulesSelector = true
+  ) {
     if (this.isSelectorHighlighted(selector)) {
       await this.inspector.highlighters.hideHighlighterType(
         this.inspector.highlighters.TYPES.SELECTOR
       );
     } else {
+      const options = {
+        hideInfoBar: true,
+        hideGuides: true,
+        // we still pass the selector (which can be the StyleRuleFront#computedSelector)
+        // even if highlightFromRulesSelector is set to true, as it's how we keep track
+        // of which selector is highlighted.
+        selector,
+      };
+      if (highlightFromRulesSelector) {
+        options.ruleActorID = rule.domRule.actorID;
+      }
       await this.inspector.highlighters.showHighlighterTypeForNode(
         this.inspector.highlighters.TYPES.SELECTOR,
         this.inspector.selection.nodeFront,
-        {
-          hideInfoBar: true,
-          hideGuides: true,
-          selector,
-        }
+        options
       );
     }
   },
@@ -433,12 +458,14 @@ CssRuleView.prototype = {
     if (target.classList.contains("js-toggle-selector-highlighter")) {
       event.stopPropagation();
       let selector = target.dataset.computedSelector;
+      const highlightFromRulesSelector =
+        !!selector && !target.dataset.isUniqueSelector;
       // dataset.computedSelector will be initially empty for inline styles (inherited or not)
       // Rules associated with a regular selector should have this data-attribute
       // set in devtools/client/inspector/rules/views/rule-editor.js
+      const rule = getRuleFromNode(target, this._elementStyle);
       if (selector === "") {
         try {
-          const rule = getRuleFromNode(target, this._elementStyle);
           if (rule.inherited) {
             // This is an inline style from an inherited rule. Need to resolve the
             // unique selector from the node which this rule is inherited from.
@@ -451,12 +478,17 @@ CssRuleView.prototype = {
 
           // Now that the selector was computed, we can store it for subsequent usage.
           target.dataset.computedSelector = selector;
+          target.dataset.isUniqueSelector = true;
         } finally {
           // Could not resolve a unique selector for the inline style.
         }
       }
 
-      this.toggleSelectorHighlighter(selector);
+      this.toggleSelectorHighlighter(
+        rule,
+        selector,
+        highlightFromRulesSelector
+      );
     }
 
     // Handle click on swatches next to flex and inline-flex CSS properties
@@ -476,6 +508,33 @@ CssRuleView.prototype = {
         "rule"
       );
     }
+
+    const valueSpan = target.closest(".ruleview-propertyvalue");
+    if (valueSpan) {
+      if (this._elementsWithPendingClicks.has(valueSpan)) {
+        // When we start handling a drag in the TextPropertyEditor valueSpan,
+        // we make the valueSpan capture the pointer. Then, `click` event target is always
+        // the valueSpan with the latest spec of Pointer Events.
+        // Therefore, we should stop immediate propagation of the `click` event
+        // if we've handled a drag to prevent moving focus to the inplace editor.
+        event.stopImmediatePropagation();
+        return;
+      }
+
+      // Handle link click in RuleEditor property value
+      if (target.nodeName === "a") {
+        event.stopPropagation();
+        event.preventDefault();
+        openContentLink(target.href, {
+          relatedToCurrent: true,
+          inBackground:
+            event.button === 1 ||
+            (lazy.AppConstants.platform === "macosx"
+              ? event.metaKey
+              : event.ctrlKey),
+        });
+      }
+    }
   },
 
   /**
@@ -509,18 +568,18 @@ CssRuleView.prototype = {
         }
         break;
 
-      // Toggle the "active" class on swatches next to flex and inline-flex CSS properties
+      // Toggle the "aria-pressed" attribute on swatches next to flex and inline-flex CSS properties
       // when the FlexboxHighlighter is shown/hidden for the currently selected node.
       case this.inspector.highlighters.TYPES.FLEXBOX:
         {
           const query = ".js-toggle-flexbox-highlighter";
           for (const node of this.styleDocument.querySelectorAll(query)) {
-            node.classList.toggle("active", eventName == "highlighter-shown");
+            node.setAttribute("aria-pressed", eventName == "highlighter-shown");
           }
         }
         break;
 
-      // Toggle the "active" class on swatches next to grid CSS properties
+      // Toggle the "aria-pressed" class on swatches next to grid CSS properties
       // when the GridHighlighter is shown/hidden for the currently selected node.
       case this.inspector.highlighters.TYPES.GRID:
         {
@@ -530,7 +589,10 @@ CssRuleView.prototype = {
             // not currently selected. The Rules view shows `display: grid` declarations
             // only for the selected node. Avoid mistakenly marking them as "active".
             if (data.nodeFront === this.inspector.selection.nodeFront) {
-              node.classList.toggle("active", eventName == "highlighter-shown");
+              node.setAttribute(
+                "aria-pressed",
+                eventName == "highlighter-shown"
+              );
             }
 
             // When the max limit of grid highlighters is reached (default 3),
@@ -569,6 +631,12 @@ CssRuleView.prototype = {
       "click",
       this._onToggleDarkColorSchemeSimulation
     );
+    const { rfpCSSColorScheme } = this.inspector.walker;
+    if (rfpCSSColorScheme) {
+      this.colorSchemeLightSimulationButton.setAttribute("disabled", true);
+      this.colorSchemeDarkSimulationButton.setAttribute("disabled", true);
+      console.warn("Color scheme simulation is disabled in RFP mode.");
+    }
   },
 
   /**
@@ -687,6 +755,9 @@ CssRuleView.prototype = {
     const element = elementStyle.element;
     const pseudoClasses = element.pseudoClassLocks;
 
+    // Clear the search input so the new rule is visible
+    this._onClearSearch();
+
     this._focusNextUserAddedRule = true;
     this.pageStyle.addNewRule(element, pseudoClasses);
   },
@@ -1379,12 +1450,11 @@ CssRuleView.prototype = {
   _createEditors() {
     // Run through the current list of rules, attaching
     // their editors in order.  Create editors if needed.
-    let lastInheritedSource = "";
-    let lastKeyframes = null;
-    let seenPseudoElement = false;
+    let lastInherited = null;
+    let lastinheritedSectionLabel = "";
     let seenNormalElement = false;
     let seenSearchTerm = false;
-    let container = null;
+    const containers = new Map();
 
     if (!this._elementStyle.rules) {
       return Promise.resolve();
@@ -1398,7 +1468,9 @@ CssRuleView.prototype = {
 
       // Initialize rule editor
       if (!rule.editor) {
-        rule.editor = new RuleEditor(this, rule);
+        rule.editor = new RuleEditor(this, rule, {
+          elementsWithPendingClicks: this._elementsWithPendingClicks,
+        });
         editorReadyPromises.push(rule.editor.once("source-link-updated"));
       }
 
@@ -1411,8 +1483,14 @@ CssRuleView.prototype = {
         }
       }
 
+      const isNonInheritedPseudo = !!rule.pseudoElement && !rule.inherited;
+
       // Only print header for this element if there are pseudo elements
-      if (seenPseudoElement && !seenNormalElement && !rule.pseudoElement) {
+      if (
+        containers.has(PSEUDO_ELEMENTS_CONTAINER_ID) &&
+        !seenNormalElement &&
+        !rule.pseudoElement
+      ) {
         seenNormalElement = true;
         const div = this.styleDocument.createElementNS(HTML_NS, "div");
         div.className = RULE_VIEW_HEADER_CLASSNAME;
@@ -1421,8 +1499,20 @@ CssRuleView.prototype = {
         this.element.appendChild(div);
       }
 
-      const inheritedSource = rule.inherited;
-      if (inheritedSource && inheritedSource !== lastInheritedSource) {
+      const { inherited, inheritedSectionLabel } = rule;
+      // We need to check both `inherited` (a NodeFront) and `inheritedSectionLabel` (string),
+      // as element-backed pseudo element rules (e.g. `::details-content`) can have the same
+      // `inherited` property as a regular rule (e.g. on `<details>`), but the element is
+      // to be considered as a child of the binding element.
+      // e.g. we want to have
+      // This element
+      // Inherited by details::details-content
+      // Inherited by details
+      if (
+        inherited &&
+        (inherited !== lastInherited ||
+          inheritedSectionLabel !== lastinheritedSectionLabel)
+      ) {
         const div = this.styleDocument.createElementNS(HTML_NS, "div");
         div.classList.add(
           RULE_VIEW_HEADER_CLASSNAME,
@@ -1430,31 +1520,46 @@ CssRuleView.prototype = {
         );
         div.setAttribute("role", "heading");
         div.setAttribute("aria-level", "3");
-        div.textContent = rule.inheritedSource;
-        lastInheritedSource = inheritedSource;
+        div.textContent = rule.inheritedSectionLabel;
+        lastInherited = inherited;
+        lastinheritedSectionLabel = inheritedSectionLabel;
         this.element.appendChild(div);
       }
 
-      if (!seenPseudoElement && rule.pseudoElement) {
-        seenPseudoElement = true;
-        container = this.createExpandableContainer(
-          this.pseudoElementLabel,
-          PSEUDO_ELEMENTS_CONTAINER_ID,
-          true
-        );
-      }
-
       const keyframes = rule.keyframes;
-      if (keyframes && keyframes !== lastKeyframes) {
-        lastKeyframes = keyframes;
-        container = this.createExpandableContainer(
-          rule.keyframesName,
-          `keyframes-container-${keyframes.name}`
-        );
+
+      let containerKey = null;
+
+      // Don't display inherited pseudo element rules (e.g. ::details-content) inside
+      // the pseudo element container
+      if (isNonInheritedPseudo) {
+        containerKey = PSEUDO_ELEMENTS_CONTAINER_ID;
+        if (!containers.has(containerKey)) {
+          containers.set(
+            containerKey,
+            this.createExpandableContainer(
+              this.pseudoElementLabel,
+              containerKey,
+              true
+            )
+          );
+        }
+      } else if (keyframes) {
+        containerKey = keyframes;
+        if (!containers.has(containerKey)) {
+          containers.set(
+            containerKey,
+            this.createExpandableContainer(
+              rule.keyframesName,
+              `keyframes-container-${keyframes.name}`
+            )
+          );
+        }
       }
 
       rule.editor.element.setAttribute("role", "article");
-      if (container && (rule.pseudoElement || keyframes)) {
+      const container = containers.get(containerKey);
+      if (container) {
         container.appendChild(rule.editor.element);
       } else {
         this.element.appendChild(rule.editor.element);
@@ -1464,7 +1569,7 @@ CssRuleView.prototype = {
       if (this._focusNextUserAddedRule && rule.domRule.userAdded) {
         this._focusNextUserAddedRule = null;
         rule.editor.selectorText.click();
-        this.emitForTests("new-rule-added");
+        this.emitForTests("new-rule-added", rule);
       }
     }
 
@@ -1608,7 +1713,7 @@ CssRuleView.prototype = {
       ? styleSheetSource === this.searchData.strictSearchValue
       : styleSheetSource.includes(this.searchValue);
 
-    if (isStyleSheetHighlighted) {
+    if (isStyleSheetHighlighted && rule.editor.source) {
       rule.editor.source.classList.add("ruleview-highlight");
     }
 
@@ -1697,7 +1802,7 @@ CssRuleView.prototype = {
     let isComputedHighlighted = false;
 
     // Highlight search matches in the computed list of properties
-    editor._populateComputed();
+    editor.populateComputed();
     for (const computed of editor.prop.computed) {
       if (computed.element) {
         // Get the actual property value displayed in the computed list
