@@ -45,6 +45,9 @@ const STYLE_INSPECTOR_PROPERTIES =
 const { LocalizationHelper } = require("resource://devtools/shared/l10n.js");
 const STYLE_INSPECTOR_L10N = new LocalizationHelper(STYLE_INSPECTOR_PROPERTIES);
 
+const COMPONENT_PROPERTIES = "devtools/client/locales/components.properties";
+const COMPONENT_L10N = new LocalizationHelper(COMPONENT_PROPERTIES);
+
 loader.lazyGetter(this, "NEW_PROPERTY_NAME_INPUT_LABEL", function () {
   return STYLE_INSPECTOR_L10N.getStr("rule.newPropertyName.label");
 });
@@ -62,8 +65,10 @@ const INDENT_STR = " ".repeat(INDENT_SIZE);
  *        The CssRuleView containg the document holding this rule editor.
  * @param {Rule} rule
  *        The Rule object we're editing.
+ * @param {Object} options
+ * @param {Set} options.elementsWithPendingClicks
  */
-function RuleEditor(ruleView, rule) {
+function RuleEditor(ruleView, rule, options = {}) {
   EventEmitter.decorate(this);
 
   this.ruleView = ruleView;
@@ -71,6 +76,7 @@ function RuleEditor(ruleView, rule) {
   this.toolbox = this.ruleView.inspector.toolbox;
   this.telemetry = this.toolbox.telemetry;
   this.rule = rule;
+  this.options = options;
 
   this.isEditable = !rule.isSystem;
   // Flag that blocks updates of the selector and properties when it is
@@ -136,16 +142,18 @@ RuleEditor.prototype = {
     // span to be placed absolutely against.
     this.element.style.position = "relative";
 
-    // Add the source link.
-    this.source = createChild(this.element, "div", {
-      class: "ruleview-rule-source theme-link",
-    });
-    this.source.addEventListener("click", this._onSourceClick);
-
-    const sourceLabel = this.doc.createElement("span");
-    sourceLabel.classList.add("ruleview-rule-source-label");
-    this.source.appendChild(sourceLabel);
+    // Add the source link for supported rules. inline style and pres hints are not visible
+    // in the StyleEditor, so don't show anything for such rule.
+    if (this.rule.domRule.type !== ELEMENT_STYLE) {
+      this.source = createChild(this.element, "div", {
+        class: "ruleview-rule-source theme-link",
+      });
+      this.source.addEventListener("click", this._onSourceClick);
 
+      const sourceLabel = this.doc.createElement("a");
+      sourceLabel.classList.add("ruleview-rule-source-label");
+      this.source.appendChild(sourceLabel);
+    }
     this.updateSourceLink();
 
     if (this.rule.domRule.ancestorData.length) {
@@ -182,10 +190,8 @@ RuleEditor.prototype = {
             }`,
           });
 
-          // We can't use a button, otherwise a line break is added when copy/pasting the rule
-          const jumpToNodeButton = createChild(selectorContainer, "span", {
+          const jumpToNodeButton = createChild(selectorContainer, "button", {
             class: "open-inspector",
-            role: "button",
             title: l10n("rule.containerQuery.selectContainerButton.tooltip"),
           });
 
@@ -253,6 +259,18 @@ RuleEditor.prototype = {
           selectorContainer.append(
             this.doc.createTextNode(`@import ${ancestorData.value}`)
           );
+        } else if (ancestorData.type == "scope") {
+          let text = `@scope`;
+          if (ancestorData.start) {
+            text += ` (${ancestorData.start})`;
+
+            if (ancestorData.end) {
+              text += ` to (${ancestorData.end})`;
+            }
+          }
+          selectorContainer.append(this.doc.createTextNode(text));
+        } else if (ancestorData.type == "starting-style") {
+          selectorContainer.append(this.doc.createTextNode(`@starting-style`));
         } else if (ancestorData.selectors) {
           ancestorData.selectors.forEach((selector, i) => {
             if (i !== 0) {
@@ -336,17 +354,16 @@ RuleEditor.prototype = {
     }
 
     if (this.rule.domRule.type !== CSSRule.KEYFRAME_RULE) {
-      let selector = "";
-      let desugaredSelector = "";
+      // This is a "normal" rule with a selector.
+      let computedSelector = "";
       if (this.rule.domRule.selectors) {
-        // This is a "normal" rule with a selector.
-        selector = this.rule.domRule.selectors.join(", ");
-        desugaredSelector = this.rule.domRule.desugaredSelectors?.join(", ");
+        computedSelector = this.rule.domRule.computedSelector;
         // Otherwise, the rule is either inherited or inline, and selectors will
         // be computed on demand when the highlighter is requested.
       }
 
-      const isHighlighted = this.ruleView.isSelectorHighlighted(selector);
+      const isHighlighted =
+        this.ruleView.isSelectorHighlighted(computedSelector);
       // Handling of click events is delegated to CssRuleView.handleEvent()
       createChild(header, "button", {
         class:
@@ -354,7 +371,7 @@ RuleEditor.prototype = {
           (isHighlighted ? " highlighted" : ""),
         "aria-pressed": isHighlighted,
         // This is used in rules.js for the selector highlighter
-        "data-computed-selector": desugaredSelector,
+        "data-computed-selector": computedSelector,
         title: l10n("rule.selectorHighlighter.tooltip"),
       });
     }
@@ -480,13 +497,14 @@ RuleEditor.prototype = {
    * Called when a tool is registered or unregistered.
    */
   _onToolChanged() {
+    if (!this.source) {
+      return;
+    }
+
     // When the source editor is registered, update the source links
     // to be clickable; and if it is unregistered, update the links to
-    // be unclickable.  However, some links are never clickable, so
-    // filter those out first.
-    if (this.source.getAttribute("unselectable") === "permanent") {
-      // Nothing.
-    } else if (this.toolbox.isToolRegistered("styleeditor")) {
+    // be unclickable.
+    if (this.toolbox.isToolRegistered("styleeditor")) {
       this.source.removeAttribute("unselectable");
     } else {
       this.source.setAttribute("unselectable", "true");
@@ -501,7 +519,8 @@ RuleEditor.prototype = {
     this.updateSourceLink();
   },
 
-  _onSourceClick() {
+  _onSourceClick(e) {
+    e.preventDefault();
     if (this.source.hasAttribute("unselectable")) {
       return;
     }
@@ -537,53 +556,51 @@ RuleEditor.prototype = {
       constructed,
       href: displayURL,
     });
-    let title = displayURL ? displayURL : sourceTextContent;
+
+    let displayLocation = displayURL ? displayURL : sourceTextContent;
     if (line > 0) {
       sourceTextContent += ":" + line;
-      title += ":" + line;
+      displayLocation += ":" + line;
     }
+    const title = COMPONENT_L10N.getFormatStr(
+      "frame.viewsourceinstyleeditor",
+      displayLocation
+    );
 
     const sourceLabel = this.element.querySelector(
       ".ruleview-rule-source-label"
     );
     sourceLabel.setAttribute("title", title);
-    sourceLabel.setAttribute("data-url", displayURL);
+    sourceLabel.setAttribute("href", displayURL);
     sourceLabel.textContent = sourceTextContent;
   },
 
   updateSourceLink() {
-    if (this.rule.isSystem) {
-      const sourceLabel = this.element.querySelector(
-        ".ruleview-rule-source-label"
-      );
-      const uaLabel = STYLE_INSPECTOR_L10N.getStr("rule.userAgentStyles");
-      sourceLabel.textContent = uaLabel + " " + this.rule.title;
-      sourceLabel.setAttribute("data-url", this.rule.sheet?.href);
-    } else {
-      this._updateLocation(null);
-    }
+    if (this.source) {
+      if (this.rule.isSystem) {
+        const sourceLabel = this.element.querySelector(
+          ".ruleview-rule-source-label"
+        );
+        const uaLabel = STYLE_INSPECTOR_L10N.getStr("rule.userAgentStyles");
+        sourceLabel.textContent = uaLabel + " " + this.rule.title;
+        sourceLabel.setAttribute("href", this.rule.sheet?.href);
+      } else {
+        this._updateLocation(null);
+      }
 
-    if (
-      this.rule.sheet &&
-      !this.rule.isSystem &&
-      this.rule.domRule.type !== ELEMENT_STYLE
-    ) {
-      // Only get the original source link if the rule isn't a system
-      // rule and if it isn't an inline rule.
-      if (this._unsubscribeSourceMap) {
-        this._unsubscribeSourceMap();
+      if (this.rule.sheet && !this.rule.isSystem) {
+        // Only get the original source link if the rule isn't a system
+        // rule and if it isn't an inline rule.
+        if (this._unsubscribeSourceMap) {
+          this._unsubscribeSourceMap();
+        }
+        this._unsubscribeSourceMap = this.sourceMapURLService.subscribeByID(
+          this.rule.sheet.resourceId,
+          this.rule.ruleLine,
+          this.rule.ruleColumn,
+          this._updateLocation
+        );
       }
-      this._unsubscribeSourceMap = this.sourceMapURLService.subscribeByID(
-        this.rule.sheet.resourceId,
-        this.rule.ruleLine,
-        this.rule.ruleColumn,
-        this._updateLocation
-      );
-      // Set "unselectable" appropriately.
-      this._onToolChanged();
-    } else if (this.rule.domRule.type === ELEMENT_STYLE) {
-      this.source.setAttribute("unselectable", "permanent");
-    } else {
       // Set "unselectable" appropriately.
       this._onToolChanged();
     }
@@ -613,66 +630,8 @@ RuleEditor.prototype = {
     } else if (this.rule.domRule.type === CSSRule.KEYFRAME_RULE) {
       this.selectorText.textContent = this.rule.domRule.keyText;
     } else {
-      const desugaredSelectors = this.rule.domRule.desugaredSelectors;
       this.rule.domRule.selectors.forEach((selector, i) => {
-        if (i !== 0) {
-          createChild(this.selectorText, "span", {
-            class: "ruleview-selector-separator",
-            textContent: ", ",
-          });
-        }
-
-        let containerClass = "ruleview-selector ";
-
-        // Only add matched/unmatched class when the rule does have some matched
-        // selectors. We don't always have some (e.g. rules for pseudo elements)
-        if (this.rule.matchedDesugaredSelectors.length) {
-          const desugaredSelector = desugaredSelectors[i];
-          const matchedSelector =
-            this.rule.matchedDesugaredSelectors.includes(desugaredSelector);
-          containerClass += matchedSelector ? "matched" : "unmatched";
-        }
-
-        const selectorContainer = createChild(this.selectorText, "span", {
-          class: containerClass,
-        });
-
-        const parsedSelector = parsePseudoClassesAndAttributes(selector);
-
-        for (const selectorText of parsedSelector) {
-          let selectorClass = "";
-
-          switch (selectorText.type) {
-            case SELECTOR_ATTRIBUTE:
-              selectorClass = "ruleview-selector-attribute";
-              break;
-            case SELECTOR_ELEMENT:
-              selectorClass = "ruleview-selector-element";
-              break;
-            case SELECTOR_PSEUDO_CLASS:
-              selectorClass = PSEUDO_CLASSES.some(
-                pseudo => selectorText.value === pseudo
-              )
-                ? "ruleview-selector-pseudo-class-lock"
-                : "ruleview-selector-pseudo-class";
-              break;
-            default:
-              break;
-          }
-
-          createChild(selectorContainer, "span", {
-            textContent: selectorText.value,
-            class: selectorClass,
-          });
-        }
-
-        const warningsContainer = this._createWarningsElementForSelector(
-          i,
-          this.rule.domRule.selectorWarnings
-        );
-        if (warningsContainer) {
-          selectorContainer.append(warningsContainer);
-        }
+        this._populateSelector(selector, i);
       });
     }
 
@@ -692,7 +651,9 @@ RuleEditor.prototype = {
 
     for (const prop of this.rule.textProps) {
       if (!prop.editor && !prop.invisible) {
-        const editor = new TextPropertyEditor(this, prop);
+        const editor = new TextPropertyEditor(this, prop, {
+          elementsWithPendingClicks: this.options.elementsWithPendingClicks,
+        });
         this.propertyList.appendChild(editor.element);
       } else if (prop.editor) {
         // If an editor already existed, append it to the bottom now to make sure the
@@ -713,6 +674,91 @@ RuleEditor.prototype = {
     }
   },
 
+  /**
+   * Render a given rule selector in this.selectorText element
+   *
+   * @param {String} selector: The selector text to display
+   * @param {Number} selectorIndex: Its index in the rule
+   */
+  _populateSelector(selector, selectorIndex) {
+    if (selectorIndex !== 0) {
+      createChild(this.selectorText, "span", {
+        class: "ruleview-selector-separator",
+        textContent: ", ",
+      });
+    }
+
+    let containerClass = "ruleview-selector ";
+
+    // Only add matched/unmatched class when the rule does have some matched
+    // selectors. We don't always have some (e.g. rules for pseudo elements)
+
+    if (this.rule.matchedSelectorIndexes.length) {
+      containerClass += this.rule.matchedSelectorIndexes.includes(selectorIndex)
+        ? "matched"
+        : "unmatched";
+    }
+
+    let selectorContainerTitle;
+    if (
+      typeof this.rule.selector.selectorsSpecificity?.[selectorIndex] !==
+      "undefined"
+    ) {
+      // The specificity that we get from the platform is a single number that we
+      // need to format into the common `(x,y,z)` specificity string.
+      const specificity =
+        this.rule.selector.selectorsSpecificity?.[selectorIndex];
+      const a = Math.floor(specificity / (1024 * 1024));
+      const b = Math.floor((specificity % (1024 * 1024)) / 1024);
+      const c = specificity % 1024;
+      selectorContainerTitle = STYLE_INSPECTOR_L10N.getFormatStr(
+        "rule.selectorSpecificity.title",
+        `(${a},${b},${c})`
+      );
+    }
+    const selectorContainer = createChild(this.selectorText, "span", {
+      class: containerClass,
+      title: selectorContainerTitle,
+    });
+
+    const parsedSelector = parsePseudoClassesAndAttributes(selector);
+
+    for (const selectorText of parsedSelector) {
+      let selectorClass = "";
+
+      switch (selectorText.type) {
+        case SELECTOR_ATTRIBUTE:
+          selectorClass = "ruleview-selector-attribute";
+          break;
+        case SELECTOR_ELEMENT:
+          selectorClass = "ruleview-selector-element";
+          break;
+        case SELECTOR_PSEUDO_CLASS:
+          selectorClass = PSEUDO_CLASSES.some(
+            pseudo => selectorText.value === pseudo
+          )
+            ? "ruleview-selector-pseudo-class-lock"
+            : "ruleview-selector-pseudo-class";
+          break;
+        default:
+          break;
+      }
+
+      createChild(selectorContainer, "span", {
+        textContent: selectorText.value,
+        class: selectorClass,
+      });
+    }
+
+    const warningsContainer = this._createWarningsElementForSelector(
+      selectorIndex,
+      this.rule.domRule.selectorWarnings
+    );
+    if (warningsContainer) {
+      selectorContainer.append(warningsContainer);
+    }
+  },
+
   /**
    * Programatically add a new property to the rule.
    *
@@ -738,7 +784,9 @@ RuleEditor.prototype = {
       siblingProp
     );
     const index = this.rule.textProps.indexOf(prop);
-    const editor = new TextPropertyEditor(this, prop);
+    const editor = new TextPropertyEditor(this, prop, {
+      elementsWithPendingClicks: this.options.elementsWithPendingClicks,
+    });
 
     // Insert this node before the DOM node that is currently at its new index
     // in the property list.  There is currently one less node in the DOM than
@@ -825,15 +873,17 @@ RuleEditor.prototype = {
     this.editor = new InplaceEditor({
       element: this.newPropSpan,
       done: this._onNewProperty,
+      // (Shift+)Tab will move the focus to the previous/next editable field
+      focusEditableFieldAfterApply: true,
+      focusEditableFieldContainerSelector: ".ruleview-rule",
       destroy: this._newPropertyDestroy,
       advanceChars: ":",
       contentType: InplaceEditor.CONTENT_TYPES.CSS_PROPERTY,
       popup: this.ruleView.popup,
       cssProperties: this.rule.cssProperties,
       inputAriaLabel: NEW_PROPERTY_NAME_INPUT_LABEL,
-      cssVariables: this.rule.elementStyle.getAllCustomProperties(
-        this.rule.pseudoElement
-      ),
+      getCssVariables: () =>
+        this.rule.elementStyle.getAllCustomProperties(this.rule.pseudoElement),
     });
 
     // Auto-close the input if multiple rules get pasted into new property.
@@ -923,8 +973,12 @@ RuleEditor.prototype = {
     this.isEditing = true;
 
     // Remove highlighter for the previous selector.
-    if (this.ruleView.isSelectorHighlighted(this.rule.selectorText)) {
-      await this.ruleView.toggleSelectorHighlighter(this.rule.selectorText);
+    const computedSelector = this.rule.domRule.computedSelector;
+    if (this.ruleView.isSelectorHighlighted(computedSelector)) {
+      await this.ruleView.toggleSelectorHighlighter(
+        this.rule,
+        computedSelector
+      );
     }
 
     try {
