@@ -149,19 +149,27 @@ class CanonicalBrowsingContext final : public BrowsingContext {
       const MaybeDiscardedBrowsingContext& aClonedStaticBrowsingContext);
   MOZ_CAN_RUN_SCRIPT void ReleaseClonedPrint(
       const MaybeDiscardedBrowsingContext& aClonedStaticBrowsingContext);
-  // Call the given callback on all top-level descendant BrowsingContexts.
+
+  enum class TopDescendantKind {
+    // All top descendants are included, even those inside a nested top
+    // browser.
+    All,
+    // Top descendants that are either direct children or under a non-nested
+    // descendant are included, but not those nested inside a separate top.
+    NonNested,
+    // Only our direct children are included. This is usually slightly less
+    // efficient than the alternatives, but might be needed in some cases.
+    ChildrenOnly,
+  };
+  // Call the given callback on top-level descendant BrowsingContexts.
   // Return Callstate::Stop from the callback to stop calling further children.
-  //
-  // If aIncludeNestedBrowsers is true, then all top descendants are included,
-  // even those inside a nested top browser.
-  void CallOnAllTopDescendants(
+  void CallOnTopDescendants(
       const FunctionRef<CallState(CanonicalBrowsingContext*)>& aCallback,
-      bool aIncludeNestedBrowsers);
+      TopDescendantKind aKind);
 
   void SessionHistoryCommit(uint64_t aLoadId, const nsID& aChangeID,
-                            uint32_t aLoadType, bool aPersist,
-                            bool aCloneEntryChildren, bool aChannelExpired,
-                            uint32_t aCacheKey);
+                            uint32_t aLoadType, bool aCloneEntryChildren,
+                            bool aChannelExpired, uint32_t aCacheKey);
 
   // Calls the session history listeners' OnHistoryReload, storing the result in
   // aCanReload. If aCanReload is set to true and we have an active or a loading
@@ -187,9 +195,9 @@ class CanonicalBrowsingContext final : public BrowsingContext {
 
   void RemoveFromSessionHistory(const nsID& aChangeID);
 
-  Maybe<int32_t> HistoryGo(int32_t aOffset, uint64_t aHistoryEpoch,
-                           bool aRequireUserInteraction, bool aUserActivation,
-                           Maybe<ContentParentId> aContentId);
+  MOZ_CAN_RUN_SCRIPT Maybe<int32_t> HistoryGo(
+      int32_t aOffset, uint64_t aHistoryEpoch, bool aRequireUserInteraction,
+      bool aUserActivation, Maybe<ContentParentId> aContentId);
 
   JSObject* WrapObject(JSContext* aCx,
                        JS::Handle<JSObject*> aGivenProto) override;
@@ -229,12 +237,16 @@ class CanonicalBrowsingContext final : public BrowsingContext {
   void LoadURI(nsIURI* aURI, const LoadURIOptions& aOptions,
                ErrorResult& aError);
 
+  MOZ_CAN_RUN_SCRIPT
   void GoBack(const Optional<int32_t>& aCancelContentJSEpoch,
               bool aRequireUserInteraction, bool aUserActivation);
+  MOZ_CAN_RUN_SCRIPT
   void GoForward(const Optional<int32_t>& aCancelContentJSEpoch,
                  bool aRequireUserInteraction, bool aUserActivation);
+  MOZ_CAN_RUN_SCRIPT
   void GoToIndex(int32_t aIndex, const Optional<int32_t>& aCancelContentJSEpoch,
                  bool aUserActivation);
+  MOZ_CAN_RUN_SCRIPT
   void Reload(uint32_t aReloadFlags);
   void Stop(uint32_t aStopFlags);
 
@@ -247,10 +259,15 @@ class CanonicalBrowsingContext final : public BrowsingContext {
 
   // Internal method to change which process a BrowsingContext is being loaded
   // in. The returned promise will resolve when the process switch is completed.
+  // The returned CanonicalBrowsingContext may be different than |this| if a BCG
+  // switch was performed.
   //
   // A NOT_REMOTE_TYPE aRemoteType argument will perform a process switch into
   // the parent process, and the method will resolve with a null BrowserParent.
-  using RemotenessPromise = MozPromise<RefPtr<BrowserParent>, nsresult, false>;
+  using RemotenessPromise = MozPromise<
+      std::pair<RefPtr<BrowserParent>, RefPtr<CanonicalBrowsingContext>>,
+      nsresult, false>;
+  MOZ_CAN_RUN_SCRIPT
   RefPtr<RemotenessPromise> ChangeRemoteness(
       const NavigationIsolationOptions& aOptions, uint64_t aPendingSwitchId);
 
@@ -283,8 +300,6 @@ class CanonicalBrowsingContext final : public BrowsingContext {
   // mixed content/https-only state for our current window is changed.
   void UpdateSecurityState();
 
-  void MaybeAddAsProgressListener(nsIWebProgress* aWebProgress);
-
   // Called when a navigation forces us to recreate our browsing
   // context (for example, when switching in or out of the parent
   // process).
@@ -311,6 +326,8 @@ class CanonicalBrowsingContext final : public BrowsingContext {
   void GetLoadingSessionHistoryInfoFromParent(
       Maybe<LoadingSessionHistoryInfo>& aLoadingInfo);
 
+  mozilla::Span<const SessionHistoryInfo> GetContiguousSessionHistoryInfos();
+
   void HistoryCommitIndexAndLength();
 
   void SynchronizeLayoutHistoryState();
@@ -346,6 +363,16 @@ class CanonicalBrowsingContext final : public BrowsingContext {
 
   bool AllowedInBFCache(const Maybe<uint64_t>& aChannelId, nsIURI* aNewURI);
 
+ private:
+  static nsresult ContainsSameOriginBfcacheEntry(
+      nsISHEntry* aEntry, mozilla::dom::BrowsingContext* aBC,
+      int32_t aChildIndex, void* aData);
+
+ public:
+  // Removes all bfcache entries that match the origin + originAttributes of the
+  // principal. Must be passed partitionedPrincipal
+  static nsresult ClearBfcacheByPrincipal(nsIPrincipal* aPrincipal);
+
   // Methods for getting and setting the active state for top level
   // browsing contexts, for the process priority manager.
   bool IsPriorityActive() const {
@@ -357,12 +384,7 @@ class CanonicalBrowsingContext final : public BrowsingContext {
     mPriorityActive = aIsActive;
   }
 
-  void SetIsActive(bool aIsActive, ErrorResult& aRv) {
-    MOZ_ASSERT(ManuallyManagesActiveness(),
-               "Shouldn't be setting active status of this browsing context if "
-               "not manually managed");
-    SetIsActiveInternal(aIsActive, aRv);
-  }
+  void SetIsActive(bool aIsActive, ErrorResult& aRv);
 
   void SetIsActiveInternal(bool aIsActive, ErrorResult& aRv) {
     SetExplicitActive(aIsActive ? ExplicitActiveStatus::Active
@@ -388,6 +410,7 @@ class CanonicalBrowsingContext final : public BrowsingContext {
   void AddPageAwakeRequest();
   void RemovePageAwakeRequest();
 
+  MOZ_CAN_RUN_SCRIPT
   void CloneDocumentTreeInto(CanonicalBrowsingContext* aSource,
                              const nsACString& aRemoteType,
                              embedding::PrintData&& aPrintData);
@@ -412,6 +435,8 @@ class CanonicalBrowsingContext final : public BrowsingContext {
 
   already_AddRefed<BounceTrackingState> GetBounceTrackingState();
 
+  bool CanOpenModalPicker();
+
  protected:
   // Called when the browsing context is being discarded.
   void CanonicalDiscard();
@@ -451,11 +476,15 @@ class CanonicalBrowsingContext final : public BrowsingContext {
     friend class CanonicalBrowsingContext;
 
     ~PendingRemotenessChange();
+    MOZ_CAN_RUN_SCRIPT
     void ProcessLaunched();
+    MOZ_CAN_RUN_SCRIPT
     void ProcessReady();
+    MOZ_CAN_RUN_SCRIPT
     void MaybeFinish();
     void Clear();
 
+    MOZ_CAN_RUN_SCRIPT
     nsresult FinishTopContent();
     nsresult FinishSubframe();
 
@@ -577,6 +606,7 @@ class CanonicalBrowsingContext final : public BrowsingContext {
     RefPtr<SessionHistoryEntry> mEntry;
   };
   nsTArray<LoadingSessionHistoryEntry> mLoadingEntries;
+  AutoCleanLinkedList<RefPtr<SessionHistoryEntry>> mActiveEntryList;
   RefPtr<SessionHistoryEntry> mActiveEntry;
 
   RefPtr<nsSecureBrowserUI> mSecureBrowserUI;
@@ -621,6 +651,8 @@ class CanonicalBrowsingContext final : public BrowsingContext {
   bool mFullyDiscarded = false;
 
   nsTArray<std::function<void(uint64_t)>> mFullyDiscardedListeners;
+
+  nsTArray<SessionHistoryInfo> mActiveContiguousEntries;
 };
 
 }  // namespace dom
