@@ -33,9 +33,11 @@
 
 #include "mozilla/Attributes.h"
 #include "mozilla/dom/BrowsingContextGroup.h"
+#include "mozilla/dom/BrowserParent.h"
 #include "mozilla/dom/CanonicalBrowsingContext.h"
 #include "mozilla/dom/ContentParent.h"
 #include "mozilla/dom/Element.h"
+#include "mozilla/dom/Navigation.h"
 #include "mozilla/dom/RemoteWebProgressRequest.h"
 #include "mozilla/dom/WindowGlobalParent.h"
 #include "mozilla/LinkedList.h"
@@ -82,7 +84,7 @@ struct ListHelper {
   LinkedList<nsSHistory> mList;
 };
 
-static ListHelper gSHistoryList;
+MOZ_RUNINIT static ListHelper gSHistoryList;
 // Max viewers allowed total, across all SHistory objects - negative default
 // means we will calculate how many viewers to cache based on total memory
 int32_t nsSHistory::sHistoryMaxTotalViewers = -1;
@@ -98,6 +100,7 @@ LazyLogModule gSHistoryLog("nsSHistory");
 #define LOG(format) MOZ_LOG(gSHistoryLog, mozilla::LogLevel::Debug, format)
 
 extern mozilla::LazyLogModule gPageCacheLog;
+extern mozilla::LazyLogModule gNavigationLog;
 extern mozilla::LazyLogModule gSHIPBFCacheLog;
 
 // This macro makes it easier to print a log message which includes a URI's
@@ -303,14 +306,14 @@ uint32_t nsSHistory::CalcMaxTotalViewers() {
 #  define MAX_TOTAL_VIEWERS_BIAS 14
 #endif
 
-  // Calculate an estimate of how many ContentViewers we should cache based
-  // on RAM.  This assumes that the average ContentViewer is 4MB (conservative)
-  // and caps the max at 8 ContentViewers
+  // Calculate an estimate of how many DocumentViewers we should cache based
+  // on RAM.  This assumes that the average DocumentViewer is 4MB (conservative)
+  // and caps the max at 8 DocumentViewers
   //
-  // TODO: Should we split the cache memory betw. ContentViewer caching and
+  // TODO: Should we split the cache memory betw. DocumentViewer caching and
   // nsCacheService?
   //
-  // RAM    | ContentViewers | on Android
+  // RAM    | DocumentViewers | on Android
   // -------------------------------------
   // 32   Mb       0                0
   // 64   Mb       1                0
@@ -339,7 +342,7 @@ uint32_t nsSHistory::CalcMaxTotalViewers() {
 
   // This is essentially the same calculation as for nsCacheService,
   // except that we divide the final memory calculation by 4, since
-  // we assume each ContentViewer takes on average 4MB
+  // we assume each DocumentViewer takes on average 4MB
   uint32_t viewers = 0;
   double x = std::log(kBytesD) / std::log(2.0) - MAX_TOTAL_VIEWERS_BIAS;
   if (x > 0) {
@@ -615,10 +618,41 @@ void nsSHistory::WalkContiguousEntries(
   }
 }
 
+// static
+void nsSHistory::WalkContiguousEntriesInOrder(
+    nsISHEntry* aEntry, const std::function<void(nsISHEntry*)>& aCallback) {
+  MOZ_ASSERT(aEntry);
+  MOZ_ASSERT(SessionHistoryInParent());
+
+  nsCOMPtr<nsIURI> targetURI = aEntry->GetURI();
+
+  // Walk backward to find the entries that have the same origin as the
+  // input entry.
+  nsCOMPtr<SessionHistoryEntry> entry = do_QueryInterface(aEntry);
+  MOZ_ASSERT(entry);
+  while (nsCOMPtr previousEntry = entry->getPrevious()) {
+    nsCOMPtr<nsIURI> uri = previousEntry->GetURI();
+    if (NS_FAILED(nsContentUtils::GetSecurityManager()->CheckSameOriginURI(
+            targetURI, uri, false, false))) {
+      break;
+    }
+    entry = previousEntry;
+  }
+
+  aCallback(entry);
+  while ((entry = entry->getNext())) {
+    nsCOMPtr<nsIURI> uri = entry->GetURI();
+    if (NS_FAILED(nsContentUtils::GetSecurityManager()->CheckSameOriginURI(
+            targetURI, uri, false, false))) {
+      break;
+    }
+    aCallback(entry);
+  }
+}
+
 NS_IMETHODIMP
-nsSHistory::AddChildSHEntryHelper(nsISHEntry* aCloneRef, nsISHEntry* aNewEntry,
-                                  BrowsingContext* aRootBC,
-                                  bool aCloneChildren) {
+nsSHistory::AddNestedSHEntry(nsISHEntry* aOldEntry, nsISHEntry* aNewEntry,
+                             BrowsingContext* aRootBC, bool aCloneChildren) {
   MOZ_ASSERT(aRootBC->IsTop());
 
   /* You are currently in the rootDocShell.
@@ -639,12 +673,17 @@ nsSHistory::AddChildSHEntryHelper(nsISHEntry* aCloneRef, nsISHEntry* aNewEntry,
   NS_ENSURE_TRUE(currentHE, NS_ERROR_FAILURE);
 
   nsresult rv = NS_OK;
-  uint32_t cloneID = aCloneRef->GetID();
+  uint32_t cloneID = aOldEntry->GetID();
   rv = nsSHistory::CloneAndReplace(currentHE, aRootBC, cloneID, aNewEntry,
                                    aCloneChildren, getter_AddRefs(child));
 
   if (NS_SUCCEEDED(rv)) {
-    rv = AddEntry(child, true);
+    if (aOldEntry->IsTransient()) {
+      rv = ReplaceEntry(mIndex, child);
+    } else {
+      rv = AddEntry(child);
+    }
+
     if (NS_SUCCEEDED(rv)) {
       child->SetDocshellID(aRootBC->GetHistoryID());
     }
@@ -725,22 +764,10 @@ void nsSHistory::HandleEntriesToSwapInDocShell(
   }
 }
 
-void nsSHistory::UpdateRootBrowsingContextState(BrowsingContext* aRootBC) {
-  if (aRootBC && aRootBC->EverAttached()) {
-    bool sameDocument = IsEmptyOrHasEntriesForSingleTopLevelPage();
-    if (sameDocument != aRootBC->GetIsSingleToplevelInHistory()) {
-      // If the browsing context is discarded then its session history is
-      // invalid and will go away.
-      Unused << aRootBC->SetIsSingleToplevelInHistory(sameDocument);
-    }
-  }
-}
-
 NS_IMETHODIMP
 nsSHistory::AddToRootSessionHistory(bool aCloneChildren, nsISHEntry* aOSHE,
                                     BrowsingContext* aRootBC,
                                     nsISHEntry* aEntry, uint32_t aLoadType,
-                                    bool aShouldPersist,
                                     Maybe<int32_t>* aPreviousEntryIndex,
                                     Maybe<int32_t>* aLoadedEntryIndex) {
   MOZ_ASSERT(aRootBC->IsTop());
@@ -777,7 +804,7 @@ nsSHistory::AddToRootSessionHistory(bool aCloneChildren, nsISHEntry* aOSHE,
   if (addToSHistory) {
     // Add to session history
     *aPreviousEntryIndex = Some(mIndex);
-    rv = AddEntry(aEntry, aShouldPersist);
+    rv = AddEntry(aEntry);
     *aLoadedEntryIndex = Some(mIndex);
     MOZ_LOG(gPageCacheLog, LogLevel::Verbose,
             ("Previous index: %d, Loaded index: %d",
@@ -793,7 +820,7 @@ nsSHistory::AddToRootSessionHistory(bool aCloneChildren, nsISHEntry* aOSHE,
  * increment the index to point to the new entry
  */
 NS_IMETHODIMP
-nsSHistory::AddEntry(nsISHEntry* aSHEntry, bool aPersist) {
+nsSHistory::AddEntry(nsISHEntry* aSHEntry) {
   NS_ENSURE_ARG(aSHEntry);
 
   nsCOMPtr<nsISHistory> shistoryOfEntry = aSHEntry->GetShistory();
@@ -820,11 +847,9 @@ nsSHistory::AddEntry(nsISHEntry* aSHEntry, bool aPersist) {
       return NS_ERROR_FAILURE;
     }
 
-    if (mEntries[mIndex] && !mEntries[mIndex]->GetPersist()) {
+    if (mEntries[mIndex] && mEntries[mIndex]->IsTransient()) {
       NotifyListeners(mListeners, [](auto l) { l->OnHistoryReplaceEntry(); });
-      aSHEntry->SetPersist(aPersist);
       mEntries[mIndex] = aSHEntry;
-      UpdateRootBrowsingContextState();
       return NS_OK;
     }
   }
@@ -843,7 +868,6 @@ nsSHistory::AddEntry(nsISHEntry* aSHEntry, bool aPersist) {
   // Remove all entries after the current one, add the new one, and set the
   // new one as the current one.
   MOZ_ASSERT(mIndex >= -1);
-  aSHEntry->SetPersist(aPersist);
   mEntries.TruncateLength(mIndex + 1);
   mEntries.AppendElement(aSHEntry);
   mIndex++;
@@ -856,8 +880,6 @@ nsSHistory::AddEntry(nsISHEntry* aSHEntry, bool aPersist) {
     PurgeHistory(Length() - gHistoryMaxSize);
   }
 
-  UpdateRootBrowsingContextState();
-
   return NS_OK;
 }
 
@@ -974,6 +996,9 @@ static void LogEntry(nsISHEntry* aEntry, int32_t aIndex, int32_t aTotal,
   MOZ_LOG(gSHLog, LogLevel::Debug,
           (" %s%s  Name = %s\n", prefix.get(), childCount > 0 ? "|" : " ",
            NS_LossyConvertUTF16toASCII(name).get()));
+  MOZ_LOG(gSHLog, LogLevel::Debug,
+          (" %s%s  Transient = %s\n", prefix.get(), childCount > 0 ? "|" : " ",
+           aEntry->IsTransient() ? "true" : "false"));
   MOZ_LOG(
       gSHLog, LogLevel::Debug,
       (" %s%s  Is in BFCache = %s\n", prefix.get(), childCount > 0 ? "|" : " ",
@@ -1091,8 +1116,6 @@ nsSHistory::PurgeHistory(int32_t aNumEntries) {
     rootBC->GetDocShell()->HistoryPurged(aNumEntries);
   }
 
-  UpdateRootBrowsingContextState(rootBC);
-
   return NS_OK;
 }
 
@@ -1151,11 +1174,8 @@ nsSHistory::ReplaceEntry(int32_t aIndex, nsISHEntry* aReplaceEntry) {
 
   NotifyListeners(mListeners, [](auto l) { l->OnHistoryReplaceEntry(); });
 
-  aReplaceEntry->SetPersist(true);
   mEntries[aIndex] = aReplaceEntry;
 
-  UpdateRootBrowsingContextState();
-
   return NS_OK;
 }
 
@@ -1222,6 +1242,7 @@ nsSHistory::EvictAllDocumentViewers() {
   return NS_OK;
 }
 
+MOZ_CAN_RUN_SCRIPT
 static void FinishRestore(CanonicalBrowsingContext* aBrowsingContext,
                           nsDocShellLoadState* aLoadState,
                           SessionHistoryEntry* aEntry,
@@ -1280,11 +1301,11 @@ static void FinishRestore(CanonicalBrowsingContext* aBrowsingContext,
       }
     }
 
-    if (aBrowsingContext->IsActive()) {
-      loadingBC->PreOrderWalk([&](BrowsingContext* aContext) {
-        if (BrowserParent* bp = aContext->Canonical()->GetBrowserParent()) {
-          ProcessPriorityManager::BrowserPriorityChanged(bp, true);
-        }
+    // Ensure browser priority to matches `IsPriorityActive` after restoring.
+    if (BrowserParent* bp = loadingBC->GetBrowserParent()) {
+      bp->VisitAll([&](BrowserParent* aBp) {
+        ProcessPriorityManager::BrowserPriorityChanged(
+            aBp, aBrowsingContext->IsPriorityActive());
       });
     }
 
@@ -1338,68 +1359,79 @@ static void FinishRestore(CanonicalBrowsingContext* aBrowsingContext,
   aBrowsingContext->LoadURI(aLoadState, false);
 }
 
+MOZ_CAN_RUN_SCRIPT
+static bool MaybeLoadBFCache(nsSHistory::LoadEntryResult& aLoadEntry) {
+  MOZ_ASSERT(XRE_IsParentProcess());
+  RefPtr<nsDocShellLoadState> loadState = aLoadEntry.mLoadState;
+  RefPtr<CanonicalBrowsingContext> canonicalBC =
+      aLoadEntry.mBrowsingContext->Canonical();
+  nsCOMPtr<SessionHistoryEntry> she = do_QueryInterface(loadState->SHEntry());
+  nsCOMPtr<SessionHistoryEntry> currentShe =
+      canonicalBC->GetActiveSessionHistoryEntry();
+  MOZ_ASSERT(she);
+  RefPtr<nsFrameLoader> frameLoader = she->GetFrameLoader();
+  if (frameLoader && canonicalBC->Group()->Toplevels().Length() == 1 &&
+      (!currentShe || (she->SharedInfo() != currentShe->SharedInfo() &&
+                       !currentShe->GetFrameLoader()))) {
+    bool canSave = (!currentShe || currentShe->GetSaveLayoutStateFlag()) &&
+                   canonicalBC->AllowedInBFCache(Nothing(), nullptr);
+
+    MOZ_LOG(gSHIPBFCacheLog, LogLevel::Debug,
+            ("nsSHistory::LoadURIOrBFCache "
+             "saving presentation=%i",
+             canSave));
+
+    nsCOMPtr<nsFrameLoaderOwner> frameLoaderOwner =
+        do_QueryInterface(canonicalBC->GetEmbedderElement());
+    if (!loadState->NotifiedBeforeUnloadListeners() && frameLoaderOwner) {
+      RefPtr<nsFrameLoader> currentFrameLoader =
+          frameLoaderOwner->GetFrameLoader();
+      if (currentFrameLoader &&
+          currentFrameLoader->GetMaybePendingBrowsingContext()) {
+        if (WindowGlobalParent* wgp =
+                currentFrameLoader->GetMaybePendingBrowsingContext()
+                    ->Canonical()
+                    ->GetCurrentWindowGlobal()) {
+          wgp->PermitUnload(
+              [canonicalBC, loadState, she, frameLoader, currentFrameLoader,
+               canSave](bool aAllow) MOZ_CAN_RUN_SCRIPT_BOUNDARY_LAMBDA {
+                if (aAllow && !canonicalBC->IsReplaced()) {
+                  FinishRestore(canonicalBC, loadState, she, frameLoader,
+                                canSave && canonicalBC->AllowedInBFCache(
+                                               Nothing(), nullptr));
+                } else if (currentFrameLoader
+                               ->GetMaybePendingBrowsingContext()) {
+                  nsISHistory* shistory =
+                      currentFrameLoader->GetMaybePendingBrowsingContext()
+                          ->Canonical()
+                          ->GetSessionHistory();
+                  if (shistory) {
+                    shistory->InternalSetRequestedIndex(-1);
+                  }
+                }
+              });
+          return true;
+        }
+      }
+    }
+
+    FinishRestore(canonicalBC, loadState, she, frameLoader, canSave);
+    return true;
+  }
+  if (frameLoader) {
+    she->SetFrameLoader(nullptr);
+    frameLoader->Destroy();
+  }
+
+  return false;
+}
+
 /* static */
 void nsSHistory::LoadURIOrBFCache(LoadEntryResult& aLoadEntry) {
   if (mozilla::BFCacheInParent() && aLoadEntry.mBrowsingContext->IsTop()) {
-    MOZ_ASSERT(XRE_IsParentProcess());
-    RefPtr<nsDocShellLoadState> loadState = aLoadEntry.mLoadState;
-    RefPtr<CanonicalBrowsingContext> canonicalBC =
-        aLoadEntry.mBrowsingContext->Canonical();
-    nsCOMPtr<SessionHistoryEntry> she = do_QueryInterface(loadState->SHEntry());
-    nsCOMPtr<SessionHistoryEntry> currentShe =
-        canonicalBC->GetActiveSessionHistoryEntry();
-    MOZ_ASSERT(she);
-    RefPtr<nsFrameLoader> frameLoader = she->GetFrameLoader();
-    if (frameLoader && canonicalBC->Group()->Toplevels().Length() == 1 &&
-        (!currentShe || (she->SharedInfo() != currentShe->SharedInfo() &&
-                         !currentShe->GetFrameLoader()))) {
-      bool canSave = (!currentShe || currentShe->GetSaveLayoutStateFlag()) &&
-                     canonicalBC->AllowedInBFCache(Nothing(), nullptr);
-
-      MOZ_LOG(gSHIPBFCacheLog, LogLevel::Debug,
-              ("nsSHistory::LoadURIOrBFCache "
-               "saving presentation=%i",
-               canSave));
-
-      nsCOMPtr<nsFrameLoaderOwner> frameLoaderOwner =
-          do_QueryInterface(canonicalBC->GetEmbedderElement());
-      if (frameLoaderOwner) {
-        RefPtr<nsFrameLoader> currentFrameLoader =
-            frameLoaderOwner->GetFrameLoader();
-        if (currentFrameLoader &&
-            currentFrameLoader->GetMaybePendingBrowsingContext()) {
-          if (WindowGlobalParent* wgp =
-                  currentFrameLoader->GetMaybePendingBrowsingContext()
-                      ->Canonical()
-                      ->GetCurrentWindowGlobal()) {
-            wgp->PermitUnload([canonicalBC, loadState, she, frameLoader,
-                               currentFrameLoader, canSave](bool aAllow) {
-              if (aAllow && !canonicalBC->IsReplaced()) {
-                FinishRestore(canonicalBC, loadState, she, frameLoader,
-                              canSave && canonicalBC->AllowedInBFCache(
-                                             Nothing(), nullptr));
-              } else if (currentFrameLoader->GetMaybePendingBrowsingContext()) {
-                nsISHistory* shistory =
-                    currentFrameLoader->GetMaybePendingBrowsingContext()
-                        ->Canonical()
-                        ->GetSessionHistory();
-                if (shistory) {
-                  shistory->InternalSetRequestedIndex(-1);
-                }
-              }
-            });
-            return;
-          }
-        }
-      }
-
-      FinishRestore(canonicalBC, loadState, she, frameLoader, canSave);
+    if (MaybeLoadBFCache(aLoadEntry)) {
       return;
     }
-    if (frameLoader) {
-      she->SetFrameLoader(nullptr);
-      frameLoader->Destroy();
-    }
   }
 
   RefPtr<BrowsingContext> bc = aLoadEntry.mBrowsingContext;
@@ -1407,8 +1439,139 @@ void nsSHistory::LoadURIOrBFCache(LoadEntryResult& aLoadEntry) {
   bc->LoadURI(loadState, false);
 }
 
+// This implements step 4 of
+// https://html.spec.whatwg.org/#checking-if-unloading-is-canceled which handles
+// the case where we have a "navigate" handler for the top level window's
+// navigation object and/or a "beforeunload" handler for that same window. The
+// tricky part is that we need to check "beforeunload" for that window, then
+// "navigate", and after that continue with "beforeunload" for the remaining
+// tree.
+MOZ_CAN_RUN_SCRIPT
+static bool MaybeCheckUnloadingIsCanceled(
+    nsTArray<nsSHistory::LoadEntryResult>& aLoadResults,
+    BrowsingContext* aTraversable,
+    std::function<void(nsTArray<nsSHistory::LoadEntryResult>&, bool)>&&
+        aResolver) {
+  // Step 4
+  if (!aTraversable || !aTraversable->IsTop() || !SessionHistoryInParent() ||
+      !aLoadResults.Length() || !Navigation::IsAPIEnabled()) {
+    return false;
+  }
+
+  // Step 4.3.2
+  auto found =
+      std::find_if(aLoadResults.begin(), aLoadResults.end(),
+                   [traversable = RefPtr{aTraversable}](const auto& result) {
+                     return result.mBrowsingContext->Id() == traversable->Id();
+                   });
+  // Step 4.3.2
+  bool needsBeforeUnload = found != aLoadResults.end();
+
+  // Step 4.2
+  // This is a bit fishy since we don't have a direct way of performing
+  // the step, but this does its best.
+  RefPtr<nsDocShellLoadState> loadState =
+      needsBeforeUnload ? found->mLoadState : aLoadResults[0].mLoadState;
+  RefPtr<CanonicalBrowsingContext> browsingContext = aTraversable->Canonical();
+  MOZ_DIAGNOSTIC_ASSERT(!needsBeforeUnload ||
+                        found->mBrowsingContext == browsingContext);
+
+  nsCOMPtr<SessionHistoryEntry> targetEntry =
+      do_QueryInterface(loadState->SHEntry());
+  nsCOMPtr<SessionHistoryEntry> currentEntry =
+      browsingContext->GetActiveSessionHistoryEntry();
+
+  // Step 4.3
+  if (!currentEntry || currentEntry->GetID() == targetEntry->GetID()) {
+    return false;
+  }
+
+  RefPtr<WindowGlobalParent> windowGlobalParent =
+      browsingContext->GetCurrentWindowGlobal();
+  // An efficiency trick. We've set this flag on the window context if we've
+  // seen a "navigate" and/or a "beforeunload" handler set. If not we know we
+  // can skip this.
+  if (!windowGlobalParent || !windowGlobalParent->NeedsBeforeUnload()) {
+    return false;
+  }
+
+  nsCOMPtr<nsIURI> targetURI = targetEntry->GetURI();
+  if (!targetURI) {
+    return false;
+  }
+
+  nsCOMPtr<nsIPrincipal> targetPrincipal =
+      BasePrincipal::CreateContentPrincipal(targetURI, OriginAttributes{});
+
+  // More of step 4.3
+  if (!windowGlobalParent->DocumentPrincipal()->Equals(targetPrincipal)) {
+    return false;
+  }
+
+  // Step 4.3.3
+  if (needsBeforeUnload) {
+    aLoadResults.RemoveElementAt(found);
+  }
+
+  // Step 4.3.4
+  // PermitUnloadTraversable only includes the process of the top level browsing
+  // context.
+  windowGlobalParent->PermitUnloadTraversable(
+      targetEntry->Info(),
+      [loadResults = CopyableTArray(std::move(aLoadResults)),
+       windowGlobalParent, aResolver](bool aAllow) mutable {
+        if (!aAllow) {
+          aResolver(loadResults, aAllow);
+          return;
+        }
+
+        // PermitUnloadTraversable includes everything except the process of the
+        // top level browsing context.
+        windowGlobalParent->PermitUnloadChildNavigables(
+            [loadResults = std::move(loadResults), aResolver](
+                bool aAllow) mutable { aResolver(loadResults, aAllow); });
+      });
+
+  return true;
+}
+
 /* static */
-void nsSHistory::LoadURIs(nsTArray<LoadEntryResult>& aLoadResults) {
+void nsSHistory::LoadURIs(nsTArray<LoadEntryResult>& aLoadResults,
+                          BrowsingContext* aTraversable) {
+  // Here we have call a path that handles firing the "traverse" navigate event
+  // if applicable.
+  if (MaybeCheckUnloadingIsCanceled(
+          aLoadResults, aTraversable,
+          [traversable = RefPtr{aTraversable}](
+              nsTArray<LoadEntryResult>& aLoadResults,
+              bool aAllow) MOZ_CAN_RUN_SCRIPT_BOUNDARY_LAMBDA {
+            if (!aAllow) {
+              if (nsCOMPtr<nsFrameLoaderOwner> frameLoaderOwner =
+                      do_QueryInterface(traversable->GetEmbedderElement())) {
+                if (RefPtr<nsFrameLoader> currentFrameLoader =
+                        frameLoaderOwner->GetFrameLoader()) {
+                  nsISHistory* shistory =
+                      currentFrameLoader->GetMaybePendingBrowsingContext()
+                          ->Canonical()
+                          ->GetSessionHistory();
+                  if (shistory) {
+                    shistory->InternalSetRequestedIndex(-1);
+                  }
+                }
+              }
+              return;
+            }
+
+            for (LoadEntryResult& loadEntry : aLoadResults) {
+              loadEntry.mLoadState->SetNotifiedBeforeUnloadListeners(true);
+              LoadURIOrBFCache(loadEntry);
+            }
+          })) {
+    return;
+  }
+
+  // And we fall back to the simple case if we shouldn't fire a "traverse"
+  // navigate event.
   for (LoadEntryResult& loadEntry : aLoadResults) {
     LoadURIOrBFCache(loadEntry);
   }
@@ -1964,8 +2127,6 @@ void nsSHistory::RemoveEntries(nsTArray<nsID>& aIDs, int32_t aStartIndex,
     }
     --index;
   }
-
-  UpdateRootBrowsingContextState();
 }
 
 void nsSHistory::RemoveFrameEntries(nsISHEntry* aEntry) {
@@ -2057,6 +2218,24 @@ nsSHistory::HasUserInteractionAtIndex(int32_t aIndex) {
   return entry->GetHasUserInteraction();
 }
 
+NS_IMETHODIMP
+nsSHistory::CanGoBackFromEntryAtIndex(int32_t aIndex, bool* aCanGoBack) {
+  *aCanGoBack = false;
+  if (!StaticPrefs::browser_navigation_requireUserInteraction()) {
+    *aCanGoBack = aIndex > 0;
+    return NS_OK;
+  }
+
+  for (int32_t i = aIndex - 1; i >= 0; i--) {
+    if (HasUserInteractionAtIndex(i)) {
+      *aCanGoBack = true;
+      break;
+    }
+  }
+
+  return NS_OK;
+}
+
 nsresult nsSHistory::LoadNextPossibleEntry(
     int32_t aNewIndex, long aLoadType, uint32_t aHistCmd,
     nsTArray<LoadEntryResult>& aLoadResults, bool aLoadCurrentEntry,
@@ -2319,8 +2498,9 @@ void nsSHistory::InitiateLoad(nsISHEntry* aFrameEntry,
       aFrameEntry->GetTriggeringPrincipal();
   loadState->SetTriggeringPrincipal(triggeringPrincipal);
   loadState->SetFirstParty(false);
-  nsCOMPtr<nsIContentSecurityPolicy> csp = aFrameEntry->GetCsp();
-  loadState->SetCsp(csp);
+  nsCOMPtr<nsIPolicyContainer> policyContainer =
+      aFrameEntry->GetPolicyContainer();
+  loadState->SetPolicyContainer(policyContainer);
 
   loadResult->mLoadState = std::move(loadState);
 }
@@ -2337,25 +2517,6 @@ nsSHistory::CreateEntry(nsISHEntry** aEntry) {
   return NS_OK;
 }
 
-NS_IMETHODIMP_(bool)
-nsSHistory::IsEmptyOrHasEntriesForSingleTopLevelPage() {
-  if (mEntries.IsEmpty()) {
-    return true;
-  }
-
-  nsISHEntry* entry = mEntries[0];
-  size_t length = mEntries.Length();
-  for (size_t i = 1; i < length; ++i) {
-    bool sharesDocument = false;
-    mEntries[i]->SharesDocumentWith(entry, &sharesDocument);
-    if (!sharesDocument) {
-      return false;
-    }
-  }
-
-  return true;
-}
-
 static void CollectEntries(
     nsTHashMap<nsIDHashKey, SessionHistoryEntry*>& aHashtable,
     SessionHistoryEntry* aEntry) {
