@@ -56,21 +56,26 @@
 #include "mozilla/ipc/UtilityProcessSandboxing.h"
 #include "mozilla/ipc/UtilityProcessManager.h"
 #include "mozilla/ipc/UtilityProcessHost.h"
+#include "mozilla/layers/WebRenderBridgeChild.h"
+#include "mozilla/layers/WebRenderLayerManager.h"
 #include "mozilla/net/UrlClassifierFeatureFactory.h"
-#include "mozilla/RemoteDecoderManagerChild.h"
+#include "mozilla/RemoteMediaManagerChild.h"
 #include "mozilla/KeySystemConfig.h"
 #include "mozilla/WheelHandlingHelper.h"
+#include "nsRFPTargetSetIDL.h"
 #include "nsIRFPTargetSetIDL.h"
-#include "nsContentSecurityUtils.h"
+#include "nsIWidget.h"
 #include "nsString.h"
 #include "nsNativeTheme.h"
 #include "nsThreadUtils.h"
 #include "mozJSModuleLoader.h"
 #include "mozilla/ProfilerLabels.h"
 #include "mozilla/ProfilerMarkers.h"
+#include "nsContentUtils.h"
 #include "nsDocShell.h"
 #include "nsIException.h"
 #include "VsyncSource.h"
+#include "imgLoader.h"
 
 #ifdef XP_UNIX
 #  include <errno.h>
@@ -88,6 +93,10 @@
 #  include "mozilla/MFCDMParent.h"
 #endif
 
+#ifdef MOZ_WIDGET_ANDROID
+#  include "mozilla/java/GeckoAppShellWrappers.h"
+#endif
+
 namespace mozilla::dom {
 
 // Setup logging
@@ -1281,22 +1290,6 @@ void ChromeUtils::ClearRecentJSDevError(GlobalObject&) {
 }
 #endif  // NIGHTLY_BUILD
 
-void ChromeUtils::ClearStyleSheetCacheByPrincipal(GlobalObject&,
-                                                  nsIPrincipal* aForPrincipal) {
-  SharedStyleSheetCache::Clear(Some(aForPrincipal));
-}
-
-void ChromeUtils::ClearStyleSheetCacheBySite(
-    GlobalObject&, const nsACString& aSchemelessSite,
-    const dom::OriginAttributesPatternDictionary& aPattern) {
-  SharedStyleSheetCache::Clear(Nothing(), Some(nsCString(aSchemelessSite)),
-                               Some(OriginAttributesPattern(aPattern)));
-}
-
-void ChromeUtils::ClearStyleSheetCache(GlobalObject&) {
-  SharedStyleSheetCache::Clear();
-}
-
 void ChromeUtils::ClearMessagingLayerSecurityStateByPrincipal(
     GlobalObject&, nsIPrincipal* aPrincipal, ErrorResult& aRv) {
   MOZ_LOG(gMlsLog, LogLevel::Debug,
@@ -1591,20 +1584,143 @@ void ChromeUtils::ClearMessagingLayerSecurityState(GlobalObject&,
   MOZ_LOG(gMlsLog, LogLevel::Debug, ("Successfully cleared all MLS state"));
 }
 
-void ChromeUtils::ClearScriptCacheByPrincipal(GlobalObject&,
-                                              nsIPrincipal* aForPrincipal) {
-  SharedScriptCache::Clear(Some(aForPrincipal));
+void ChromeUtils::ClearResourceCache(
+    GlobalObject& aGlobal, const dom::ClearResourceCacheOptions& aOptions,
+    ErrorResult& aRv) {
+  bool clearStyleSheet = false;
+  bool clearScript = false;
+  bool clearImage = false;
+
+  if (aOptions.mTypes.WasPassed()) {
+    for (const auto& type : aOptions.mTypes.Value()) {
+      switch (type) {
+        case ResourceCacheType::Stylesheet:
+          clearStyleSheet = true;
+          break;
+        case ResourceCacheType::Script:
+          clearScript = true;
+          break;
+        case ResourceCacheType::Image:
+          clearImage = true;
+          break;
+      }
+    }
+  } else {
+    clearStyleSheet = true;
+    clearScript = true;
+    clearImage = true;
+  }
+
+  int filterCount = 0;
+  if (aOptions.mTarget.WasPassed()) {
+    filterCount++;
+  }
+  if (aOptions.mPrincipal.WasPassed()) {
+    filterCount++;
+  }
+  if (aOptions.mSchemelessSite.WasPassed()) {
+    filterCount++;
+  }
+  if (aOptions.mUrl.WasPassed()) {
+    filterCount++;
+  }
+  if (filterCount > 1) {
+    aRv.ThrowInvalidStateError(
+        "target, principal, schemelessSite, and url properties are mutually "
+        "exclusive");
+    return;
+  }
+
+  if (aOptions.mTarget.WasPassed()) {
+    Maybe<bool> chrome;
+    switch (aOptions.mTarget.Value()) {
+      case ResourceCacheTarget::Chrome:
+        chrome.emplace(true);
+        break;
+      case ResourceCacheTarget::Content:
+        chrome.emplace(false);
+        break;
+    }
+
+    if (clearStyleSheet) {
+      SharedStyleSheetCache::Clear(chrome);
+    }
+    if (clearScript) {
+      SharedScriptCache::Clear(chrome);
+    }
+    if (clearImage) {
+      imgLoader::ClearCache(Nothing(), chrome);
+    }
+    return;
+  }
+
+  if (aOptions.mPrincipal.WasPassed()) {
+    nsCOMPtr<nsIPrincipal> principal = aOptions.mPrincipal.Value().get();
+
+    if (clearStyleSheet) {
+      SharedStyleSheetCache::Clear(Nothing(), Some(principal));
+    }
+    if (clearScript) {
+      SharedScriptCache::Clear(Nothing(), Some(principal));
+    }
+    if (clearImage) {
+      imgLoader::ClearCache(Nothing(), Nothing(), Some(principal));
+    }
+    return;
+  }
+
+  if (aOptions.mSchemelessSite.WasPassed()) {
+    nsCString schemelessSite(aOptions.mSchemelessSite.Value());
+    mozilla::OriginAttributesPattern pattern(aOptions.mPattern);
+
+    if (clearStyleSheet) {
+      SharedStyleSheetCache::Clear(Nothing(), Nothing(), Some(schemelessSite),
+                                   Some(pattern));
+    }
+    if (clearScript) {
+      SharedScriptCache::Clear(Nothing(), Nothing(), Some(schemelessSite),
+                               Some(pattern));
+    }
+    if (clearImage) {
+      imgLoader::ClearCache(Nothing(), Nothing(), Nothing(),
+                            Some(schemelessSite), Some(pattern));
+    }
+    return;
+  }
+
+  if (aOptions.mUrl.WasPassed()) {
+    nsCString url(aOptions.mUrl.Value());
+
+    if (clearStyleSheet) {
+      SharedStyleSheetCache::Clear(Nothing(), Nothing(), Nothing(), Nothing(),
+                                   Some(url));
+    }
+    if (clearScript) {
+      SharedScriptCache::Clear(Nothing(), Nothing(), Nothing(), Nothing(),
+                               Some(url));
+    }
+    if (clearImage) {
+      imgLoader::ClearCache(Nothing(), Nothing(), Nothing(), Nothing(),
+                            Nothing(), Some(url));
+    }
+    return;
+  }
+
+  if (clearStyleSheet) {
+    SharedStyleSheetCache::Clear();
+  }
+  if (clearScript) {
+    SharedScriptCache::Clear();
+  }
+  if (clearImage) {
+    imgLoader::ClearCache();
+  }
 }
 
-void ChromeUtils::ClearScriptCacheBySite(
-    GlobalObject&, const nsACString& aSchemelessSite,
-    const dom::OriginAttributesPatternDictionary& aPattern) {
-  SharedScriptCache::Clear(Nothing(), Some(nsCString(aSchemelessSite)),
-                           Some(aPattern));
-}
-
-void ChromeUtils::ClearScriptCache(GlobalObject&) {
-  SharedScriptCache::Clear();
+void ChromeUtils::ClearBfcacheByPrincipal(GlobalObject& aGlobal,
+                                          nsIPrincipal* aPrincipal,
+                                          ErrorResult& aRv) {
+  aRv = CanonicalBrowsingContext::ClearBfcacheByPrincipal(aPrincipal);
 }
 
 #define PROCTYPE_TO_WEBIDL_CASE(_procType, _webidl) \
@@ -2144,12 +2260,63 @@ void ChromeUtils::ResetLastExternalProtocolIframeAllowed(
 }
 
 /* static */
-void ChromeUtils::EndWheelTransaction(GlobalObject& aGlobal) {
+already_AddRefed<Promise> ChromeUtils::EndWheelTransaction(
+    GlobalObject& aGlobal, WindowProxyHolder& aWindow, ErrorResult& aRv) {
+  nsCOMPtr<nsIGlobalObject> global = do_QueryInterface(aGlobal.GetAsSupports());
+  MOZ_ASSERT(global);
+
+  RefPtr<Promise> promise = Promise::Create(global, aRv);
+  if (aRv.Failed()) {
+    return nullptr;
+  }
+
   // This allows us to end the current wheel transaction from the browser
   // chrome. We do not need to perform any checks before calling
   // EndTransaction(), as it should do nothing in the case that there is
   // no current wheel transaction.
   WheelTransaction::EndTransaction();
+
+  // We also need to end the wheel transaction in APZ.
+  nsIDocShell* docShell = aWindow.get()->GetDocShell();
+  if (!docShell) {
+    promise->MaybeResolveWithUndefined();
+    return promise.forget();
+  }
+
+  nsIWidget* widget =
+      nsContentUtils::GetWidget(docShell->GetPresShell(), nullptr);
+  if (!widget) {
+    promise->MaybeResolveWithUndefined();
+    return promise.forget();
+  }
+
+  WindowRenderer* renderer = widget->GetWindowRenderer();
+  if (!renderer) {
+    promise->MaybeResolveWithUndefined();
+    return promise.forget();
+  }
+
+  layers::WebRenderLayerManager* wr = renderer->AsWebRender();
+  if (!wr) {
+    promise->MaybeResolveWithUndefined();
+    return promise.forget();
+  }
+
+  layers::WebRenderBridgeChild* wrbc = wr->WrBridge();
+  if (!wrbc) {
+    promise->MaybeResolveWithUndefined();
+    return promise.forget();
+  }
+
+  wrbc->SendEndWheelTransaction()->Then(
+      GetCurrentSerialEventTarget(), __func__,
+      [promise](bool) { promise->MaybeResolveWithUndefined(); },
+      [promise](mozilla::ipc::ResponseRejectReason) {
+        promise->MaybeRejectWithUnknownError(
+            "actor died while ending wheel transaction");
+      });
+
+  return promise.forget();
 }
 
 /* static */
@@ -2157,7 +2324,11 @@ void ChromeUtils::RegisterWindowActor(const GlobalObject& aGlobal,
                                       const nsACString& aName,
                                       const WindowActorOptions& aOptions,
                                       ErrorResult& aRv) {
-  MOZ_ASSERT(XRE_IsParentProcess());
+  if (!XRE_IsParentProcess()) {
+    aRv.ThrowNotAllowedError(
+        "registerWindowActor() may only be called in the parent process");
+    return;
+  }
 
   RefPtr<JSActorService> service = JSActorService::GetSingleton();
   service->RegisterWindowActor(aName, aOptions, aRv);
@@ -2165,8 +2336,13 @@ void ChromeUtils::RegisterWindowActor(const GlobalObject& aGlobal,
 
 /* static */
 void ChromeUtils::UnregisterWindowActor(const GlobalObject& aGlobal,
-                                        const nsACString& aName) {
-  MOZ_ASSERT(XRE_IsParentProcess());
+                                        const nsACString& aName,
+                                        ErrorResult& aRv) {
+  if (!XRE_IsParentProcess()) {
+    aRv.ThrowNotAllowedError(
+        "unregisterWindowActor() may only be called in the parent process");
+    return;
+  }
 
   RefPtr<JSActorService> service = JSActorService::GetSingleton();
   service->UnregisterWindowActor(aName);
@@ -2177,7 +2353,11 @@ void ChromeUtils::RegisterProcessActor(const GlobalObject& aGlobal,
                                        const nsACString& aName,
                                        const ProcessActorOptions& aOptions,
                                        ErrorResult& aRv) {
-  MOZ_ASSERT(XRE_IsParentProcess());
+  if (!XRE_IsParentProcess()) {
+    aRv.ThrowNotAllowedError(
+        "registerProcessActor() may only be called in the parent process");
+    return;
+  }
 
   RefPtr<JSActorService> service = JSActorService::GetSingleton();
   service->RegisterProcessActor(aName, aOptions, aRv);
@@ -2185,13 +2365,47 @@ void ChromeUtils::RegisterProcessActor(const GlobalObject& aGlobal,
 
 /* static */
 void ChromeUtils::UnregisterProcessActor(const GlobalObject& aGlobal,
-                                         const nsACString& aName) {
-  MOZ_ASSERT(XRE_IsParentProcess());
+                                         const nsACString& aName,
+                                         ErrorResult& aRv) {
+  if (!XRE_IsParentProcess()) {
+    aRv.ThrowNotAllowedError(
+        "unregisterProcessActor() may only be called in the parent process");
+    return;
+  }
 
   RefPtr<JSActorService> service = JSActorService::GetSingleton();
   service->UnregisterProcessActor(aName);
 }
 
+/* static */
+already_AddRefed<Promise> ChromeUtils::EnsureHeadlessContentProcess(
+    const GlobalObject& aGlobal, const nsACString& aRemoteType,
+    ErrorResult& aRv) {
+  if (!XRE_IsParentProcess()) {
+    aRv.ThrowNotAllowedError(
+        "ensureHeadlessContentProcess() may only be called in the parent "
+        "process");
+    return nullptr;
+  }
+
+  nsCOMPtr<nsIGlobalObject> global = do_QueryInterface(aGlobal.GetAsSupports());
+  RefPtr<Promise> promise = Promise::Create(global, aRv);
+  if (aRv.Failed()) {
+    return nullptr;
+  }
+
+  ContentParent::GetNewOrUsedBrowserProcessAsync(aRemoteType)
+      ->Then(
+          GetCurrentSerialEventTarget(), __func__,
+          [promise](UniqueContentParentKeepAlive&& aKeepAlive) {
+            nsCOMPtr<nsIContentParentKeepAlive> jsKeepAlive =
+                WrapContentParentKeepAliveForJS(std::move(aKeepAlive));
+            promise->MaybeResolve(jsKeepAlive);
+          },
+          [promise](nsresult aError) { promise->MaybeReject(aError); });
+  return promise.forget();
+}
+
 /* static */
 bool ChromeUtils::IsClassifierBlockingErrorCode(GlobalObject& aGlobal,
                                                 uint32_t aError) {
@@ -2321,19 +2535,20 @@ bool ChromeUtils::ShouldResistFingerprinting(
     nsIRFPTargetSetIDL* aOverriddenFingerprintingSettings,
     const Optional<bool>& aIsPBM) {
   RFPTarget target;
+#define JSRFP_TARGET_TO_RFP_TARGET(rfptarget) \
+  case JSRFPTarget::rfptarget:                \
+    target = RFPTarget::rfptarget;            \
+    break;
   switch (aTarget) {
-    case JSRFPTarget::RoundWindowSize:
-      target = RFPTarget::RoundWindowSize;
-      break;
-    case JSRFPTarget::SiteSpecificZoom:
-      target = RFPTarget::SiteSpecificZoom;
-      break;
-    case JSRFPTarget::CSSPrefersColorScheme:
-      target = RFPTarget::CSSPrefersColorScheme;
-      break;
+    JSRFP_TARGET_TO_RFP_TARGET(RoundWindowSize);
+    JSRFP_TARGET_TO_RFP_TARGET(SiteSpecificZoom);
+    JSRFP_TARGET_TO_RFP_TARGET(CSSPrefersColorScheme);
+    JSRFP_TARGET_TO_RFP_TARGET(JSLocalePrompt);
+    JSRFP_TARGET_TO_RFP_TARGET(HttpUserAgent);
     default:
       MOZ_CRASH("Unhandled JSRFPTarget enum value");
   }
+#undef JSRFP_TARGET_TO_RFP_TARGET
 
   bool isPBM = false;
   if (aIsPBM.WasPassed()) {
@@ -2354,14 +2569,9 @@ bool ChromeUtils::ShouldResistFingerprinting(
 
   Maybe<RFPTargetSet> overriddenFingerprintingSettings;
   if (aOverriddenFingerprintingSettings) {
-    uint64_t low, hi;
-    aOverriddenFingerprintingSettings->GetLow(&low);
-    aOverriddenFingerprintingSettings->GetHigh(&hi);
-    std::bitset<128> bitset;
-    bitset |= hi;
-    bitset <<= 64;
-    bitset |= low;
-    overriddenFingerprintingSettings.emplace(RFPTargetSet(bitset));
+    overriddenFingerprintingSettings.emplace(
+        static_cast<nsRFPTargetSetIDL*>(aOverriddenFingerprintingSettings)
+            ->ToRFPTargetSet());
   }
 
   // This global object appears to be the global window, not for individual
@@ -2372,15 +2582,63 @@ bool ChromeUtils::ShouldResistFingerprinting(
 }
 
 /* static */
-void ChromeUtils::SanitizeTelemetryFileURL(
-    GlobalObject& aGlobal, const nsACString& aURL,
-    FileNameTypeDetails& aFileTypeDetails) {
-  FilenameTypeAndDetails result =
-      nsContentSecurityUtils::FilenameToFilenameType(aURL, true);
+void ChromeUtils::CallFunctionAndLogException(
+    GlobalObject& aGlobal, JS::Handle<JS::Value> aTargetGlobal,
+    JS::Handle<JS::Value> aFunction, JS::MutableHandle<JS::Value> aRetVal,
+    ErrorResult& aRv) {
+  JSContext* cx = aGlobal.Context();
+  if (!aTargetGlobal.isObject() || !aFunction.isObject()) {
+    aRv.Throw(NS_ERROR_INVALID_ARG);
+    return;
+  }
 
-  aFileTypeDetails.mFileNameType = result.first;
-  if (result.second.isSome()) {
-    aFileTypeDetails.mFileNameDetails.Construct(*result.second);
+  JS::Rooted<JS::Realm*> contextRealm(cx, JS::GetCurrentRealmOrNull(cx));
+  if (!contextRealm) {
+    aRv.Throw(NS_ERROR_INVALID_ARG);
+    return;
+  }
+
+  JS::Rooted<JSObject*> global(
+      cx, js::CheckedUnwrapDynamic(&aTargetGlobal.toObject(), cx));
+  if (!global) {
+    aRv.Throw(NS_ERROR_INVALID_ARG);
+    return;
+  }
+
+  // Use AutoJSAPI in order to trigger AutoJSAPI::ReportException
+  // which will do most of the work required for this function.
+  //
+  // We only have to pick the right global for which we want to flag
+  // the exception against.
+  dom::AutoJSAPI jsapi;
+  if (!jsapi.Init(global)) {
+    aRv.Throw(NS_ERROR_UNEXPECTED);
+    return;
+  }
+  JSContext* ccx = jsapi.cx();
+
+  // AutoJSAPI picks `aTargetGlobal` as execution compartment
+  // whereas we expect to run `aFunction` from the callsites compartment.
+  JSAutoRealm ar(ccx, JS::GetRealmGlobalOrNull(contextRealm));
+
+  JS::Rooted<JS::Value> funVal(ccx, aFunction);
+  if (!JS_WrapValue(ccx, &funVal)) {
+    aRv.Throw(NS_ERROR_FAILURE);
+    return;
+  }
+  if (!JS_CallFunctionValue(ccx, nullptr, funVal, JS::HandleValueArray::empty(),
+                            aRetVal)) {
+    // Ensure re-throwing the exception which may have been thrown by
+    // `aFunction`
+    if (JS_IsExceptionPending(ccx)) {
+      JS::Rooted<JS::Value> exception(cx);
+      if (JS_GetPendingException(ccx, &exception)) {
+        if (JS_WrapValue(cx, &exception)) {
+          aRv.MightThrowJSException();
+          aRv.ThrowJSException(cx, exception);
+        }
+      }
+    }
   }
 }
 
@@ -2407,6 +2665,11 @@ void ChromeUtils::NotifyDevToolsClosed(GlobalObject& aGlobal) {
   ChromeUtils::sDevToolsOpenedCount--;
 }
 
+/* static */
+bool ChromeUtils::IsJSIdentifier(GlobalObject& aGlobal, const nsAString& aStr) {
+  return JS_IsIdentifier(aStr.BeginReading(), aStr.Length());
+}
+
 #ifdef MOZ_WMF_CDM
 /* static */
 already_AddRefed<Promise> ChromeUtils::GetWMFContentDecryptionModuleInformation(
@@ -2436,4 +2699,17 @@ already_AddRefed<Promise> ChromeUtils::GetGMPContentDecryptionModuleInformation(
   return domPromise.forget();
 }
 
+void ChromeUtils::AndroidMoveTaskToBack(GlobalObject& aGlobal) {
+#ifdef MOZ_WIDGET_ANDROID
+  MOZ_RELEASE_ASSERT(XRE_IsParentProcess());
+  java::GeckoAppShell::MoveTaskToBack();
+#endif
+}
+
+already_AddRefed<nsIContentSecurityPolicy> ChromeUtils::CreateCSPFromHeader(
+    GlobalObject& aGlobal, const nsAString& aHeader, nsIURI* aSelfURI,
+    nsIPrincipal* aLoadingPrincipal, ErrorResult& aRv) {
+  return CSP_CreateFromHeader(aHeader, aSelfURI, aLoadingPrincipal, aRv);
+}
+
 }  // namespace mozilla::dom
