@@ -119,6 +119,7 @@ struct URLValue;
 namespace dom {
 struct CheckVisibilityOptions;
 struct CustomElementData;
+struct SetHTMLUnsafeOptions;
 struct SetHTMLOptions;
 struct GetHTMLOptions;
 struct GetAnimationsOptions;
@@ -139,12 +140,14 @@ class PopoverData;
 class Promise;
 class Sanitizer;
 class ShadowRoot;
+class StylePropertyMapReadOnly;
 class TrustedHTMLOrString;
 class UnrestrictedDoubleOrKeyframeAnimationOptions;
 template <typename T>
 class Optional;
 enum class CallerType : uint32_t;
 enum class ReferrerPolicy : uint8_t;
+enum class FetchPriority : uint8_t;
 }  // namespace dom
 }  // namespace mozilla
 
@@ -274,21 +277,6 @@ class TrustedHTMLOrTrustedScriptOrTrustedScriptURLOrString;
     ExplicitlySetAttrElements(nsGkAtoms::attr, aElements);                  \
   }
 
-// TODO(keithamus): Reference the spec link once merged.
-// https://github.com/whatwg/html/pull/9841/files#diff-41cf6794ba4200b839c53531555f0f3998df4cbb01a4d5cb0b94e3ca5e23947dR86024
-enum class InvokeAction : uint8_t {
-  Invalid,
-  Custom,
-  Auto,
-  TogglePopover,
-  ShowPopover,
-  HidePopover,
-  ShowModal,
-  Toggle,
-  Close,
-  Open,
-};
-
 class Element : public FragmentOrElement {
  public:
 #ifdef MOZILLA_INTERNAL_API
@@ -307,7 +295,7 @@ class Element : public FragmentOrElement {
 
 #endif  // MOZILLA_INTERNAL_API
 
-  NS_DECLARE_STATIC_IID_ACCESSOR(NS_ELEMENT_IID)
+  NS_INLINE_DECL_STATIC_IID(NS_ELEMENT_IID)
 
   NS_DECL_ADDSIZEOFEXCLUDINGTHIS
 
@@ -1070,12 +1058,12 @@ class Element : public FragmentOrElement {
    * Get the namespace / name / prefix of a given attribute.
    *
    * @param   aIndex the index of the attribute name
-   * @returns The name at the given index, or null if the index is
-   *          out-of-bounds.
+   * @returns The name at the given index.
    * @note    The document returned by NodeInfo()->GetDocument() (if one is
    *          present) is *not* necessarily the owner document of the element.
    * @note    The pointer returned by this function is only valid until the
    *          next call of either GetAttrNameAt or SetAttr on the element.
+   * @note    This will crash if the index is invalid.
    */
   const nsAttrName* GetAttrNameAt(uint32_t aIndex) const {
     return mAttrs.GetSafeAttrNameAt(aIndex);
@@ -1088,6 +1076,14 @@ class Element : public FragmentOrElement {
     return mAttrs.AttrNameAt(aIndex);
   }
 
+  /**
+   * A fallible overload of GetAttrNameAt.
+   */
+  [[nodiscard]] bool GetAttrNameAt(uint32_t aIndex,
+                                   const nsAttrName** aResult) const {
+    return mAttrs.GetSafeAttrNameAt(aIndex, aResult);
+  }
+
   /**
    * Gets the attribute info (name and value) for this element at a given index.
    */
@@ -1161,21 +1157,33 @@ class Element : public FragmentOrElement {
     return FindAttributeDependence(aAttribute, aMaps, N);
   }
 
-  virtual bool IsValidInvokeAction(InvokeAction aAction) const {
-    return aAction == InvokeAction::Auto;
-  }
+  // https://html.spec.whatwg.org/#attr-button-command
+  enum class Command : uint8_t {
+    Invalid,
+    Custom,
+    TogglePopover,
+    ShowPopover,
+    HidePopover,
+    ShowModal,
+    RequestClose,
+    Toggle,
+    Close,
+    Open,
+  };
+
+  virtual bool IsValidCommandAction(Command aCommand) const { return false; }
 
   /**
    * Elements can provide their own default behaviours for "Invoke" (see
-   * invoketarget/invokeaction attributes).
+   * command/commandfor attributes).
    * If the action is not recognised, they can choose to ignore it and `return
    * false`. If an action is recognised then they should `return true` to
    * indicate to sub-classes that this has been handled and no further steps
    * should be run.
    */
-  MOZ_CAN_RUN_SCRIPT virtual bool HandleInvokeInternal(Element* invoker,
-                                                       InvokeAction aAction,
-                                                       ErrorResult& aRv) {
+  MOZ_CAN_RUN_SCRIPT virtual bool HandleCommandInternal(Element* aSource,
+                                                        Command aCommand,
+                                                        ErrorResult& aRv) {
     return false;
   }
 
@@ -1429,13 +1437,14 @@ class Element : public FragmentOrElement {
   already_AddRefed<Promise> RequestFullscreen(CallerType, ErrorResult&);
   void RequestPointerLock(CallerType aCallerType);
   Attr* GetAttributeNode(const nsAString& aName);
-  already_AddRefed<Attr> SetAttributeNode(Attr& aNewAttr, ErrorResult& aError);
+  MOZ_CAN_RUN_SCRIPT already_AddRefed<Attr> SetAttributeNode(
+      Attr& aNewAttr, nsIPrincipal* aSubjectPrincipal, ErrorResult& aError);
   already_AddRefed<Attr> RemoveAttributeNode(Attr& aOldAttr,
                                              ErrorResult& aError);
   Attr* GetAttributeNodeNS(const nsAString& aNamespaceURI,
                            const nsAString& aLocalName);
-  already_AddRefed<Attr> SetAttributeNodeNS(Attr& aNewAttr,
-                                            ErrorResult& aError);
+  MOZ_CAN_RUN_SCRIPT already_AddRefed<Attr> SetAttributeNodeNS(
+      Attr& aNewAttr, nsIPrincipal* aSubjectPrincipal, ErrorResult& aError);
 
   MOZ_CAN_RUN_SCRIPT already_AddRefed<DOMRectList> GetClientRects();
   MOZ_CAN_RUN_SCRIPT already_AddRefed<DOMRect> GetBoundingClientRect();
@@ -1566,10 +1575,10 @@ class Element : public FragmentOrElement {
   MOZ_CAN_RUN_SCRIPT void ScrollTo(const ScrollToOptions& aOptions);
   MOZ_CAN_RUN_SCRIPT void ScrollBy(double aXScrollDif, double aYScrollDif);
   MOZ_CAN_RUN_SCRIPT void ScrollBy(const ScrollToOptions& aOptions);
-  MOZ_CAN_RUN_SCRIPT int32_t ScrollTop();
-  MOZ_CAN_RUN_SCRIPT void SetScrollTop(int32_t aScrollTop);
-  MOZ_CAN_RUN_SCRIPT int32_t ScrollLeft();
-  MOZ_CAN_RUN_SCRIPT void SetScrollLeft(int32_t aScrollLeft);
+  MOZ_CAN_RUN_SCRIPT double ScrollTop();
+  MOZ_CAN_RUN_SCRIPT void SetScrollTop(double aScrollTop);
+  MOZ_CAN_RUN_SCRIPT double ScrollLeft();
+  MOZ_CAN_RUN_SCRIPT void SetScrollLeft(double aScrollLeft);
   MOZ_CAN_RUN_SCRIPT int32_t ScrollWidth();
   MOZ_CAN_RUN_SCRIPT int32_t ScrollHeight();
   MOZ_CAN_RUN_SCRIPT void MozScrollSnap();
@@ -1590,10 +1599,10 @@ class Element : public FragmentOrElement {
   MOZ_CAN_RUN_SCRIPT int32_t ScreenY();
   MOZ_CAN_RUN_SCRIPT already_AddRefed<nsIScreen> GetScreen();
 
-  MOZ_CAN_RUN_SCRIPT int32_t ScrollTopMin();
-  MOZ_CAN_RUN_SCRIPT int32_t ScrollTopMax();
-  MOZ_CAN_RUN_SCRIPT int32_t ScrollLeftMin();
-  MOZ_CAN_RUN_SCRIPT int32_t ScrollLeftMax();
+  MOZ_CAN_RUN_SCRIPT double ScrollTopMin();
+  MOZ_CAN_RUN_SCRIPT double ScrollTopMax();
+  MOZ_CAN_RUN_SCRIPT double ScrollLeftMin();
+  MOZ_CAN_RUN_SCRIPT double ScrollLeftMax();
 
   MOZ_CAN_RUN_SCRIPT double ClientHeightDouble() {
     return CSSPixel::FromAppUnits(GetClientAreaRect().Height());
@@ -1605,6 +1614,38 @@ class Element : public FragmentOrElement {
 
   MOZ_CAN_RUN_SCRIPT double CurrentCSSZoom();
 
+  Element* GetOffsetParent() {
+    CSSIntRect rcFrame;
+    return GetOffsetRect(rcFrame);
+  }
+  int32_t OffsetTop() {
+    CSSIntRect rcFrame;
+    GetOffsetRect(rcFrame);
+    return rcFrame.y;
+  }
+  int32_t OffsetLeft() {
+    CSSIntRect rcFrame;
+    GetOffsetRect(rcFrame);
+    return rcFrame.x;
+  }
+  int32_t OffsetWidth() {
+    CSSIntRect rcFrame;
+    GetOffsetRect(rcFrame);
+    return rcFrame.Width();
+  }
+  int32_t OffsetHeight() {
+    CSSIntRect rcFrame;
+    GetOffsetRect(rcFrame);
+    return rcFrame.Height();
+  }
+  /**
+   * Get the frame's offset information for offsetTop/Left/Width/Height.
+   * Returns the parent the offset is relative to.
+   * @note This method flushes pending notifications (FlushType::Layout).
+   * @param aRect the offset information [OUT]
+   */
+  Element* GetOffsetRect(CSSIntRect& aRect);
+
   // This function will return the block size of first line box, no matter if
   // the box is 'block' or 'inline'. The return unit is pixel. If the element
   // can't get a primary frame, we will return be zero.
@@ -1658,16 +1699,27 @@ class Element : public FragmentOrElement {
   void GetOuterHTML(OwningTrustedHTMLOrNullIsEmptyString& aOuterHTML);
 
   MOZ_CAN_RUN_SCRIPT void SetOuterHTML(
-      const TrustedHTMLOrNullIsEmptyString& aOuterHTML, ErrorResult& aError);
+      const TrustedHTMLOrNullIsEmptyString& aOuterHTML,
+      nsIPrincipal* aSubjectPrincipal, ErrorResult& aError);
 
   MOZ_CAN_RUN_SCRIPT void InsertAdjacentHTML(
       const nsAString& aPosition,
-      const TrustedHTMLOrString& aTrustedHTMLOrString, ErrorResult& aError);
+      const TrustedHTMLOrString& aTrustedHTMLOrString,
+      nsIPrincipal* aSubjectPrincipal, ErrorResult& aError);
+
+  virtual void SetHTML(const nsAString& aInnerHTML,
+                       const SetHTMLOptions& aOptions, ErrorResult& aError);
+
+  MOZ_CAN_RUN_SCRIPT
+  virtual void SetHTMLUnsafe(const TrustedHTMLOrString& aHTML,
+                             const SetHTMLUnsafeOptions& aOptions,
+                             nsIPrincipal* aSubjectPrincipal,
+                             ErrorResult& aError);
 
-  void SetHTML(const nsAString& aInnerHTML, const SetHTMLOptions& aOptions,
-               ErrorResult& aError);
   void GetHTML(const GetHTMLOptions& aOptions, nsAString& aResult);
 
+  StylePropertyMapReadOnly* ComputedStyleMap();
+
   //----------------------------------------
 
   /**
@@ -2207,6 +2259,10 @@ class Element : public FragmentOrElement {
   MOZ_CAN_RUN_SCRIPT
   nsresult PostHandleEventForLinks(EventChainPostVisitor& aVisitor);
 
+  mozilla::dom::FetchPriority GetFetchPriority() const;
+
+  static void ParseFetchPriority(const nsAString& aValue, nsAttrValue& aResult);
+
  public:
   /**
    * Check if this element is a link. This matches the CSS definition of the
@@ -2253,8 +2309,7 @@ class Element : public FragmentOrElement {
   virtual bool Translate() const;
 
   MOZ_CAN_RUN_SCRIPT
-  virtual void SetHTMLUnsafe(const TrustedHTMLOrString& aHTML,
-                             ErrorResult& aError);
+  void FireBeforematchEvent(ErrorResult& aRv);
 
  protected:
   enum class ReparseAttributes { No, Yes };
@@ -2327,8 +2382,6 @@ class Element : public FragmentOrElement {
   AttrArray mAttrs;
 };
 
-NS_DEFINE_STATIC_IID_ACCESSOR(Element, NS_ELEMENT_IID)
-
 inline bool Element::HasNonEmptyAttr(int32_t aNameSpaceID,
                                      const nsAtom* aName) const {
   MOZ_ASSERT(aNameSpaceID > kNameSpaceID_Unknown, "Must have namespace");
