@@ -14,6 +14,7 @@
 #include "mozilla/StaticPtr.h"
 
 #include "mozilla/dom/FragmentOrElement.h"
+#include "mozilla/dom/AncestorIterator.h"
 #include "DOMIntersectionObserver.h"
 #include "mozilla/AsyncEventDispatcher.h"
 #include "mozilla/EffectSet.h"
@@ -40,6 +41,8 @@
 #include "mozilla/dom/CustomElementRegistry.h"
 #include "mozilla/dom/Document.h"
 #include "mozilla/dom/DocumentInlines.h"
+#include "mozilla/dom/StylePropertyMap.h"
+#include "mozilla/dom/StylePropertyMapReadOnly.h"
 #include "nsIControllers.h"
 #include "nsIDocumentEncoder.h"
 #include "nsFocusManager.h"
@@ -141,12 +144,12 @@ NS_IMPL_CYCLE_COLLECTING_RELEASE_WITH_LAST_RELEASE_AND_DESTROY(nsIContent,
 
 nsIContent* nsIContent::FindFirstNonChromeOnlyAccessContent() const {
   // This handles also nested native anonymous content.
-  for (const nsIContent* content = this; content;
-       content = content->GetChromeOnlyAccessSubtreeRootParent()) {
+  // Oops, this function signature allows casting const to non-const.  (Then
+  // again, so does GetFirstChild()->GetParent().)
+  for (nsIContent* content = const_cast<nsIContent*>(this); content;
+       content = content->GetClosestNativeAnonymousSubtreeRootParentOrHost()) {
     if (!content->ChromeOnlyAccess()) {
-      // Oops, this function signature allows casting const to
-      // non-const.  (Then again, so does GetFirstChild()->GetParent().)
-      return const_cast<nsIContent*>(content);
+      return content;
     }
   }
   return nullptr;
@@ -238,6 +241,8 @@ dom::Element* nsIContent::GetEditingHost() {
 
   // If this is in designMode, we should return <body>
   if (IsInDesignMode() && !IsInShadowTree()) {
+    // FIXME: There may be no <body>.  In such case and aLimitInBodyElement is
+    // "No", we should use root element instead.
     return doc->GetBodyElement();
   }
 
@@ -570,6 +575,12 @@ void FragmentOrElement::nsDOMSlots::Traverse(
 
   NS_CYCLE_COLLECTION_NOTE_EDGE_NAME(aCb, "mSlots->mClassList");
   aCb.NoteXPCOMChild(mClassList.get());
+
+  NS_CYCLE_COLLECTION_NOTE_EDGE_NAME(aCb, "mSlots->mComputedStyleMap");
+  aCb.NoteXPCOMChild(mComputedStyleMap.get());
+
+  NS_CYCLE_COLLECTION_NOTE_EDGE_NAME(aCb, "mSlots->mAttributeStyleMap");
+  aCb.NoteXPCOMChild(mAttributeStyleMap.get());
 }
 
 void FragmentOrElement::nsDOMSlots::Unlink(nsINode& aNode) {
@@ -581,6 +592,8 @@ void FragmentOrElement::nsDOMSlots::Unlink(nsINode& aNode) {
   }
   mChildrenList = nullptr;
   mClassList = nullptr;
+  mComputedStyleMap = nullptr;
+  mAttributeStyleMap = nullptr;
 }
 
 size_t FragmentOrElement::nsDOMSlots::SizeOfIncludingThis(
@@ -604,6 +617,14 @@ size_t FragmentOrElement::nsDOMSlots::SizeOfIncludingThis(
     n += mChildrenList->SizeOfIncludingThis(aMallocSizeOf);
   }
 
+  if (mComputedStyleMap) {
+    n += mComputedStyleMap->SizeOfIncludingThis(aMallocSizeOf);
+  }
+
+  if (mAttributeStyleMap) {
+    n += mAttributeStyleMap->SizeOfIncludingThis(aMallocSizeOf);
+  }
+
   // Measurement of the following members may be added later if DMD finds it is
   // worthwhile:
   // - Superclass members (nsINode::nsSlots)
@@ -637,7 +658,8 @@ void FragmentOrElement::nsExtendedDOMSlots::UnlinkExtendedSlots(
     mAnimations = nullptr;
     aContent.ClearMayHaveAnimations();
   }
-  mExplicitlySetAttrElements.Clear();
+  mExplicitlySetAttrElementMap.Clear();
+  mAttrElementsMap.Clear();
   mRadioGroupContainer = nullptr;
   mPart = nullptr;
 }
@@ -661,6 +683,15 @@ void FragmentOrElement::nsExtendedDOMSlots::TraverseExtendedSlots(
   NS_CYCLE_COLLECTION_NOTE_EDGE_NAME(aCb, "mSlots->mPart");
   aCb.NoteXPCOMChild(mPart.get());
 
+  for (auto& tableEntry : mAttrElementsMap) {
+    auto& [explicitlySetElements, cachedAttrElements] =
+        *tableEntry.GetModifiableData();
+    if (cachedAttrElements) {
+      ImplCycleCollectionTraverse(aCb, *cachedAttrElements,
+                                  "cached attribute elements entry", 0);
+    }
+  }
+
   if (mCustomElementData) {
     mCustomElementData->Traverse(aCb);
   }
@@ -723,19 +754,19 @@ FragmentOrElement::~FragmentOrElement() {
   }
 }
 
-static nsINode* FindChromeAccessOnlySubtreeOwner(nsINode* aNode) {
-  if (!aNode->ChromeOnlyAccess()) {
+static nsINode* FindChromeAccessOnlySubtreeOwnerForEvents(nsINode* aNode) {
+  if (!aNode->ChromeOnlyAccessForEvents()) {
     return aNode;
   }
-  return const_cast<nsIContent*>(aNode->GetChromeOnlyAccessSubtreeRootParent());
+  return aNode->GetClosestNativeAnonymousSubtreeRootParentOrHost();
 }
 
-nsINode* FindChromeAccessOnlySubtreeOwner(EventTarget* aTarget) {
+nsINode* FindChromeAccessOnlySubtreeOwnerForEvents(EventTarget* aTarget) {
   nsINode* node = nsINode::FromEventTargetOrNull(aTarget);
   if (!node) {
     return nullptr;
   }
-  return FindChromeAccessOnlySubtreeOwner(node);
+  return FindChromeAccessOnlySubtreeOwnerForEvents(node);
 }
 
 void nsIContent::GetEventTargetParent(EventChainPreVisitor& aVisitor) {
@@ -772,20 +803,20 @@ void nsIContent::GetEventTargetParent(EventChainPreVisitor& aVisitor) {
       if (isAnonForEvents || aVisitor.mRelatedTargetIsInAnon ||
           (aVisitor.mEvent->mOriginalTarget == this &&
            (aVisitor.mRelatedTargetIsInAnon =
-                relatedTarget->ChromeOnlyAccess()))) {
-        nsINode* anonOwner = FindChromeAccessOnlySubtreeOwner(this);
+                relatedTarget->ChromeOnlyAccessForEvents()))) {
+        nsINode* anonOwner = FindChromeAccessOnlySubtreeOwnerForEvents(this);
         if (anonOwner) {
           nsINode* anonOwnerRelated =
-              FindChromeAccessOnlySubtreeOwner(relatedTarget);
+              FindChromeAccessOnlySubtreeOwnerForEvents(relatedTarget);
           if (anonOwnerRelated) {
             // Note, anonOwnerRelated may still be inside some other
             // native anonymous subtree. The case where anonOwner is still
             // inside native anonymous subtree will be handled when event
             // propagates up in the DOM tree.
             while (anonOwner != anonOwnerRelated &&
-                   anonOwnerRelated->ChromeOnlyAccess()) {
+                   anonOwnerRelated->ChromeOnlyAccessForEvents()) {
               anonOwnerRelated =
-                  FindChromeAccessOnlySubtreeOwner(anonOwnerRelated);
+                  FindChromeAccessOnlySubtreeOwnerForEvents(anonOwnerRelated);
             }
             if (anonOwner == anonOwnerRelated) {
 #ifdef DEBUG_smaug
@@ -843,17 +874,11 @@ void nsIContent::GetEventTargetParent(EventChainPreVisitor& aVisitor) {
     // all the events are allowed even in the native anonymous content..
     nsIContent* t =
         nsIContent::FromEventTargetOrNull(aVisitor.mEvent->mOriginalTarget);
-    NS_ASSERTION(!t || !t->ChromeOnlyAccess() ||
+    NS_ASSERTION(!t || !t->ChromeOnlyAccessForEvents() ||
                      aVisitor.mEvent->mClass != eMutationEventClass ||
                      aVisitor.mDOMEvent,
                  "Mutation event dispatched in native anonymous content!?!");
 #endif
-    if (aVisitor.mEvent->mClass == eTransitionEventClass ||
-        aVisitor.mEvent->mClass == eAnimationEventClass) {
-      // Event should not propagate to non-anon content.
-      aVisitor.SetParentTarget(nullptr, false);
-      return;
-    }
     aVisitor.mEventTargetAtParent = parent;
   } else if (parent && aVisitor.mOriginalTargetIsInAnon) {
     nsIContent* content =
@@ -865,7 +890,7 @@ void nsIContent::GetEventTargetParent(EventChainPreVisitor& aVisitor) {
   }
 
   if (!aVisitor.mEvent->mFlags.mComposedInNativeAnonymousContent &&
-      IsRootOfNativeAnonymousSubtree() && OwnerDoc()->GetWindow()) {
+      isAnonForEvents && OwnerDoc()->GetWindow()) {
     aVisitor.SetParentTarget(OwnerDoc()->GetWindow()->GetParentTarget(), true);
   } else if (parent) {
     aVisitor.SetParentTarget(parent, false);
@@ -879,7 +904,8 @@ void nsIContent::GetEventTargetParent(EventChainPreVisitor& aVisitor) {
     aVisitor.SetParentTarget(GetComposedDoc(), false);
   }
 
-  if (!ChromeOnlyAccess() && !aVisitor.mRelatedTargetRetargetedInCurrentScope) {
+  if (!ChromeOnlyAccessForEvents() &&
+      !aVisitor.mRelatedTargetRetargetedInCurrentScope) {
     // We don't support Shadow DOM in native anonymous content yet.
     aVisitor.mRelatedTargetRetargetedInCurrentScope = true;
     if (aVisitor.mEvent->mOriginalRelatedTarget) {
@@ -893,16 +919,16 @@ void nsIContent::GetEventTargetParent(EventChainPreVisitor& aVisitor) {
       // anonymous content, but we need to deal with non-native anonymous
       // content there.
       bool initialTarget = this == aVisitor.mEvent->mOriginalTarget;
-      nsCOMPtr<nsINode> originalTargetAsNode;
+      nsINode* originalTargetAsNode = nullptr;
       // Use of mOriginalTargetIsInAnon is an optimization here.
       if (!initialTarget && aVisitor.mOriginalTargetIsInAnon) {
-        originalTargetAsNode =
-            FindChromeAccessOnlySubtreeOwner(aVisitor.mEvent->mOriginalTarget);
+        originalTargetAsNode = FindChromeAccessOnlySubtreeOwnerForEvents(
+            aVisitor.mEvent->mOriginalTarget);
         initialTarget = originalTargetAsNode == this;
       }
       if (initialTarget) {
-        nsCOMPtr<nsINode> relatedTargetAsNode =
-            FindChromeAccessOnlySubtreeOwner(
+        nsINode* relatedTargetAsNode =
+            FindChromeAccessOnlySubtreeOwnerForEvents(
                 aVisitor.mEvent->mOriginalRelatedTarget);
         if (!originalTargetAsNode) {
           originalTargetAsNode =
@@ -932,52 +958,49 @@ void nsIContent::GetEventTargetParent(EventChainPreVisitor& aVisitor) {
           //  relatedTarget, and false."
           aVisitor.mRetargetedRelatedTarget = retargetedRelatedTarget;
         }
-      } else {
-        nsCOMPtr<nsINode> relatedTargetAsNode =
-            FindChromeAccessOnlySubtreeOwner(
-                aVisitor.mEvent->mOriginalRelatedTarget);
-        if (relatedTargetAsNode) {
-          // Step 11.3.
-          // "Let relatedTarget be the result of retargeting event's
-          // relatedTarget against parent if event's relatedTarget is non-null,
-          // and null otherwise.".
-          nsINode* retargetedRelatedTarget =
-              nsContentUtils::Retarget(relatedTargetAsNode, this);
-          nsCOMPtr<nsINode> targetInKnownToBeHandledScope =
-              FindChromeAccessOnlySubtreeOwner(
-                  aVisitor.mTargetInKnownToBeHandledScope);
-          // If aVisitor.mTargetInKnownToBeHandledScope wasn't nsINode,
-          // targetInKnownToBeHandledScope will be null. This may happen when
-          // dispatching event to Window object in a content page and
-          // propagating the event to a chrome Element.
-          if (targetInKnownToBeHandledScope &&
-              IsShadowIncludingInclusiveDescendantOf(
-                  targetInKnownToBeHandledScope->SubtreeRoot())) {
-            // Part of step 11.4.
-            // "If target's root is a shadow-including inclusive ancestor of
-            //  parent, then"
-            // "...Append to an event path with event, parent, null,
-            // relatedTarget, "   and slot-in-closed-tree."
-            aVisitor.mRetargetedRelatedTarget = retargetedRelatedTarget;
-          } else if (this == retargetedRelatedTarget) {
-            // Step 11.5
-            // "Otherwise, if parent and relatedTarget are identical, then set
-            //  parent to null."
-            aVisitor.IgnoreCurrentTargetBecauseOfShadowDOMRetargeting();
-            // Old code relies on mTarget to point to the first element which
-            // was not added to the event target chain because of mCanHandle
-            // being false, but in Shadow DOM case mTarget really should
-            // point to a node in Shadow DOM.
-            aVisitor.mEvent->mTarget = aVisitor.mTargetInKnownToBeHandledScope;
-            return;
-          } else if (targetInKnownToBeHandledScope) {
-            // Note, if targetInKnownToBeHandledScope is null,
-            // mTargetInKnownToBeHandledScope could be Window object in content
-            // page and we're in chrome document in the same process.
+      } else if (nsINode* relatedTargetAsNode =
+                     FindChromeAccessOnlySubtreeOwnerForEvents(
+                         aVisitor.mEvent->mOriginalRelatedTarget)) {
+        // Step 11.3.
+        // "Let relatedTarget be the result of retargeting event's
+        // relatedTarget against parent if event's relatedTarget is non-null,
+        // and null otherwise.".
+        nsINode* retargetedRelatedTarget =
+            nsContentUtils::Retarget(relatedTargetAsNode, this);
+        nsINode* targetInKnownToBeHandledScope =
+            FindChromeAccessOnlySubtreeOwnerForEvents(
+                aVisitor.mTargetInKnownToBeHandledScope);
+        // If aVisitor.mTargetInKnownToBeHandledScope wasn't nsINode,
+        // targetInKnownToBeHandledScope will be null. This may happen when
+        // dispatching event to Window object in a content page and
+        // propagating the event to a chrome Element.
+        if (targetInKnownToBeHandledScope &&
+            IsShadowIncludingInclusiveDescendantOf(
+                targetInKnownToBeHandledScope->SubtreeRoot())) {
+          // Part of step 11.4.
+          // "If target's root is a shadow-including inclusive ancestor of
+          //  parent, then"
+          // "...Append to an event path with event, parent, null,
+          // relatedTarget, "   and slot-in-closed-tree."
+          aVisitor.mRetargetedRelatedTarget = retargetedRelatedTarget;
+        } else if (this == retargetedRelatedTarget) {
+          // Step 11.5
+          // "Otherwise, if parent and relatedTarget are identical, then set
+          //  parent to null."
+          aVisitor.IgnoreCurrentTargetBecauseOfShadowDOMRetargeting();
+          // Old code relies on mTarget to point to the first element which
+          // was not added to the event target chain because of mCanHandle
+          // being false, but in Shadow DOM case mTarget really should
+          // point to a node in Shadow DOM.
+          aVisitor.mEvent->mTarget = aVisitor.mTargetInKnownToBeHandledScope;
+          return;
+        } else if (targetInKnownToBeHandledScope) {
+          // Note, if targetInKnownToBeHandledScope is null,
+          // mTargetInKnownToBeHandledScope could be Window object in content
+          // page and we're in chrome document in the same process.
 
-            // Step 11.6
-            aVisitor.mRetargetedRelatedTarget = retargetedRelatedTarget;
-          }
+          // Step 11.6
+          aVisitor.mRetargetedRelatedTarget = retargetedRelatedTarget;
         }
       }
     }
@@ -1030,6 +1053,28 @@ Element* nsIContent::GetAutofocusDelegate(IsFocusableFlags aFlags) const {
   return nullptr;
 }
 
+bool nsIContent::CanStartSelectionAsWebCompatHack() const {
+  if (!StaticPrefs::dom_selection_mimic_chrome_tostring_enabled()) {
+    return true;
+  }
+
+  for (const nsIContent* content = this; content;
+       content = content->GetFlattenedTreeParent()) {
+    if (content->IsEditable()) {
+      return true;
+    }
+    nsIFrame* frame = content->GetPrimaryFrame();
+    if (!frame) {
+      return true;
+    }
+    if (!frame->IsSelectable(nullptr)) {
+      return false;
+    }
+  }
+
+  return true;
+}
+
 Element* nsIContent::GetFocusDelegate(IsFocusableFlags aFlags) const {
   const nsIContent* whereToLook = this;
   if (ShadowRoot* root = GetShadowRoot()) {
@@ -1819,13 +1864,13 @@ static inline bool IsVoidTag(nsAtom* aTag) {
   static bool sInitialized = false;
   if (!sInitialized) {
     sInitialized = true;
-    for (uint32_t i = 0; i < ArrayLength(voidElements); ++i) {
+    for (uint32_t i = 0; i < std::size(voidElements); ++i) {
       sFilter.add(voidElements[i]);
     }
   }
 
   if (sFilter.mightContain(aTag)) {
-    for (uint32_t i = 0; i < ArrayLength(voidElements); ++i) {
+    for (uint32_t i = 0; i < std::size(voidElements); ++i) {
       if (aTag == voidElements[i]) {
         return true;
       }
@@ -1960,9 +2005,7 @@ void FragmentOrElement::SetInnerHTMLInternal(const nsAString& aInnerHTML,
 
   // Remove childnodes.
   nsAutoMutationBatch mb(target, true, false);
-  while (target->HasChildren()) {
-    target->RemoveChildNode(target->GetFirstChild(), true);
-  }
+  target->RemoveAllChildren(true);
   mb.RemovalDone();
 
   nsAutoScriptLoaderDisabler sld(doc);
@@ -1985,7 +2028,8 @@ void FragmentOrElement::SetInnerHTMLInternal(const nsAString& aInnerHTML,
         doc->GetCompatibilityMode() == eCompatibility_NavQuirks, true);
     doc->ResumeDOMNotifications();
     if (target->GetFirstChild()) {
-      MutationObservers::NotifyContentAppended(target, target->GetFirstChild());
+      MutationObservers::NotifyContentAppended(target, target->GetFirstChild(),
+                                               {});
     }
     mb.NodesAdded();
     // HTML5 parser has notified, but not fired mutation events.
