@@ -6,28 +6,55 @@
 #include "TextDirectiveFinder.h"
 #include "Document.h"
 #include "TextDirectiveUtil.h"
+#include "mozilla/glean/DomMetrics.h"
+#include "nsFind.h"
 #include "nsRange.h"
 #include "fragmentdirectives_ffi_generated.h"
+#include "mozilla/CycleCollectedUniquePtr.h"
+#include "mozilla/ResultVariant.h"
 
 namespace mozilla::dom {
 
 TextDirectiveFinder::TextDirectiveFinder(
-    Document& aDocument, nsTArray<TextDirective>&& aTextDirectives)
-    : mDocument(aDocument),
+    Document* aDocument, nsTArray<TextDirective>&& aTextDirectives)
+    : mDocument(WrapNotNull(aDocument)),
       mUninvokedTextDirectives(std::move(aTextDirectives)) {}
 
+TextDirectiveFinder::~TextDirectiveFinder() {
+  if (mFoundDirectiveCount) {
+    glean::dom_textfragment::find_directives.AccumulateRawDuration(
+        mFindTextDirectivesDuration);
+
+    TEXT_FRAGMENT_LOG("Found {} directives in {}ms", mFoundDirectiveCount,
+                      mFindTextDirectivesDuration.ToMilliseconds());
+  }
+  if (HasUninvokedDirectives()) {
+    mDocument->SetUseCounter(eUseCounter_custom_InvalidTextDirectives);
+  }
+}
+
+void TextDirectiveFinder::Traverse(
+    nsCycleCollectionTraversalCallback& aCallback) {
+  CycleCollectionNoteChild(aCallback, mDocument.get().get(),
+                           "TextDirectiveFinder::mDocument", aCallback.Flags());
+}
+
 bool TextDirectiveFinder::HasUninvokedDirectives() const {
   return !mUninvokedTextDirectives.IsEmpty();
 }
+
 nsTArray<RefPtr<nsRange>> TextDirectiveFinder::FindTextDirectivesInDocument() {
   if (mUninvokedTextDirectives.IsEmpty()) {
     return {};
   }
-  auto uri = TextDirectiveUtil::ShouldLog() && mDocument.GetDocumentURI()
-                 ? mDocument.GetDocumentURI()->GetSpecOrDefault()
+
+  const TimeStamp start = TimeStamp::Now();
+
+  auto uri = TextDirectiveUtil::ShouldLog() && mDocument->GetDocumentURI()
+                 ? mDocument->GetDocumentURI()->GetSpecOrDefault()
                  : nsCString();
   TEXT_FRAGMENT_LOG("Trying to find text directives in document '{}'.", uri);
-  mDocument.FlushPendingNotifications(FlushType::Layout);
+  mDocument->FlushPendingNotifications(FlushType::Layout);
   // https://wicg.github.io/scroll-to-text-fragment/#invoke-text-directives
   // To invoke text directives, given as input a list of text directives text
   // directives and a Document document, run these steps:
@@ -78,22 +105,32 @@ nsTArray<RefPtr<nsRange>> TextDirectiveFinder::FindTextDirectivesInDocument() {
   }
   mUninvokedTextDirectives = std::move(uninvokedTextDirectives);
 
+  mFindTextDirectivesDuration += TimeStamp::Now() - start;
+  mFoundDirectiveCount += static_cast<int64_t>(textDirectiveRanges.Length());
+
   // 3. Return ranges.
   return textDirectiveRanges;
 }
 
 RefPtr<nsRange> TextDirectiveFinder::FindRangeForTextDirective(
     const TextDirective& aTextDirective) {
+  // This method follows this spec algorithm and applies some changes:
+  // https://wicg.github.io/scroll-to-text-fragment/#find-a-range-from-a-text-directive
   TEXT_FRAGMENT_LOG("Find range for text directive '{}'.",
                     ToString(aTextDirective).c_str());
   // 1. Let searchRange be a range with start (document, 0) and end (document,
   // document’s length)
   ErrorResult rv;
   RefPtr<nsRange> searchRange =
-      nsRange::Create(&mDocument, 0, &mDocument, mDocument.Length(), rv);
+      nsRange::Create(mDocument, 0, mDocument, mDocument->Length(), rv);
   if (rv.Failed()) {
     return nullptr;
   }
+
+  nsContentUtils::NodeIndexCache nodeIndexCache;
+  RefPtr<nsFind> finder = new nsFind();
+  finder->SetNodeIndexCache(&nodeIndexCache);
+
   // 2. While searchRange is not collapsed:
   while (!searchRange->Collapsed()) {
     // 2.1. Let potentialMatch be null.
@@ -104,7 +141,8 @@ RefPtr<nsRange> TextDirectiveFinder::FindRangeForTextDirective(
       // in range steps with query parsedValues’s prefix, searchRange
       // searchRange, wordStartBounded true and wordEndBounded false.
       RefPtr<nsRange> prefixMatch = TextDirectiveUtil::FindStringInRange(
-          searchRange, aTextDirective.prefix, true, false);
+          finder, searchRange->StartRef(), searchRange->EndRef(),
+          aTextDirective.prefix, true, false);
       // 2.2.2. If prefixMatch is null, return null.
       if (!prefixMatch) {
         TEXT_FRAGMENT_LOG(
@@ -118,10 +156,9 @@ RefPtr<nsRange> TextDirectiveFinder::FindRangeForTextDirective(
 
       // 2.2.3. Set searchRange’s start to the first boundary point after
       // prefixMatch’s start
+      MOZ_DIAGNOSTIC_ASSERT(prefixMatch->GetStartContainer()->IsText());
       const RangeBoundary boundaryPoint =
-          TextDirectiveUtil::MoveRangeBoundaryOneWord(
-              {prefixMatch->GetStartContainer(), prefixMatch->StartOffset()},
-              TextScanDirection::Right);
+          TextDirectiveUtil::MoveToNextBoundaryPoint(prefixMatch->StartRef());
       if (!boundaryPoint.IsSetAndValid()) {
         return nullptr;
       }
@@ -132,6 +169,12 @@ RefPtr<nsRange> TextDirectiveFinder::FindRangeForTextDirective(
 
       // 2.2.4. Let matchRange be a range whose start is prefixMatch’s end and
       // end is searchRange’s end.
+      // Note:
+      // The spec is very inefficient. The start text must _immediately_ follow
+      // after the end of the prefix. Therefore, it would be a huge waste to
+      // search until the end of the document. Since the following `start`
+      // attribute can't go across a block boundary, it is sufficient to do a
+      // search until the next block boundary.
       RefPtr<nsRange> matchRange = nsRange::Create(
           prefixMatch->GetEndContainer(), prefixMatch->EndOffset(),
           searchRange->GetEndContainer(), searchRange->EndOffset(), rv);
@@ -150,6 +193,12 @@ RefPtr<nsRange> TextDirectiveFinder::FindRangeForTextDirective(
       // (matchRange’s start now points to the next non-whitespace text data
       // following a matched prefix.)
       MOZ_ASSERT(matchRange->GetStartContainer()->IsText());
+      // Set `matchRange`s end to the next block boundary.
+      auto nextBlockBoundary =
+          TextDirectiveUtil::FindNextBlockBoundary<TextScanDirection::Right>(
+              matchRange->StartRef());
+
+      matchRange->SetEnd(nextBlockBoundary.AsRaw(), IgnoreErrors());
 
       // 2.2.8. Let mustEndAtWordBoundary be true if parsedValues’s end is
       // non-null or parsedValues’s suffix is null, false otherwise.
@@ -159,17 +208,19 @@ RefPtr<nsRange> TextDirectiveFinder::FindRangeForTextDirective(
       // range steps with query parsedValues’s start, searchRange matchRange,
       // wordStartBounded false, and wordEndBounded mustEndAtWordBoundary.
       potentialMatch = TextDirectiveUtil::FindStringInRange(
-          matchRange, aTextDirective.start, false, mustEndAtWordBoundary);
+          finder, matchRange->StartRef(), matchRange->EndRef(),
+          aTextDirective.start, false, mustEndAtWordBoundary);
       // 2.2.10. If potentialMatch is null, return null.
+      // Note: Because the search range for start only goes to the next block
+      // boundary, this statement is wrong. If potentialMatch is null, the loop
+      // needs to be restarted.
       if (!potentialMatch) {
         TEXT_FRAGMENT_LOG(
-            "Did not find start '{}'. The text directive does not exist "
-            "in the document.",
+            "Did not find start '{}' in the sub range of the end of `prefix` "
+            "and the next block boundary. Restarting outer loop.",
             NS_ConvertUTF16toUTF8(aTextDirective.start));
-        return nullptr;
+        continue;
       }
-      TEXT_FRAGMENT_LOG("Did find start '{}'.",
-                        NS_ConvertUTF16toUTF8(aTextDirective.start));
       // 2.2.11. If potentialMatch’s start is not matchRange’s start, then
       // continue.
       // (In this case, we found a prefix but it was followed by something other
@@ -178,9 +229,11 @@ RefPtr<nsRange> TextDirectiveFinder::FindRangeForTextDirective(
       if (potentialMatch->StartRef() != matchRange->StartRef()) {
         TEXT_FRAGMENT_LOG(
             "The prefix is not directly followed by the start element. "
-            "Discarding this attempt.");
+            "Restarting outer loop.");
         continue;
       }
+      TEXT_FRAGMENT_LOG("Did find start '{}'.",
+                        NS_ConvertUTF16toUTF8(aTextDirective.start));
     }
     // 2.3. Otherwise:
     else {
@@ -192,7 +245,8 @@ RefPtr<nsRange> TextDirectiveFinder::FindRangeForTextDirective(
       // range steps with query parsedValues’s start, searchRange searchRange,
       // wordStartBounded true, and wordEndBounded mustEndAtWordBoundary.
       potentialMatch = TextDirectiveUtil::FindStringInRange(
-          searchRange, aTextDirective.start, true, mustEndAtWordBoundary);
+          finder, searchRange->StartRef(), searchRange->EndRef(),
+          aTextDirective.start, true, mustEndAtWordBoundary);
       // 2.3.3. If potentialMatch is null, return null.
       if (!potentialMatch) {
         TEXT_FRAGMENT_LOG(
@@ -201,13 +255,17 @@ RefPtr<nsRange> TextDirectiveFinder::FindRangeForTextDirective(
             NS_ConvertUTF16toUTF8(aTextDirective.start));
         return nullptr;
       }
+      if (potentialMatch && aTextDirective.end.IsEmpty() &&
+          aTextDirective.suffix.IsEmpty()) {
+        return potentialMatch;
+      }
       // 2.3.4. Set searchRange’s start to the first boundary point after
       // potentialMatch’s start
-      RangeBoundary newRangeBoundary =
-          TextDirectiveUtil::MoveRangeBoundaryOneWord(
-              {potentialMatch->GetStartContainer(),
-               potentialMatch->StartOffset()},
-              TextScanDirection::Right);
+      MOZ_DIAGNOSTIC_ASSERT(potentialMatch->GetStartContainer()->IsText());
+      const RangeBoundary newRangeBoundary =
+          TextDirectiveUtil::MoveToNextBoundaryPoint(
+              potentialMatch->StartRef());
+
       if (!newRangeBoundary.IsSetAndValid()) {
         return nullptr;
       }
@@ -236,7 +294,8 @@ RefPtr<nsRange> TextDirectiveFinder::FindRangeForTextDirective(
         // rangeEndSearchRange, wordStartBounded true, and wordEndBounded
         // mustEndAtWordBoundary.
         RefPtr<nsRange> endMatch = TextDirectiveUtil::FindStringInRange(
-            rangeEndSearchRange, aTextDirective.end, true,
+            finder, rangeEndSearchRange->StartRef(),
+            rangeEndSearchRange->EndRef(), aTextDirective.end, true,
             mustEndAtWordBoundary);
         // 2.5.1.3. If endMatch is null then return null.
         if (!endMatch) {
@@ -261,6 +320,8 @@ RefPtr<nsRange> TextDirectiveFinder::FindRangeForTextDirective(
       }
       // 2.5.4. Let suffixRange be a range with start equal to potentialMatch’s
       // end and end equal to searchRange’s end.
+      // Note: Again, this is highly inefficient. It's perfectly fine to only
+      // search up to the next block boundary.
       RefPtr<nsRange> suffixRange = nsRange::Create(
           potentialMatch->GetEndContainer(), potentialMatch->EndOffset(),
           searchRange->GetEndContainer(), searchRange->EndOffset(), rv);
@@ -269,46 +330,67 @@ RefPtr<nsRange> TextDirectiveFinder::FindRangeForTextDirective(
       }
       // 2.5.5. Advance suffixRange's start to the next non-whitespace position.
       TextDirectiveUtil::AdvanceStartToNextNonWhitespacePosition(*suffixRange);
+      auto nextBlockBoundary =
+          TextDirectiveUtil::FindNextBlockBoundary<TextScanDirection::Right>(
+              suffixRange->StartRef());
+      suffixRange->SetEnd(nextBlockBoundary.AsRaw(), IgnoreErrors());
 
       // 2.5.6. Let suffixMatch be result of running the find a string in range
       // steps with query parsedValue's suffix, searchRange suffixRange,
       // wordStartBounded false, and wordEndBounded true.
       RefPtr<nsRange> suffixMatch = TextDirectiveUtil::FindStringInRange(
-          suffixRange, aTextDirective.suffix, false, true);
-
+          finder, suffixRange->StartRef(), suffixRange->EndRef(),
+          aTextDirective.suffix, false, true);
       // 2.5.7. If suffixMatch is null, return null.
       // (If the suffix doesn't appear in the remaining text of the document,
       // there's no possible way to make a match.)
-      if (!suffixMatch) {
-        TEXT_FRAGMENT_LOG(
-            "Did not find suffix '{}'. The text directive does not exist "
-            "in the document.",
-            NS_ConvertUTF16toUTF8(aTextDirective.suffix));
-        return nullptr;
-      }
       // 2.5.8. If suffixMatch's start is suffixRange's start, return
       // potentialMatch.
-      if (suffixMatch->GetStartContainer() ==
-              suffixRange->GetStartContainer() &&
-          suffixMatch->StartOffset() == suffixRange->StartOffset()) {
-        TEXT_FRAGMENT_LOG("Did find a match.");
-        return potentialMatch;
-      }
       // 2.5.9. If parsedValue's end item is null then break;
       // (If this is an exact match and the suffix doesn’t match, start
       // searching for the next range start by breaking out of this loop without
       // rangeEndSearchRange being collapsed. If we’re looking for a range
       // match, we’ll continue iterating this inner loop since the range start
       // will already be correct.)
-      if (aTextDirective.end.IsEmpty()) {
-        break;
-      }
       // 2.5.10. Set rangeEndSearchRange's start to potentialMatch's end.
       // (Otherwise, it is possible that we found the correct range start, but
       // not the correct range end. Continue the inner loop to keep searching
       // for another matching instance of rangeEnd.)
+      // Note: the steps above are not correct anymore because of restricting
+      // the suffix find to a sub range.
+      // Therefore, the code looks different, but _essentially_ does the same as
+      // what's described in the spec steps.
       rangeEndSearchRange->SetStart(potentialMatch->GetEndContainer(),
                                     potentialMatch->EndOffset());
+      if (!suffixMatch) {
+        if (aTextDirective.end.IsEmpty()) {
+          TEXT_FRAGMENT_LOG(
+              "Did not find suffix in the sub range of the end of `start` and "
+              "the next block boundary. Restarting outer loop.");
+          break;
+        }
+        TEXT_FRAGMENT_LOG(
+            "Did not find suffix in the sub range of the end of `end` and the "
+            "next block boundary. Discarding this `end` candidate and "
+            "continuing inner loop.");
+        continue;
+      }
+      if (suffixMatch->GetStartContainer() ==
+              suffixRange->GetStartContainer() &&
+          suffixMatch->StartOffset() == suffixRange->StartOffset()) {
+        TEXT_FRAGMENT_LOG("Did find a match.");
+        return potentialMatch;
+      }
+      if (aTextDirective.end.IsEmpty()) {
+        TEXT_FRAGMENT_LOG(
+            "Did find suffix in the sub range of end of `start` to the end of "
+            "the next block boundary, but not at the start. Restarting outer "
+            "loop.");
+        break;
+      }
+      TEXT_FRAGMENT_LOG(
+          "Did find `suffix` in the sub range of end of `end` to the end of "
+          "the current block, but not at the start. Restarting inner loop.");
     }
     // 2.6. If rangeEndSearchRange is collapsed then:
     if (rangeEndSearchRange->Collapsed()) {
