@@ -11,7 +11,6 @@
 #include "mozilla/ScopeExit.h"
 #include "mozilla/StaticPrefs_dom.h"
 #include "mozilla/StaticPrefs_privacy.h"
-#include "mozilla/Telemetry.h"
 #include "mozilla/ThrottledEventQueue.h"
 #include "mozilla/TimeStamp.h"
 #include "nsINamed.h"
@@ -21,22 +20,24 @@
 #include "mozilla/dom/ContentChild.h"
 #include "mozilla/dom/TimeoutHandler.h"
 #include "TimeoutExecutor.h"
-#include "TimeoutBudgetManager.h"
 #include "mozilla/net/WebSocketEventService.h"
 #include "mozilla/MediaManager.h"
 #include "mozilla/dom/WorkerScope.h"
+#include "mozilla/dom/WebTaskScheduler.h"
 
 using namespace mozilla;
 using namespace mozilla::dom;
 
 LazyLogModule gTimeoutLog("Timeout");
 
-static int32_t gRunningTimeoutDepth = 0;
+TimeoutBudgetManager TimeoutManager::sBudgetManager{};
 
 // static
 const uint32_t TimeoutManager::InvalidFiringId = 0;
 
 namespace {
+static int32_t gRunningTimeoutDepth = 0;
+
 double GetRegenerationFactor(bool aIsBackground) {
   // Lookup function for "dom.timeout.{background,
   // foreground}_budget_regeneration_rate".
@@ -146,7 +147,7 @@ void TimeoutManager::MoveIdleToActive() {
       TimeDuration elapsed = now - timeout->SubmitTime();
       TimeDuration target = timeout->When() - timeout->SubmitTime();
       TimeDuration delta = now - timeout->When();
-      if (mGlobalObject.GetAsInnerWindow()) {
+      if (mIsWindow) {
         nsPrintfCString marker(
             "Releasing deferred setTimeout() for %dms (original target time "
             "was "
@@ -203,8 +204,7 @@ TimeDuration TimeoutManager::MinSchedulingDelay() const {
   }
 
   // do not throttle workers if dom_workers_throttling is disabled
-  if (!GetInnerWindow() && StaticPrefs::dom_workers_timeoutmanager() &&
-      !StaticPrefs::dom_workers_throttling_enabled()) {
+  if (!mIsWindow && !StaticPrefs::dom_workers_throttling_enabled_AtStartup()) {
     return TimeDuration();
   }
 
@@ -321,7 +321,7 @@ TimeDuration TimeoutManager::CalculateDelay(Timeout* aTimeout) const {
   TimeDuration result = aTimeout->mInterval;
 
   if (aTimeout->mNestingLevel >=
-      StaticPrefs::dom_clamp_timeout_nesting_level_AtStartup()) {
+      StaticPrefs::dom_clamp_timeout_nesting_level()) {
     uint32_t minTimeoutValue = StaticPrefs::dom_min_timeout_value();
     result = TimeDuration::Max(result,
                                TimeDuration::FromMilliseconds(minTimeoutValue));
@@ -332,8 +332,9 @@ TimeDuration TimeoutManager::CalculateDelay(Timeout* aTimeout) const {
 
 void TimeoutManager::RecordExecution(Timeout* aRunningTimeout,
                                      Timeout* aTimeout) {
-  TimeoutBudgetManager& budgetManager = TimeoutBudgetManager::Get();
   TimeStamp now = TimeStamp::Now();
+  TimeoutBudgetManager& budgetManager{mIsWindow ? sBudgetManager
+                                                : mBudgetManager};
 
   if (aRunningTimeout) {
     // If we're running a timeout callback, record any execution until
@@ -422,7 +423,8 @@ TimeoutManager::TimeoutManager(nsIGlobalObject& aHandle,
       mThrottleTrackingTimeouts(false),
       mBudgetThrottleTimeouts(false),
       mIsLoading(false),
-      mEventTarget(aEventTarget) {
+      mEventTarget(aEventTarget),
+      mIsWindow(aHandle.GetAsInnerWindow()) {
   MOZ_LOG(gTimeoutLog, LogLevel::Debug,
           ("TimeoutManager %p created, tracking bucketing %s\n", this,
            StaticPrefs::privacy_trackingprotection_annotate_channels()
@@ -431,7 +433,7 @@ TimeoutManager::TimeoutManager(nsIGlobalObject& aHandle,
 }
 
 TimeoutManager::~TimeoutManager() {
-  if (GetInnerWindow()) {
+  if (mIsWindow) {
     MOZ_DIAGNOSTIC_ASSERT(mGlobalObject.IsDying());
   }
   MOZ_DIAGNOSTIC_ASSERT(!mThrottleTimeoutsTimer);
@@ -465,6 +467,7 @@ int32_t TimeoutManager::GetTimeoutId(Timeout::Reason aReason) {
         }
         break;
       case Timeout::Reason::eDelayedWebTaskTimeout:
+      case Timeout::Reason::eJSTimeout:
       default:
         return -1;  // no cancellation support
     }
@@ -480,13 +483,19 @@ nsresult TimeoutManager::SetTimeout(TimeoutHandler* aHandler, int32_t interval,
                                     int32_t* aReturn) {
   // If we don't have a document (we could have been unloaded since
   // the call to setTimeout was made), do nothing.
-  if (mGlobalObject.GetAsInnerWindow()) {
+  if (mIsWindow) {
     nsCOMPtr<Document> doc = mGlobalObject.GetAsInnerWindow()->GetExtantDoc();
     if (!doc || mGlobalObject.IsDying()) {
       return NS_OK;
     }
   }
 
+  auto scopeExit = MakeScopeExit([&] {
+    if (!mIsWindow && !HasTimeouts()) {
+      mGlobalObject.TriggerUpdateCCFlag();
+    }
+  });
+
   // Disallow negative intervals.
   interval = std::max(0, interval);
 
@@ -508,15 +517,18 @@ nsresult TimeoutManager::SetTimeout(TimeoutHandler* aHandler, int32_t interval,
   timeout->mScriptHandler = aHandler;
   timeout->mReason = aReason;
 
-  // No popups from timeouts by default
-  timeout->mPopupState = PopupBlocker::openAbused;
+  if (mIsWindow) {
+    // No popups from timeouts by default
+    timeout->mPopupState = PopupBlocker::openAbused;
+  }
 
   // XXX: Does eIdleCallbackTimeout need clamping?
   if (aReason == Timeout::Reason::eTimeoutOrInterval ||
       aReason == Timeout::Reason::eIdleCallbackTimeout) {
-    const uint32_t nestingLevel{GetNestingLevel()};
+    const uint32_t nestingLevel{mIsWindow ? GetNestingLevelForWindow()
+                                          : GetNestingLevelForWorker()};
     timeout->mNestingLevel =
-        nestingLevel < StaticPrefs::dom_clamp_timeout_nesting_level_AtStartup()
+        nestingLevel < StaticPrefs::dom_clamp_timeout_nesting_level()
             ? nestingLevel + 1
             : nestingLevel;
   }
@@ -534,18 +546,20 @@ nsresult TimeoutManager::SetTimeout(TimeoutHandler* aHandler, int32_t interval,
     }
   }
 
-  if (gRunningTimeoutDepth == 0 &&
-      PopupBlocker::GetPopupControlState() < PopupBlocker::openBlocked) {
-    // This timeout is *not* set from another timeout and it's set
-    // while popups are enabled. Propagate the state to the timeout if
-    // its delay (interval) is equal to or less than what
-    // "dom.disable_open_click_delay" is set to (in ms).
+  if (mIsWindow) {
+    if (gRunningTimeoutDepth == 0 &&
+        PopupBlocker::GetPopupControlState() < PopupBlocker::openBlocked) {
+      // This timeout is *not* set from another timeout and it's set
+      // while popups are enabled. Propagate the state to the timeout if
+      // its delay (interval) is equal to or less than what
+      // "dom.disable_open_click_delay" is set to (in ms).
 
-    // This is checking |interval|, not realInterval, on purpose,
-    // because our lower bound for |realInterval| could be pretty high
-    // in some cases.
-    if (interval <= StaticPrefs::dom_disable_open_click_delay()) {
-      timeout->mPopupState = PopupBlocker::GetPopupControlState();
+      // This is checking |interval|, not realInterval, on purpose,
+      // because our lower bound for |realInterval| could be pretty high
+      // in some cases.
+      if (interval <= StaticPrefs::dom_disable_open_click_delay()) {
+        timeout->mPopupState = PopupBlocker::GetPopupControlState();
+      }
     }
   }
 
@@ -661,6 +675,17 @@ void TimeoutManager::RunTimeout(const TimeStamp& aNow,
     return;
   }
 
+  if (!GetInnerWindow()) {
+    // Workers don't use TaskController at the moment, so all the
+    // runnables have the same priorities. So we special case it
+    // here to allow "higher" prority tasks to run first before
+    // timers.
+    if (mGlobalObject.HasScheduledNormalOrHighPriorityWebTasks()) {
+      MOZ_ALWAYS_SUCCEEDS(MaybeSchedule(aNow));
+      return;
+    }
+  }
+
   Timeouts& timeouts(aProcessIdle ? mIdleTimeouts : mTimeouts);
 
   // Limit the overall time spent in RunTimeout() to reduce jank.
@@ -962,8 +987,12 @@ void TimeoutManager::RunTimeout(const TimeStamp& aNow,
       }
       // Check to see if we have run out of time to execute timeout handlers.
       // If we've exceeded our time budget then terminate the loop immediately.
+      //
+      // Or if there are high priority tasks dispatched by the Scheduler API,
+      // they should run first before timers.
       TimeDuration elapsed = now - start;
-      if (elapsed >= totalTimeLimit) {
+      if (elapsed >= totalTimeLimit ||
+          mGlobalObject.HasScheduledNormalOrHighPriorityWebTasks()) {
         // We ran out of time.  Make sure to schedule the executor to
         // run immediately for the next timer, if it exists.  Its possible,
         // however, that the last timeout handler suspended the window.  If
@@ -1009,7 +1038,7 @@ bool TimeoutManager::RescheduleTimeout(Timeout* aTimeout,
   // Automatically increase the nesting level when a setInterval()
   // is rescheduled just as if it was using a chained setTimeout().
   if (aTimeout->mNestingLevel <
-      StaticPrefs::dom_clamp_timeout_nesting_level_AtStartup()) {
+      StaticPrefs::dom_clamp_timeout_nesting_level()) {
     aTimeout->mNestingLevel += 1;
   }
 
@@ -1109,14 +1138,18 @@ void TimeoutManager::Timeouts::Insert(Timeout* aTimeout, SortBy aSortBy) {
 Timeout* TimeoutManager::BeginRunningTimeout(Timeout* aTimeout) {
   Timeout* currentTimeout = mRunningTimeout;
   mRunningTimeout = aTimeout;
-  ++gRunningTimeoutDepth;
+  if (mIsWindow) {
+    ++gRunningTimeoutDepth;
+  }
 
   RecordExecution(currentTimeout, aTimeout);
   return currentTimeout;
 }
 
 void TimeoutManager::EndRunningTimeout(Timeout* aTimeout) {
-  --gRunningTimeoutDepth;
+  if (mIsWindow) {
+    --gRunningTimeoutDepth;
+  }
 
   RecordExecution(mRunningTimeout, aTimeout);
   mRunningTimeout = aTimeout;
@@ -1275,8 +1308,7 @@ ThrottleTimeoutsCallback::Notify(nsITimer* aTimer) {
 
 bool TimeoutManager::BudgetThrottlingEnabled(bool aIsBackground) const {
   // do not throttle workers if dom_workers_throttling is disabled
-  if (!GetInnerWindow() && StaticPrefs::dom_workers_timeoutmanager() &&
-      !StaticPrefs::dom_workers_throttling_enabled()) {
+  if (!mIsWindow && !StaticPrefs::dom_workers_throttling_enabled_AtStartup()) {
     return false;
   }
 
