@@ -65,7 +65,7 @@ class MOZ_STACK_CLASS DragDataProducer {
                    nsIContent* aSelectionTargetNode, bool aIsAltKeyPressed);
   nsresult Produce(DataTransfer* aDataTransfer, bool* aCanDrag,
                    Selection** aSelection, nsIContent** aDragNode,
-                   nsIContentSecurityPolicy** aCsp,
+                   nsIPolicyContainer** aPolicyContainer,
                    nsICookieJarSettings** aCookieJarSettings);
 
  private:
@@ -110,7 +110,7 @@ nsresult nsContentAreaDragDrop::GetDragData(
     nsPIDOMWindowOuter* aWindow, nsIContent* aTarget,
     nsIContent* aSelectionTargetNode, bool aIsAltKeyPressed,
     DataTransfer* aDataTransfer, bool* aCanDrag, Selection** aSelection,
-    nsIContent** aDragNode, nsIContentSecurityPolicy** aCsp,
+    nsIContent** aDragNode, nsIPolicyContainer** aPolicyContainer,
     nsICookieJarSettings** aCookieJarSettings) {
   NS_ENSURE_TRUE(aSelectionTargetNode, NS_ERROR_INVALID_ARG);
 
@@ -118,8 +118,8 @@ nsresult nsContentAreaDragDrop::GetDragData(
 
   DragDataProducer provider(aWindow, aTarget, aSelectionTargetNode,
                             aIsAltKeyPressed);
-  return provider.Produce(aDataTransfer, aCanDrag, aSelection, aDragNode, aCsp,
-                          aCookieJarSettings);
+  return provider.Produce(aDataTransfer, aCanDrag, aSelection, aDragNode,
+                          aPolicyContainer, aCookieJarSettings);
 }
 
 NS_IMPL_ISUPPORTS(nsContentAreaDragDropDataProvider, nsIFlavorDataProvider)
@@ -146,7 +146,11 @@ nsresult nsContentAreaDragDropDataProvider::SaveURIToFile(
   NS_ENSURE_SUCCESS(rv, rv);
 
   persist->SetPersistFlags(
-      nsIWebBrowserPersist::PERSIST_FLAGS_AUTODETECT_APPLY_CONVERSION);
+      nsIWebBrowserPersist::PERSIST_FLAGS_AUTODETECT_APPLY_CONVERSION |
+      // Do not HTTPS-Only/-First upgrade this request. If we reach this point,
+      // any potential upgrades should have already happened, or the URI may
+      // have already been exempt.
+      nsIWebBrowserPersist::PERSIST_FLAGS_DISABLE_HTTPS_ONLY);
 
   // referrer policy can be anything since the referrer is nullptr
   return persist->SaveURI(inSourceURI, inTriggeringPrincipal, 0, nullptr,
@@ -154,47 +158,6 @@ nsresult nsContentAreaDragDropDataProvider::SaveURIToFile(
                           inContentPolicyType, isPrivate);
 }
 
-/*
- * Check if the provided filename extension is valid for the MIME type and
- * return the MIME type's primary extension.
- *
- * @param aExtension           [in]  the extension to check
- * @param aMimeType            [in]  the MIME type to check the extension with
- * @param aIsValidExtension    [out] true if |aExtension| is valid for
- *                                   |aMimeType|
- * @param aPrimaryExtension    [out] the primary extension for the MIME type
- *                                   to potentially be used as a replacement
- *                                   for |aExtension|
- */
-nsresult CheckAndGetExtensionForMime(const nsCString& aExtension,
-                                     const nsCString& aMimeType,
-                                     bool* aIsValidExtension,
-                                     nsACString* aPrimaryExtension) {
-  nsresult rv;
-
-  nsCOMPtr<nsIMIMEService> mimeService = do_GetService("@mozilla.org/mime;1");
-  if (NS_WARN_IF(!mimeService)) {
-    return NS_ERROR_FAILURE;
-  }
-
-  nsCOMPtr<nsIMIMEInfo> mimeInfo;
-  rv = mimeService->GetFromTypeAndExtension(aMimeType, ""_ns,
-                                            getter_AddRefs(mimeInfo));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  mimeInfo->GetPrimaryExtension(*aPrimaryExtension);
-
-  if (aExtension.IsEmpty()) {
-    *aIsValidExtension = false;
-    return NS_OK;
-  }
-
-  rv = mimeInfo->ExtensionExists(aExtension, aIsValidExtension);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  return NS_OK;
-}
-
 // This is our nsIFlavorDataProvider callback. There are several
 // assumptions here that make this work:
 //
@@ -376,45 +339,48 @@ void DragDataProducer::CreateLinkText(const nsAString& inURL,
 
 nsresult DragDataProducer::GetImageData(imgIContainer* aImage,
                                         imgIRequest* aRequest) {
+  MOZ_ASSERT(aImage);
+  MOZ_ASSERT(aRequest);
+
   nsCOMPtr<nsIURI> imgUri = aRequest->GetURI();
+  if (!imgUri) {
+    return NS_ERROR_FAILURE;
+  }
 
-  nsCOMPtr<nsIURL> imgUrl(do_QueryInterface(imgUri));
-  if (imgUrl) {
-    nsAutoCString spec;
-    nsresult rv = imgUrl->GetSpec(spec);
-    NS_ENSURE_SUCCESS(rv, rv);
+  nsAutoCString spec;
+  nsresult rv = imgUri->GetSpec(spec);
+  NS_ENSURE_SUCCESS(rv, rv);
 
-    // pass out the image source string
-    CopyUTF8toUTF16(spec, mImageSourceString);
+  // pass out the image source string
+  CopyUTF8toUTF16(spec, mImageSourceString);
 
-    nsCString mimeType;
-    aRequest->GetMimeType(getter_Copies(mimeType));
+  nsCString mimeType;
+  aRequest->GetMimeType(getter_Copies(mimeType));
 
-    nsAutoCString fileName;
-    aRequest->GetFileName(fileName);
+  nsAutoCString fileName;
+  aRequest->GetFileName(fileName);
 
 #if defined(XP_MACOSX)
-    // Save the MIME type so we can make sure the extension
-    // is compatible (and replace it if it isn't) when the
-    // image is dropped. On Mac, we need to get the OS MIME
-    // handler information in the parent due to sandboxing.
-    CopyUTF8toUTF16(mimeType, mImageRequestMime);
-    CopyUTF8toUTF16(fileName, mImageDestFileName);
+  // Save the MIME type so we can make sure the extension
+  // is compatible (and replace it if it isn't) when the
+  // image is dropped. On Mac, we need to get the OS MIME
+  // handler information in the parent due to sandboxing.
+  CopyUTF8toUTF16(mimeType, mImageRequestMime);
+  CopyUTF8toUTF16(fileName, mImageDestFileName);
 #else
-    nsCOMPtr<nsIMIMEService> mimeService = do_GetService("@mozilla.org/mime;1");
-    if (NS_WARN_IF(!mimeService)) {
-      return NS_ERROR_FAILURE;
-    }
+  nsCOMPtr<nsIMIMEService> mimeService = do_GetService("@mozilla.org/mime;1");
+  if (NS_WARN_IF(!mimeService)) {
+    return NS_ERROR_FAILURE;
+  }
 
-    CopyUTF8toUTF16(fileName, mImageDestFileName);
-    mimeService->ValidateFileNameForSaving(mImageDestFileName, mimeType,
-                                           nsIMIMEService::VALIDATE_DEFAULT,
-                                           mImageDestFileName);
+  CopyUTF8toUTF16(fileName, mImageDestFileName);
+  mimeService->ValidateFileNameForSaving(mImageDestFileName, mimeType,
+                                         nsIMIMEService::VALIDATE_DEFAULT,
+                                         mImageDestFileName);
 #endif
 
-    // and the image object
-    mImage = aImage;
-  }
+  // and the image object
+  mImage = aImage;
 
   return NS_OK;
 }
@@ -422,7 +388,7 @@ nsresult DragDataProducer::GetImageData(imgIContainer* aImage,
 nsresult DragDataProducer::Produce(DataTransfer* aDataTransfer, bool* aCanDrag,
                                    Selection** aSelection,
                                    nsIContent** aDragNode,
-                                   nsIContentSecurityPolicy** aCsp,
+                                   nsIPolicyContainer** aPolicyContainer,
                                    nsICookieJarSettings** aCookieJarSettings) {
   MOZ_ASSERT(aCanDrag && aSelection && aDataTransfer && aDragNode,
              "null pointer passed to Produce");
@@ -647,9 +613,9 @@ nsresult DragDataProducer::Produce(DataTransfer* aDataTransfer, bool* aCanDrag,
     nsCOMPtr<Document> doc = mWindow->GetDoc();
     NS_ENSURE_TRUE(doc, NS_ERROR_FAILURE);
 
-    nsCOMPtr<nsIContentSecurityPolicy> csp = doc->GetCsp();
-    if (csp) {
-      NS_IF_ADDREF(*aCsp = csp);
+    nsCOMPtr<nsIPolicyContainer> policyContainer = doc->GetPolicyContainer();
+    if (policyContainer) {
+      policyContainer.forget(aPolicyContainer);
     }
 
     nsCOMPtr<nsICookieJarSettings> cookieJarSettings = doc->CookieJarSettings();
