@@ -98,6 +98,9 @@ class nsFocusManager final : public nsIFocusManager,
    * Return a focused window. Version of nsIFocusManager::GetFocusedWindow.
    */
   nsPIDOMWindowOuter* GetFocusedWindow() const { return mFocusedWindow; }
+  static nsPIDOMWindowOuter* GetFocusedWindowStatic() {
+    return sInstance ? sInstance->GetFocusedWindow() : nullptr;
+  }
 
   /**
    * In the chrome process, retrieves the BrowsingContext corresponding
@@ -990,12 +993,6 @@ class nsFocusManager final : public nsIFocusManager,
   // focused.
   RefPtr<mozilla::dom::Element> mFocusedElement;
 
-  // these fields store a content node temporarily while it is being focused
-  // or blurred to ensure that a recursive call doesn't refire the same event.
-  // They will always be cleared afterwards.
-  RefPtr<mozilla::dom::Element> mFirstBlurEvent;
-  RefPtr<mozilla::dom::Element> mFirstFocusEvent;
-
   // keep track of a window while it is being lowered
   nsCOMPtr<nsPIDOMWindowOuter> mWindowBeingLowered;
 
