@@ -102,12 +102,8 @@ typedef struct _GtkWidget GtkWidget;
 #endif
 
 // IID for nsFrameLoader, because some places want to QI to it.
-#define NS_FRAMELOADER_IID                           \
-  {                                                  \
-    0x297fd0ea, 0x1b4a, 0x4c9a, {                    \
-      0xa4, 0x04, 0xe5, 0x8b, 0xe8, 0x95, 0x10, 0x50 \
-    }                                                \
-  }
+#define NS_FRAMELOADER_IID \
+  {0x297fd0ea, 0x1b4a, 0x4c9a, {0xa4, 0x04, 0xe5, 0x8b, 0xe8, 0x95, 0x10, 0x50}}
 
 class nsFrameLoader final : public nsStubMutationObserver,
                             public mozilla::dom::ipc::MessageManagerCallback,
@@ -137,7 +133,7 @@ class nsFrameLoader final : public nsStubMutationObserver,
       const mozilla::dom::NavigationIsolationOptions& aRemotenessOptions,
       bool aIsRemote, bool aNetworkCreated, bool aPreserveContext);
 
-  NS_DECLARE_STATIC_IID_ACCESSOR(NS_FRAMELOADER_IID)
+  NS_INLINE_DECL_STATIC_IID(NS_FRAMELOADER_IID)
 
   NS_DECL_CYCLE_COLLECTING_ISUPPORTS
   NS_DECL_CYCLE_COLLECTION_WRAPPERCACHE_CLASS(nsFrameLoader)
@@ -201,13 +197,14 @@ class nsFrameLoader final : public nsStubMutationObserver,
    * @param aTriggeringPrincipal The triggering principal for the load. May be
    *        null, in which case the node principal of the owner content will be
    *        used.
-   * @param aCsp The CSP to be used for the load. That is not the CSP to be
-   *        applied to subresources within the frame, but to the iframe load
-   *        itself. E.g. if the CSP holds upgrade-insecure-requests the the
-   *        frame load is upgraded from http to https.
+   * @param aPolicyContainer The policyContainer to be used for the load. That
+   * is not the policyContainer to be applied to subresources within the frame,
+   * but to the iframe load itself. E.g. if the policyContainer's CSP holds
+   * upgrade-insecure-requests the the frame load is upgraded from http to
+   * https.
    */
   nsresult LoadURI(nsIURI* aURI, nsIPrincipal* aTriggeringPrincipal,
-                   nsIContentSecurityPolicy* aCsp, bool aOriginalSrc,
+                   nsIPolicyContainer* aPolicyContainer, bool aOriginalSrc,
                    bool aShouldCheckForRecursion);
 
   /**
@@ -388,10 +385,10 @@ class nsFrameLoader final : public nsStubMutationObserver,
   void ApplySandboxFlags(uint32_t sandboxFlags);
 
   void GetURL(nsString& aURL, nsIPrincipal** aTriggeringPrincipal,
-              nsIContentSecurityPolicy** aCsp);
+              nsIPolicyContainer** aPolicyContainer);
 
   // Properly retrieves documentSize of any subdocument type.
-  nsresult GetWindowDimensions(nsIntRect& aRect);
+  nsresult GetWindowDimensions(mozilla::LayoutDeviceIntRect& aRect);
 
   virtual mozilla::dom::ProcessMessageManager* GetProcessMessageManager()
       const override;
@@ -501,7 +498,7 @@ class nsFrameLoader final : public nsStubMutationObserver,
   RefPtr<mozilla::dom::BrowsingContext> mPendingBrowsingContext;
   nsCOMPtr<nsIURI> mURIToLoad;
   nsCOMPtr<nsIPrincipal> mTriggeringPrincipal;
-  nsCOMPtr<nsIContentSecurityPolicy> mCsp;
+  nsCOMPtr<nsIPolicyContainer> mPolicyContainer;
   nsCOMPtr<nsIOpenWindowInfo> mOpenWindowInfo;
   mozilla::dom::Element* mOwnerContent;  // WEAK
 
@@ -524,7 +521,7 @@ class nsFrameLoader final : public nsStubMutationObserver,
   RefPtr<nsDocShell> mDocShell;
 
   // Holds the last known size of the frame.
-  mozilla::ScreenIntSize mLazySize;
+  mozilla::LayoutDeviceIntSize mLazySize;
 
   // Actor for collecting session store data from content children. This will be
   // cleared and set to null eagerly when taking down the frameloader to break
@@ -568,8 +565,6 @@ class nsFrameLoader final : public nsStubMutationObserver,
   bool mTabProcessCrashFired : 1;
 };
 
-NS_DEFINE_STATIC_IID_ACCESSOR(nsFrameLoader, NS_FRAMELOADER_IID)
-
 inline nsISupports* ToSupports(nsFrameLoader* aFrameLoader) {
   return aFrameLoader;
 }
