@@ -77,7 +77,7 @@ class nsGlobalWindowOuter;
 class nsDOMWindowUtils;
 class nsIUserIdleService;
 struct nsRect;
-
+class nsWindowRoot;
 class nsWindowSizes;
 
 class IdleRequestExecutor;
@@ -120,7 +120,6 @@ class IdleRequestCallback;
 class InstallTriggerImpl;
 class IntlUtils;
 class MediaQueryList;
-class Navigation;
 class OwningExternalOrWindowProxy;
 class Promise;
 class PostMessageEvent;
@@ -131,6 +130,7 @@ class Selection;
 struct SizeToContentConstraints;
 class WebTaskScheduler;
 class WebTaskSchedulerMainThread;
+class WebTaskSchedulingState;
 class SpeechSynthesis;
 class Timeout;
 class TrustedTypePolicyFactory;
@@ -335,10 +335,16 @@ class nsGlobalWindowInner final : public mozilla::dom::EventTarget,
   void Freeze(bool aIncludeSubWindows = true);
   void Thaw(bool aIncludeSubWindows = true);
   virtual bool IsFrozen() const override;
-  virtual bool HasActiveIndexedDBDatabases() override;
+  virtual bool HasActiveIndexedDBDatabases() const override;
   virtual bool HasActivePeerConnections() override;
   virtual bool HasOpenWebSockets() const override;
+  void AudioPlaybackChanged(bool aIsPlayingAudio);
+  virtual bool HasScheduledNormalOrHighPriorityWebTasks() const override;
   void SyncStateFromParentWindow();
+  virtual void UpdateWebSocketCount(int32_t aDelta) override;
+  // Increase/Decrease the number of active IndexedDB databases for the
+  // decision making of timeout-throttling.
+  virtual void UpdateActiveIndexedDBDatabaseCount(int32_t aDelta) override;
 
   // Called on the current inner window of a browsing context when its
   // background state changes according to selected tab or visibility of the
@@ -359,9 +365,9 @@ class nsGlobalWindowInner final : public mozilla::dom::EventTarget,
   mozilla::Maybe<mozilla::dom::ServiceWorkerDescriptor> GetController()
       const override;
 
-  void SetCsp(nsIContentSecurityPolicy* aCsp);
+  void SetPolicyContainer(nsIPolicyContainer* aPolicyContainer);
+  nsIPolicyContainer* GetPolicyContainer();
   void SetPreloadCsp(nsIContentSecurityPolicy* aPreloadCsp);
-  nsIContentSecurityPolicy* GetCsp();
 
   virtual already_AddRefed<mozilla::dom::ServiceWorkerContainer>
   GetServiceWorkerContainer() override;
@@ -423,11 +429,6 @@ class nsGlobalWindowInner final : public mozilla::dom::EventTarget,
 
   static bool DeviceSensorsEnabled(JSContext*, JSObject*);
 
-  static bool CachesEnabled(JSContext* aCx, JSObject*);
-
-  // WebIDL permission Func for whether Glean APIs are permitted.
-  static bool IsGleanNeeded(JSContext*, JSObject*);
-
   bool DoResolve(
       JSContext* aCx, JS::Handle<JSObject*> aObj, JS::Handle<jsid> aId,
       JS::MutableHandle<mozilla::Maybe<JS::PropertyDescriptor>> aDesc);
@@ -538,10 +539,6 @@ class nsGlobalWindowInner final : public mozilla::dom::EventTarget,
   // Enable/disable updates for VR
   void EnableVRUpdates();
   void DisableVRUpdates();
-  // Reset telemetry data when switching windows.
-  // aUpdate, true for accumulating the result to the histogram.
-  // false for only resetting the timestamp.
-  void ResetVRTelemetry(bool aUpdate);
 
   void StartVRActivity();
   void StopVRActivity();
@@ -618,7 +615,6 @@ class nsGlobalWindowInner final : public mozilla::dom::EventTarget,
   void SetName(const nsAString& aName, mozilla::ErrorResult& aError);
   mozilla::dom::Location* Location() override;
   nsHistory* GetHistory(mozilla::ErrorResult& aError);
-  mozilla::dom::Navigation* Navigation();
   mozilla::dom::CustomElementRegistry* CustomElements() override;
   mozilla::dom::CustomElementRegistry* GetExistingCustomElements();
   mozilla::dom::BarProp* GetLocationbar(mozilla::ErrorResult& aError);
@@ -723,7 +719,7 @@ class nsGlobalWindowInner final : public mozilla::dom::EventTarget,
       JSContext* aCx,
       const mozilla::dom::FunctionOrTrustedScriptOrString& aHandler,
       int32_t aTimeout, const mozilla::dom::Sequence<JS::Value>& /* unused */,
-      mozilla::ErrorResult& aError);
+      nsIPrincipal* aSubjectPrincipal, mozilla::ErrorResult& aError);
 
   MOZ_CAN_RUN_SCRIPT
   void ClearTimeout(int32_t aHandle);
@@ -734,7 +730,7 @@ class nsGlobalWindowInner final : public mozilla::dom::EventTarget,
       const mozilla::dom::FunctionOrTrustedScriptOrString& aHandler,
       const int32_t aTimeout,
       const mozilla::dom::Sequence<JS::Value>& /* unused */,
-      mozilla::ErrorResult& aError);
+      nsIPrincipal* aSubjectPrincipal, mozilla::ErrorResult& aError);
 
   MOZ_CAN_RUN_SCRIPT
   void ClearInterval(int32_t aHandle);
@@ -765,6 +761,7 @@ class nsGlobalWindowInner final : public mozilla::dom::EventTarget,
       const nsACString& aQuery, mozilla::dom::CallerType aCallerType,
       mozilla::ErrorResult& aError);
   nsScreen* Screen();
+  bool HasScreen() const { return !!mScreen; }
   void MoveTo(int32_t aXPos, int32_t aYPos,
               mozilla::dom::CallerType aCallerType,
               mozilla::ErrorResult& aError);
@@ -965,8 +962,15 @@ class nsGlobalWindowInner final : public mozilla::dom::EventTarget,
 
   // https://whatpr.org/html/4734/structured-data.html#cross-origin-isolated
   bool CrossOriginIsolated() const override;
+  bool OriginAgentCluster() const;
 
   mozilla::dom::WebTaskScheduler* Scheduler();
+  void SetWebTaskSchedulingState(
+      mozilla::dom::WebTaskSchedulingState* aState) override;
+  mozilla::dom::WebTaskSchedulingState* GetWebTaskSchedulingState()
+      const override {
+    return mWebTaskSchedulingState;
+  }
 
  protected:
   // Web IDL helpers
@@ -1037,7 +1041,6 @@ class nsGlobalWindowInner final : public mozilla::dom::EventTarget,
   template <typename Method, typename... Args>
   mozilla::CallState CallOnInProcessDescendants(Method aMethod,
                                                 Args&&... aArgs) {
-    MOZ_ASSERT(IsCurrentInnerWindow());
     return CallOnInProcessDescendantsInternal(GetBrowsingContext(), false,
                                               aMethod, aArgs...);
   }
@@ -1075,7 +1078,8 @@ class nsGlobalWindowInner final : public mozilla::dom::EventTarget,
       JSContext* aCx,
       const mozilla::dom::FunctionOrTrustedScriptOrString& aHandler,
       int32_t aTimeout, const mozilla::dom::Sequence<JS::Value>& aArguments,
-      bool aIsInterval, mozilla::ErrorResult& aError);
+      bool aIsInterval, nsIPrincipal* aSubjectPrincipal,
+      mozilla::ErrorResult& aError);
 
   // Return true if |aTimeout| was cleared while its handler ran.
   MOZ_CAN_RUN_SCRIPT
@@ -1094,7 +1098,7 @@ class nsGlobalWindowInner final : public mozilla::dom::EventTarget,
 
   void FlushPendingNotifications(mozilla::FlushType aType);
 
-  void ScrollTo(const mozilla::CSSIntPoint& aScroll,
+  void ScrollTo(const mozilla::CSSPoint& aScroll,
                 const mozilla::dom::ScrollOptions& aOptions);
 
   already_AddRefed<nsIWidget> GetMainWidget();
@@ -1173,7 +1177,6 @@ class nsGlobalWindowInner final : public mozilla::dom::EventTarget,
   friend class nsPIDOMWindowOuter;
 
   bool IsBackgroundInternal() const override;
-  bool IsPlayingAudio() override;
 
   // NOTE: Chrome Only
   void DisconnectAndClearGroupMessageManagers() {
@@ -1211,6 +1214,7 @@ class nsGlobalWindowInner final : public mozilla::dom::EventTarget,
 
  public:
   static uint32_t GetShortcutsPermission(nsIPrincipal* aPrincipal);
+  bool IsPlayingAudio() override;
 
   // Dispatch a runnable related to the global.
   nsresult Dispatch(already_AddRefed<nsIRunnable>&& aRunnable) const final;
@@ -1283,6 +1287,7 @@ class nsGlobalWindowInner final : public mozilla::dom::EventTarget,
   RefPtr<mozilla::dom::ContentMediaController> mContentMediaController;
 
   RefPtr<mozilla::dom::WebTaskSchedulerMainThread> mWebTaskScheduler;
+  RefPtr<mozilla::dom::WebTaskSchedulingState> mWebTaskSchedulingState;
 
   RefPtr<mozilla::dom::TrustedTypePolicyFactory> mTrustedTypePolicyFactory;
 
@@ -1296,7 +1301,7 @@ class nsGlobalWindowInner final : public mozilla::dom::EventTarget,
   // Represents whether the inner window's page has had a slow script notice.
   // Only used by inner windows; will always be false for outer windows.
   // This is used to implement Telemetry measures such as
-  // SLOW_SCRIPT_PAGE_COUNT.
+  // SLOW_SCRIPT_PAGE_COUNT (glean::dom::slow_script_page_count).
   bool mHasHadSlowScript : 1;
 
   // Fast way to tell if this is a chrome window (without having to QI).
@@ -1388,20 +1393,19 @@ class nsGlobalWindowInner final : public mozilla::dom::EventTarget,
   RefPtr<mozilla::EventListenerManager> mListenerManager;
   RefPtr<mozilla::dom::Location> mLocation;
   RefPtr<nsHistory> mHistory;
-  RefPtr<mozilla::dom::Navigation> mNavigation;
   RefPtr<mozilla::dom::CustomElementRegistry> mCustomElements;
 
   nsTObserverArray<RefPtr<mozilla::dom::SharedWorker>> mSharedWorkers;
 
   RefPtr<mozilla::dom::VisualViewport> mVisualViewport;
 
-  // The document's principals and CSP are only stored if
+  // The document's principals and policyContainer are only stored if
   // FreeInnerObjects has been called.
   nsCOMPtr<nsIPrincipal> mDocumentPrincipal;
   nsCOMPtr<nsIPrincipal> mDocumentCookiePrincipal;
   nsCOMPtr<nsIPrincipal> mDocumentStoragePrincipal;
   nsCOMPtr<nsIPrincipal> mDocumentPartitionedPrincipal;
-  nsCOMPtr<nsIContentSecurityPolicy> mDocumentCsp;
+  nsCOMPtr<nsIPolicyContainer> mDocumentPolicyContainer;
 
   // Used to cache the result of StorageAccess::StorageAllowedForWindow.
   // Don't use this field directly, use StorageAccess::StorageAllowedForWindow
