@@ -32,6 +32,7 @@ class nsICSSDeclaration;
 class nsIDocShell;
 class nsIDocShellTreeOwner;
 class nsDocShellLoadState;
+class nsIPolicyContainer;
 class nsIPrincipal;
 class nsIRunnable;
 class nsIScriptTimeoutHandler;
@@ -58,6 +59,7 @@ class Element;
 class Location;
 class MediaDevices;
 class MediaKeys;
+class Navigation;
 class Navigator;
 class Performance;
 class Selection;
@@ -65,6 +67,7 @@ class ServiceWorker;
 class ServiceWorkerDescriptor;
 class Timeout;
 class TimeoutManager;
+class WebIdentityHandler;
 class WindowContext;
 class WindowGlobalChild;
 class CustomElementRegistry;
@@ -83,20 +86,12 @@ enum class FullscreenReason {
 };
 
 // Must be kept in sync with xpcom/rust/xpcom/src/interfaces/nonidl.rs
-#define NS_PIDOMWINDOWINNER_IID                      \
-  {                                                  \
-    0x775dabc9, 0x8f43, 0x4277, {                    \
-      0x9a, 0xdb, 0xf1, 0x99, 0x0d, 0x77, 0xcf, 0xfb \
-    }                                                \
-  }
+#define NS_PIDOMWINDOWINNER_IID \
+  {0x775dabc9, 0x8f43, 0x4277, {0x9a, 0xdb, 0xf1, 0x99, 0x0d, 0x77, 0xcf, 0xfb}}
 
 // Must be kept in sync with xpcom/rust/xpcom/src/interfaces/nonidl.rs
-#define NS_PIDOMWINDOWOUTER_IID                      \
-  {                                                  \
-    0x769693d4, 0xb009, 0x4fe2, {                    \
-      0xaf, 0x18, 0x7d, 0xc8, 0xdf, 0x74, 0x96, 0xdf \
-    }                                                \
-  }
+#define NS_PIDOMWINDOWOUTER_IID \
+  {0x769693d4, 0xb009, 0x4fe2, {0xaf, 0x18, 0x7d, 0xc8, 0xdf, 0x74, 0x96, 0xdf}}
 
 class nsPIDOMWindowInner : public mozIDOMWindow {
  protected:
@@ -110,7 +105,7 @@ class nsPIDOMWindowInner : public mozIDOMWindow {
   ~nsPIDOMWindowInner();
 
  public:
-  NS_DECLARE_STATIC_IID_ACCESSOR(NS_PIDOMWINDOWINNER_IID)
+  NS_INLINE_DECL_STATIC_IID(NS_PIDOMWINDOWINNER_IID)
 
   nsIGlobalObject* AsGlobal();
   const nsIGlobalObject* AsGlobal() const;
@@ -231,6 +226,29 @@ class nsPIDOMWindowInner : public mozIDOMWindow {
     mMayHavePointerEnterLeaveEventListener = true;
   }
 
+  /**
+   * Call this to check whether some node (this window, its document,
+   * or content in that document) has a pointerrawupdate event listener.
+   */
+  bool HasPointerRawUpdateEventListeners() const {
+    return mMayHavePointerRawUpdateEventListener;
+  }
+
+  /**
+   * Call this to indicate that some node (this window, its document,
+   * or content in that document) has a pointerrawupdate event listener.
+   * This may not accept that if the event is not available in this window.
+   */
+  void MaybeSetHasPointerRawUpdateEventListeners();
+
+ protected:
+  /**
+   * Call this to clear whether some nodes has a pointerrawupdate event
+   * listener.
+   */
+  void ClearHasPointerRawUpdateEventListeners();
+
+ public:
   /**
    * Call this to check whether some node (this window, its document,
    * or content in that document) has a transition* event listeners.
@@ -341,41 +359,20 @@ class nsPIDOMWindowInner : public mozIDOMWindow {
    */
   void RemovePeerConnection();
 
-  /**
-   * Check whether the active peer connection count is non-zero.
-   */
-  bool HasActivePeerConnections();
-
-  bool IsPlayingAudio();
-
   bool IsDocumentLoaded() const;
 
   // To cache top inner-window if available after constructed for tab-wised
   // indexedDB counters.
   void TryToCacheTopInnerWindow();
 
-  // Increase/Decrease the number of active IndexedDB databases for the
-  // decision making of timeout-throttling.
-  void UpdateActiveIndexedDBDatabaseCount(int32_t aDelta);
-
-  // Return true if there is any active IndexedDB databases which could block
-  // timeout-throttling.
-  bool HasActiveIndexedDBDatabases();
-
-  // Increase/Decrease the number of open WebSockets.
-  void UpdateWebSocketCount(int32_t aDelta);
-
-  // Return true if there are any open WebSockets that could block
-  // timeout-throttling.
-  bool HasOpenWebSockets() const;
-
   mozilla::Maybe<mozilla::dom::ClientInfo> GetClientInfo() const;
   mozilla::Maybe<mozilla::dom::ClientState> GetClientState() const;
   mozilla::Maybe<mozilla::dom::ServiceWorkerDescriptor> GetController() const;
 
-  void SetCsp(nsIContentSecurityPolicy* aCsp);
+  void SetPolicyContainer(nsIPolicyContainer* aPolicyContainer);
+  nsIPolicyContainer* GetPolicyContainer();
+
   void SetPreloadCsp(nsIContentSecurityPolicy* aPreloadCsp);
-  nsIContentSecurityPolicy* GetCsp();
 
   void NoteCalledRegisterForServiceWorkerScope(const nsACString& aScope);
 
@@ -421,13 +418,6 @@ class nsPIDOMWindowInner : public mozIDOMWindow {
   // true if it was.
   bool RemoveFromBFCacheSync();
 
-  // Determine if the window is suspended or frozen.  Outer windows
-  // will forward this call to the inner window for convenience.  If
-  // there is no inner window then the outer window is considered
-  // suspended and frozen by default.
-  virtual bool IsSuspended() const = 0;
-  virtual bool IsFrozen() const = 0;
-
   // Fire any DOM notification events related to things that happened while
   // the window was frozen.
   virtual nsresult FireDelayedDOMEvents(bool aIncludeSubWindows) = 0;
@@ -463,18 +453,6 @@ class nsPIDOMWindowInner : public mozIDOMWindow {
     return mMayHaveDOMActivateEventListeners;
   }
 
-  /**
-   * Call this to indicate that some node (this window, its document,
-   * or content in that document) has a paint event listener.
-   */
-  void SetHasPaintEventListeners() { mMayHavePaintEventListener = true; }
-
-  /**
-   * Call this to check whether some node (this window, its document,
-   * or content in that document) has a paint event listener.
-   */
-  bool HasPaintEventListeners() { return mMayHavePaintEventListener; }
-
   /**
    * Call this to indicate that some node (this window, its document,
    * or content in that document) has a touch event listener.
@@ -620,6 +598,7 @@ class nsPIDOMWindowInner : public mozIDOMWindow {
 
   uint32_t GetMarkedCCGeneration() { return mMarkedCCGeneration; }
 
+  mozilla::dom::Navigation* Navigation();
   mozilla::dom::Navigator* Navigator();
   mozilla::dom::MediaDevices* GetExtantMediaDevices() const;
   virtual mozilla::dom::Location* Location() = 0;
@@ -645,6 +624,8 @@ class nsPIDOMWindowInner : public mozIDOMWindow {
 
   bool UsingStorageAccess();
 
+  mozilla::dom::WebIdentityHandler* GetOrCreateWebIdentityHandler();
+
   uint32_t UpdateLockCount(bool aIncrement) {
     MOZ_ASSERT_IF(!aIncrement, mLockCount > 0);
     mLockCount += aIncrement ? 1 : -1;
@@ -661,6 +642,12 @@ class nsPIDOMWindowInner : public mozIDOMWindow {
 
   mozilla::dom::CloseWatcherManager* EnsureCloseWatcherManager();
 
+  // Called when a CloseWatcher is added to the manager
+  void NotifyCloseWatcherAdded();
+
+  // Called when a CloseWatcher is removed from the manager
+  void NotifyCloseWatcherRemoved();
+
  protected:
   void CreatePerformanceObjectIfNeeded();
 
@@ -690,29 +677,32 @@ class nsPIDOMWindowInner : public mozIDOMWindow {
 
   RefPtr<mozilla::dom::Performance> mPerformance;
   mozilla::UniquePtr<mozilla::dom::TimeoutManager> mTimeoutManager;
+  RefPtr<mozilla::dom::WebIdentityHandler> mWebIdentityHandler;
+
+  RefPtr<mozilla::dom::Navigation> mNavigation;
 
   RefPtr<mozilla::dom::Navigator> mNavigator;
 
   // These variables are only used on inner windows.
-  uint32_t mMutationBits;
+  uint32_t mMutationBits = 0;
 
   uint32_t mActivePeerConnections = 0;
 
-  bool mIsDocumentLoaded;
-  bool mIsHandlingResizeEvent;
-  bool mMayHaveDOMActivateEventListeners;
-  bool mMayHavePaintEventListener;
-  bool mMayHaveTouchEventListener;
-  bool mMayHaveSelectionChangeEventListener;
-  bool mMayHaveFormSelectEventListener;
-  bool mMayHaveMouseEnterLeaveEventListener;
-  bool mMayHavePointerEnterLeaveEventListener;
-  bool mMayHaveTransitionEventListener;
-  bool mMayHaveSMILTimeEventListener;
+  bool mIsDocumentLoaded = false;
+  bool mIsHandlingResizeEvent = false;
+  bool mMayHaveDOMActivateEventListeners = false;
+  bool mMayHaveTouchEventListener = false;
+  bool mMayHaveSelectionChangeEventListener = false;
+  bool mMayHaveFormSelectEventListener = false;
+  bool mMayHaveMouseEnterLeaveEventListener = false;
+  bool mMayHavePointerEnterLeaveEventListener = false;
+  bool mMayHavePointerRawUpdateEventListener = false;
+  bool mMayHaveTransitionEventListener = false;
+  bool mMayHaveSMILTimeEventListener = false;
   // Only used for telemetry probes.  This may be wrong if some nodes have
   // come from another document with `Document.adoptNode`.
-  bool mMayHaveBeforeInputEventListenerForTelemetry;
-  bool mMutationObserverHasObservedNodeForTelemetry;
+  bool mMayHaveBeforeInputEventListenerForTelemetry = false;
+  bool mMutationObserverHasObservedNodeForTelemetry = false;
 
   // Our inner window's outer window.
   nsCOMPtr<nsPIDOMWindowOuter> mOuterWindow;
@@ -733,11 +723,11 @@ class nsPIDOMWindowInner : public mozIDOMWindow {
 
   // A unique (as long as our 64-bit counter doesn't roll over) id for
   // this window.
-  uint64_t mWindowID;
+  uint64_t mWindowID = 0;
 
   // Set to true once we've sent the (chrome|content)-document-global-created
   // notification.
-  bool mHasNotifiedGlobalCreated;
+  bool mHasNotifiedGlobalCreated = false;
 
   // Whether when focused via an "unknown" focus method, we should show outlines
   // by default or not. The initial value of this is true (so as to show
@@ -745,7 +735,7 @@ class nsPIDOMWindowInner : public mozIDOMWindow {
   // without any other user interaction).
   bool mUnknownFocusMethodShouldShowOutline = true;
 
-  uint32_t mMarkedCCGeneration;
+  uint32_t mMarkedCCGeneration = 0;
 
   // mTopInnerWindow is used for tab-wise check by timeout throttling. It could
   // be null.
@@ -754,17 +744,17 @@ class nsPIDOMWindowInner : public mozIDOMWindow {
   // The evidence that we have tried to cache mTopInnerWindow only once from
   // SetNewDocument(). Note: We need this extra flag because mTopInnerWindow
   // could be null and we don't want it to be set multiple times.
-  bool mHasTriedToCacheTopInnerWindow;
+  bool mHasTriedToCacheTopInnerWindow = false;
 
   // The number of active IndexedDB databases.
-  uint32_t mNumOfIndexedDBDatabases;
+  uint32_t mNumOfIndexedDBDatabases = 0;
 
   // The number of open WebSockets.
-  uint32_t mNumOfOpenWebSockets;
+  uint32_t mNumOfOpenWebSockets = 0;
 
   // The event dispatch code sets and unsets this while keeping
   // the event object alive.
-  mozilla::dom::Event* mEvent;
+  mozilla::dom::Event* mEvent = nullptr;
 
   // The WindowGlobalChild actor for this window.
   //
@@ -772,7 +762,7 @@ class nsPIDOMWindowInner : public mozIDOMWindow {
   // during SetNewDocument, and cleared during FreeInnerObjects.
   RefPtr<mozilla::dom::WindowGlobalChild> mWindowGlobalChild;
 
-  bool mWasSuspendedByGroup;
+  bool mWasSuspendedByGroup = false;
 
   /**
    * Count of the number of active LockRequest objects, including ones from
@@ -789,8 +779,6 @@ class nsPIDOMWindowInner : public mozIDOMWindow {
   RefPtr<mozilla::dom::CloseWatcherManager> mCloseWatcherManager;
 };
 
-NS_DEFINE_STATIC_IID_ACCESSOR(nsPIDOMWindowInner, NS_PIDOMWINDOWINNER_IID)
-
 class nsPIDOMWindowOuter : public mozIDOMWindowProxy {
  protected:
   using Document = mozilla::dom::Document;
@@ -802,7 +790,7 @@ class nsPIDOMWindowOuter : public mozIDOMWindowProxy {
   void NotifyResumingDelayedMedia();
 
  public:
-  NS_DECLARE_STATIC_IID_ACCESSOR(NS_PIDOMWINDOWOUTER_IID)
+  NS_INLINE_DECL_STATIC_IID(NS_PIDOMWINDOWOUTER_IID)
 
   NS_IMPL_FROMEVENTTARGET_HELPER_WITH_GETTER(nsPIDOMWindowOuter,
                                              GetAsOuterWindow())
@@ -909,9 +897,9 @@ class nsPIDOMWindowOuter : public mozIDOMWindowProxy {
   }
 
   // Set the window up with an about:blank document with the given principal and
-  // potentially a CSP and a COEP.
+  // potentially a policyContainer and a COEP.
   virtual void SetInitialPrincipal(
-      nsIPrincipal* aNewWindowPrincipal, nsIContentSecurityPolicy* aCSP,
+      nsIPrincipal* aNewWindowPrincipal, nsIPolicyContainer* aPolicyContainer,
       const mozilla::Maybe<nsILoadInfo::CrossOriginEmbedderPolicy>& aCoep) = 0;
 
   // Returns an object containing the window's state.  This also suspends
@@ -921,13 +909,6 @@ class nsPIDOMWindowOuter : public mozIDOMWindowProxy {
   // Restore the window state from aState.
   virtual nsresult RestoreWindowState(nsISupports* aState) = 0;
 
-  // Determine if the window is suspended or frozen.  Outer windows
-  // will forward this call to the inner window for convenience.  If
-  // there is no inner window then the outer window is considered
-  // suspended and frozen by default.
-  virtual bool IsSuspended() const = 0;
-  virtual bool IsFrozen() const = 0;
-
   // Fire any DOM notification events related to things that happened while
   // the window was frozen.
   virtual nsresult FireDelayedDOMEvents(bool aIncludeSubWindows) = 0;
@@ -1197,8 +1178,6 @@ class nsPIDOMWindowOuter : public mozIDOMWindowProxy {
   uint32_t mMarkedCCGeneration;
 };
 
-NS_DEFINE_STATIC_IID_ACCESSOR(nsPIDOMWindowOuter, NS_PIDOMWINDOWOUTER_IID)
-
 #include "nsPIDOMWindowInlines.h"
 
 #endif  // nsPIDOMWindow_h__
