@@ -23,6 +23,8 @@
 #include "mozilla/RangeBoundary.h"
 #include "mozilla/RefPtr.h"
 
+class nsIPrincipal;
+
 namespace mozilla {
 class RectCallback;
 namespace dom {
@@ -38,11 +40,6 @@ class TrustedHTMLOrString;
 enum class RangeBehaviour : uint8_t {
   // Keep both ranges
   KeepDefaultRangeAndCrossShadowBoundaryRanges,
-  // Merge both ranges; This is the case where the range boundaries was in
-  // different roots initially, and becoming in the same roots now. Since
-  // they start to be in the same root, using normal range is good enough
-  // to represent it
-  MergeDefaultRangeAndCrossShadowBoundaryRanges,
   // Collapse the default range
   CollapseDefaultRange,
   // Collapse both the default range and the cross-shadow-boundary range
@@ -86,19 +83,21 @@ class nsRange final : public mozilla::dom::AbstractRange,
     return nsRange::Create(aAbstractRange->StartRef(), aAbstractRange->EndRef(),
                            aRv);
   }
-  static already_AddRefed<nsRange> Create(nsINode* aStartContainer,
-                                          uint32_t aStartOffset,
-                                          nsINode* aEndContainer,
-                                          uint32_t aEndOffset,
-                                          ErrorResult& aRv) {
+  static already_AddRefed<nsRange> Create(
+      nsINode* aStartContainer, uint32_t aStartOffset, nsINode* aEndContainer,
+      uint32_t aEndOffset, ErrorResult& aRv,
+      AllowRangeCrossShadowBoundary aAllowCrossShadowBoundary =
+          AllowRangeCrossShadowBoundary::No) {
     return nsRange::Create(RawRangeBoundary(aStartContainer, aStartOffset),
-                           RawRangeBoundary(aEndContainer, aEndOffset), aRv);
+                           RawRangeBoundary(aEndContainer, aEndOffset), aRv,
+                           aAllowCrossShadowBoundary);
   }
   template <typename SPT, typename SRT, typename EPT, typename ERT>
   static already_AddRefed<nsRange> Create(
       const mozilla::RangeBoundaryBase<SPT, SRT>& aStartBoundary,
       const mozilla::RangeBoundaryBase<EPT, ERT>& aEndBoundary,
-      ErrorResult& aRv);
+      ErrorResult& aRv,
+      AllowRangeCrossShadowBoundary = AllowRangeCrossShadowBoundary::No);
 
   NS_DECL_ISUPPORTS_INHERITED
   NS_IMETHODIMP_(void) DeleteCycleCollectable(void) override;
@@ -158,17 +157,23 @@ class nsRange final : public mozilla::dom::AbstractRange,
    * collapsed at the end point.  Similarly, if they are in different root,
    * the range will be collapsed at the end point.
    */
-  nsresult SetStartAndEnd(nsINode* aStartContainer, uint32_t aStartOffset,
-                          nsINode* aEndContainer, uint32_t aEndOffset) {
+  nsresult SetStartAndEnd(
+      nsINode* aStartContainer, uint32_t aStartOffset, nsINode* aEndContainer,
+      uint32_t aEndOffset,
+      AllowRangeCrossShadowBoundary aAllowCrossShadowBoundary =
+          AllowRangeCrossShadowBoundary::No) {
     return SetStartAndEnd(RawRangeBoundary(aStartContainer, aStartOffset),
-                          RawRangeBoundary(aEndContainer, aEndOffset));
+                          RawRangeBoundary(aEndContainer, aEndOffset),
+                          aAllowCrossShadowBoundary);
   }
   template <typename SPT, typename SRT, typename EPT, typename ERT>
   nsresult SetStartAndEnd(
       const mozilla::RangeBoundaryBase<SPT, SRT>& aStartBoundary,
-      const mozilla::RangeBoundaryBase<EPT, ERT>& aEndBoundary) {
-    return AbstractRange::SetStartAndEndInternal(aStartBoundary, aEndBoundary,
-                                                 this);
+      const mozilla::RangeBoundaryBase<EPT, ERT>& aEndBoundary,
+      AllowRangeCrossShadowBoundary aAllowCrossShadowBoundary =
+          AllowRangeCrossShadowBoundary::No) {
+    return AbstractRange::SetStartAndEndInternal(
+        aStartBoundary, aEndBoundary, this, aAllowCrossShadowBoundary);
   }
 
   /**
@@ -216,6 +221,7 @@ class nsRange final : public mozilla::dom::AbstractRange,
       const nsAString& aString, ErrorResult& aError) const;
   MOZ_CAN_RUN_SCRIPT already_AddRefed<mozilla::dom::DocumentFragment>
   CreateContextualFragment(const mozilla::dom::TrustedHTMLOrString&,
+                           nsIPrincipal* aSubjectPrincipal,
                            ErrorResult& aError) const;
   already_AddRefed<mozilla::dom::DocumentFragment> CloneContents(
       ErrorResult& aErr);
@@ -324,16 +330,22 @@ class nsRange final : public mozilla::dom::AbstractRange,
       const mozilla::RangeBoundaryBase<EPT, ERT>& aEndBoundary,
       const nsINode* aRootNode, bool aNotInsertedYet = false);
 
+  using ElementHandler = void (*)(mozilla::dom::Element*);
   /**
    * Cut or delete the range's contents.
    *
    * @param aFragment DocumentFragment containing the nodes.
    *                  May be null to indicate the caller doesn't want a
    *                  fragment.
+   * @param aElementHandler If this handler is provided, any element node
+   *                        subtree root fully contained in this range is
+   *                        passed to it, instead of being deleted. Any
+   *                        mutation that trips nsMutationGuard is disallowed.
+   *                        Currently incompatible with non-null aFragment.
    * @param aRv The error if any.
    */
   void CutContents(mozilla::dom::DocumentFragment** aFragment,
-                   ErrorResult& aRv);
+                   ElementHandler aElementHandler, ErrorResult& aRv);
 
   static nsresult CloneParentsBetween(nsINode* aAncestor, nsINode* aNode,
                                       nsINode** aClosestAncestor,
@@ -428,7 +440,6 @@ class nsRange final : public mozilla::dom::AbstractRange,
 
   void ResetCrossShadowBoundaryRange() { mCrossShadowBoundaryRange = nullptr; }
 
-#ifdef DEBUG
   bool CrossShadowBoundaryRangeCollapsed() const {
     MOZ_ASSERT(mCrossShadowBoundaryRange);
 
@@ -438,7 +449,6 @@ class nsRange final : public mozilla::dom::AbstractRange,
             mCrossShadowBoundaryRange->StartOffset() ==
                 mCrossShadowBoundaryRange->EndOffset());
   }
-#endif
 
   /*
    * The methods marked with MayCrossShadowBoundary[..] additionally check for
@@ -468,13 +478,13 @@ class nsRange final : public mozilla::dom::AbstractRange,
   nsINode* GetMayCrossShadowBoundaryStartContainer() const {
     return mCrossShadowBoundaryRange
                ? mCrossShadowBoundaryRange->GetStartContainer()
-               : mStart.Container();
+               : mStart.GetContainer();
   }
 
   nsINode* GetMayCrossShadowBoundaryEndContainer() const {
     return mCrossShadowBoundaryRange
                ? mCrossShadowBoundaryRange->GetEndContainer()
-               : mEnd.Container();
+               : mEnd.GetContainer();
   }
 
   uint32_t MayCrossShadowBoundaryStartOffset() const {
@@ -497,6 +507,20 @@ class nsRange final : public mozilla::dom::AbstractRange,
                                      : EndRef();
   }
 
+  /**
+   * Suppress rendering of selected nodes for print selection, assuming that
+   * this range represents a part of nodes that are outside of the selection.
+   * For all non-Element nodes, this behaves identically to DeleteContents().
+   * Elements that are wholly contained by this range is instead marked with
+   * state that matches :-moz-suppress-for-print-selection.
+   * This is required to preserve matched style rules using tree-structural
+   * pseudo-classes, as well as any feature that depends on the location of
+   * the style node in the DOM tree (e.g. @scope with implicit scope).
+   *
+   * @param aRv The error, if any.
+   */
+  void SuppressContentsForPrintSelection(ErrorResult& aRv);
+
  protected:
   /**
    * DoSetRange() is called when `AbstractRange::SetStartAndEndInternal()` sets
