@@ -711,6 +711,9 @@ struct TypedArray_base : public SpiderMonkeyInterfaceObjectStorage,
     MOZ_RELEASE_ASSERT(
         !ArrayT::fromObject(mImplObj).isResizable(),
         "Bindings must have checked ArrayBuffer{View} is non-resizable");
+    MOZ_RELEASE_ASSERT(
+        !ArrayT::fromObject(mImplObj).isImmutable(),
+        "Bindings must have checked ArrayBuffer{View} is mutable");
 
     // Intentionally return a pointer and length that escape from a nogc region.
     // Private so it can only be used in very limited situations.
@@ -871,6 +874,8 @@ using Int32Array = TypedArray<JS::Int32Array>;
 using Uint32Array = TypedArray<JS::Uint32Array>;
 using Float32Array = TypedArray<JS::Float32Array>;
 using Float64Array = TypedArray<JS::Float64Array>;
+using BigInt64Array = TypedArray<JS::BigInt64Array>;
+using BigUint64Array = TypedArray<JS::BigUint64Array>;
 using ArrayBufferView = ArrayBufferView_base<JS_GetArrayBufferViewType>;
 using ArrayBuffer = TypedArray<JS::ArrayBuffer>;
 
@@ -906,46 +911,46 @@ namespace binding_detail {
 template <typename Union, typename UnionMemberType, typename = int>
 struct ApplyToTypedArray;
 
-#define APPLY_IMPL(type)                                                        \
-  template <typename Union>                                                     \
-  struct ApplyToTypedArray<Union, type, decltype((void)&Union::Is##type, 0)> {  \
-    /* Return type of calling the lambda with a TypedArray 'type'.         */   \
-    template <typename F>                                                       \
-    using FunReturnType = decltype(std::declval<F>()(std::declval<type>()));    \
-                                                                                \
-    /* Whether the return type of calling the lambda with a TypedArray     */   \
-    /* 'type' is void. */                                                       \
-    template <typename F>                                                       \
-    static constexpr bool FunReturnsVoid =                                      \
-        std::is_same_v<FunReturnType<F>, void>;                                 \
-                                                                                \
-    /* The return type of calling Apply with a union that has 'type' as    */   \
-    /* one of its union member types depends on the return type of         */   \
-    /* calling the lambda. This return type will be bool if the lambda     */   \
-    /* returns void, or it will be a Maybe<…> with the inner type being    */ \
-    /* the actual return type of calling the lambda. If the union          */   \
-    /* contains a value of the right type, then calling Apply will return  */   \
-    /* either 'true', or 'Some(…)' containing the return value of calling  */ \
-    /* the lambda. If the union does not contain a value of the right      */   \
-    /* type, then calling Apply will return either 'false', or             */   \
-    /* 'Nothing()'.                                                        */   \
-    template <typename F>                                                       \
-    using ApplyReturnType =                                                     \
-        std::conditional_t<FunReturnsVoid<F>, bool, Maybe<FunReturnType<F>>>;   \
-                                                                                \
-   public:                                                                      \
-    template <typename F>                                                       \
-    static ApplyReturnType<F> Apply(const Union& aUnion, F&& aFun) {            \
-      if (!aUnion.Is##type()) {                                                 \
-        return ApplyReturnType<F>(); /* false or Nothing() */                   \
-      }                                                                         \
-      if constexpr (FunReturnsVoid<F>) {                                        \
-        std::forward<F>(aFun)(aUnion.GetAs##type());                            \
-        return true;                                                            \
-      } else {                                                                  \
-        return Some(std::forward<F>(aFun)(aUnion.GetAs##type()));               \
-      }                                                                         \
-    }                                                                           \
+#define APPLY_IMPL(type)                                                       \
+  template <typename Union>                                                    \
+  struct ApplyToTypedArray<Union, type, decltype((void)&Union::Is##type, 0)> { \
+    /* Return type of calling the lambda with a TypedArray 'type'.         */  \
+    template <typename F>                                                      \
+    using FunReturnType = decltype(std::declval<F>()(std::declval<type>()));   \
+                                                                               \
+    /* Whether the return type of calling the lambda with a TypedArray     */  \
+    /* 'type' is void. */                                                      \
+    template <typename F>                                                      \
+    static constexpr bool FunReturnsVoid =                                     \
+        std::is_same_v<FunReturnType<F>, void>;                                \
+                                                                               \
+    /* The return type of calling Apply with a union that has 'type' as    */  \
+    /* one of its union member types depends on the return type of         */  \
+    /* calling the lambda. This return type will be bool if the lambda     */  \
+    /* returns void, or it will be a Maybe<…> with the inner type being    */  \
+    /* the actual return type of calling the lambda. If the union          */  \
+    /* contains a value of the right type, then calling Apply will return  */  \
+    /* either 'true', or 'Some(…)' containing the return value of calling  */  \
+    /* the lambda. If the union does not contain a value of the right      */  \
+    /* type, then calling Apply will return either 'false', or             */  \
+    /* 'Nothing()'.                                                        */  \
+    template <typename F>                                                      \
+    using ApplyReturnType =                                                    \
+        std::conditional_t<FunReturnsVoid<F>, bool, Maybe<FunReturnType<F>>>;  \
+                                                                               \
+   public:                                                                     \
+    template <typename F>                                                      \
+    static ApplyReturnType<F> Apply(const Union& aUnion, F&& aFun) {           \
+      if (!aUnion.Is##type()) {                                                \
+        return ApplyReturnType<F>(); /* false or Nothing() */                  \
+      }                                                                        \
+      if constexpr (FunReturnsVoid<F>) {                                       \
+        std::forward<F>(aFun)(aUnion.GetAs##type());                           \
+        return true;                                                           \
+      } else {                                                                 \
+        return Some(std::forward<F>(aFun)(aUnion.GetAs##type()));              \
+      }                                                                        \
+    }                                                                          \
   };
 
 APPLY_IMPL(Int8Array)
@@ -957,6 +962,8 @@ APPLY_IMPL(Int32Array)
 APPLY_IMPL(Uint32Array)
 APPLY_IMPL(Float32Array)
 APPLY_IMPL(Float64Array)
+APPLY_IMPL(BigInt64Array)
+APPLY_IMPL(BigUint64Array)
 APPLY_IMPL(ArrayBufferView)
 APPLY_IMPL(ArrayBuffer)
 
